//
//              VPEDEMO.CPP
//              ===========
//
// Demonstration of VPE
//
// This code compiles on all supported platforms.
//
// Copyright  1993 - 2008 IDEAL Software. All rights reserved.
//

#ifdef _WINDOWS
	#define STRICT
	#include <windows.h>
	#include <windowsx.h>
	#include <tchar.h>
	#include <io.h>
	#include <direct.h>
	#include "vpedemo.rh"
#else
	#include <unistd.h>
	#include <sys/stat.h>
	#include "VpeTypes.h"
	#define TCHAR		char
	#define _T(s)		(s)
	#define TRUE		1
	#define FALSE		0
	#define wsprintf	sprintf
#endif

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include <locale.h>

#include "colors.h"
#include "vpecomon.h"
#include "vpiface.h"


//----------------------------------------------------------------------------
// Defines:
//----------------------------------------------------------------------------

#define HELVETICA	"Helvetica"		// Windows: exchange with "Arial", if you want to use the True-Type font instead
#define TIMES		"Times"			// Windows: exchange with "Times New Roman", if you want to use the True-Type font instead

#if defined _MSC_VER && _MSC_VER >= 1400
	#pragma warning(disable:4996)	// fopen_s, etc. secure functions
#endif

// #define CREATE_FILES     // if defined, all Docs are written to file
// #define CREATE_SWAPFILES // if defined, all Docs are opened as SwapFiles
                         // if both are defined, WriteDoc() writes to other filename

#define PRODUCT "Virtual Print Engine v%d.%d Demo"


//----------------------------------------------------------------------------
// Globals:
//----------------------------------------------------------------------------
#ifdef _WINDOWS
	HWND  hMainWindow;
	HMENU hMenu;
#else
	void *hMainWindow = NULL;
#endif

static char szAppName[] = "VPE Demo";
VpeCoord LineHeight;         // used for Rendering

const char *WOOD, *SAND, *BAU, *SILENCE, *MARBLE, *PROD, *VPELOGO, *IDEALLOGO, *GEW, *FRUITS;
  
VpeHandle Welcome = 0;
VpeHandle Precision = 0;
VpeHandle PBackGnd = 0;
VpeHandle Speed = 0;
VpeHandle Colors = 0;
VpeHandle Report = 0;
VpeHandle ColorTableGrad = 0;

VpeHandle ParentBookmark = 0;	// used for creating bookmarks

int Theme = VGUI_THEME_WHIDBEY;
int Units = CM;

struct lconv Locale;


// ========================================================================
//                               Msg
// ========================================================================
void Msg(char *s)
{
#ifdef _WINDOWS
	MessageBox(hMainWindow, s, szAppName, MB_OK);
#else
	printf("%s\n", s);
#endif
}


// ========================================================================
//                               WinMsg
// For Windows only
// ========================================================================
void WinMsg(char *s)
{
#ifdef _WINDOWS
	MessageBox(hMainWindow, s, szAppName, MB_OK);
#endif
}


// =========================================================================
//								GetDefaultMetricSystem()
//
// Returns: 0 = metric; 1 = inch
// =========================================================================
int GetDefaultMetricSystem()
{
#ifdef _WINDOWS
	TCHAR s[8];

	if (GetLocaleInfo(GetUserDefaultLCID(), LOCALE_SABBREVCTRYNAME, s, sizeof(s)) == 0)
		_tcscpy(s, _T(""));
	
	if (_tcsnicmp(s, _T("us"), 2) == 0)
		return 1;
#endif

	return 0;
}


// ========================================================================
//							  GetTmpDir
// ========================================================================
TCHAR *GetTmpDir()
{
#ifdef _WINDOWS
	TCHAR *s;

	s = _tgetenv(_T("TMP"));
	if (!s)
		s = _T("c:\\tmp");

	if (_taccess(s, 2))
	{
		s = _tgetenv(_T("TEMP"));
		if (!s)
			s = _T("c:\\temp");

		if (_taccess(s, 2))
		{
			s = _tgetenv(_T("ALLUSERSPROFILE"));
			if (!s)
				s = _T("c:");
		}
	}

	return s;

#else
	char *s;
	s = getenv("HOME");
	if (!s)
		s = "/tmp";
	return s;
#endif
}


// ========================================================================
//                         ExportDocument
// ========================================================================
void ExportDocument(VpeHandle hDoc, TCHAR* FName)
{
	VpeSetSubject(hDoc, "Tutorial for VPE");
	VpeSetAuthor(hDoc, "IDEAL Software GmbH");
	VpeSetKeywords(hDoc, "Tutorial VPE");
	VpeSetCreator(hDoc, "VPE Demo");

	VpeSetEncryption(hDoc, 1);
	VpeSetProtection(hDoc, PDF_ALLOW_PRINT + PDF_ALLOW_HIQ_PRINT);
	VpeSetEmbedAllFonts(hDoc, 1);
	VpeSetSubsetAllFonts(hDoc, 1);
	VpeSetFastWebView(hDoc, 1);
	// VpeSetPDFVersion(hDoc, VPE_PDF_ACROBAT_5);

#ifdef _WINDOWS
	HWND hWnd = VpeGetWindowHandle(hDoc);
	TCHAR dir[MAX_PATH];
	wsprintf(dir, _T("%s\\%s"), GetTmpDir(), FName);

	if (MessageBox(hWnd, "Create and show PDF?", szAppName, MB_YESNO | MB_ICONQUESTION) == IDYES)
	{
		_int64 start;
		QueryPerformanceCounter((LARGE_INTEGER *)&start);

		if (VpeWriteDoc(hDoc, dir) == TRUE)
		{
			_int64 now, freq;
			QueryPerformanceCounter((LARGE_INTEGER *)&now);
			QueryPerformanceFrequency((LARGE_INTEGER *)&freq);

			char Msg[255];
			sprintf(Msg, "Elapsed Time: %f seconds", ((double)(now - start) / (double)freq));
			MessageBox(hWnd, Msg, "Message", MB_OK);

			MessageBox(hWnd, "PDF Document successfully created!", szAppName, MB_OK | MB_ICONINFORMATION);
			ShellExecute(0, NULL, dir, NULL, NULL, SW_SHOWMAXIMIZED);
		}
		else
		{
			MessageBox(hWnd, "Error creating PDF file!\n"
			"Possible reasons:\n"
			" - the file is open in Acrobat Reader\n"
			" - hard disk full\n"
			" - no access rights to the folder\n"
			" - out of memory\n", szAppName, MB_OK | MB_ICONERROR);
		}
	}
#else
	char dir[4096];
	sprintf(dir, _T("%s/%s"), GetTmpDir(), FName);

	printf("creating file %s\n", dir);
	if (!VpeWriteDoc(hDoc, dir))
	{
		printf("Error creating PDF file!\n"
			"Possible reasons:\n"
			" - the file is open in a PDF Reader\n"
			" - hard disk full\n"
			" - no access rights to the folder\n"
			" - out of memory\n");
	}
#endif
}


// ========================================================================
//                            NumberString
//
// Create a number string depending on the current country settings in the
// control panel.
//
// Expects the decimal point in the parameter string as "." !
//
// RETURNS:   The string passed as parameter is directly modified.
//
// ATTENTION: Therefore do not pass a string from the Constant Data Segment
//            to this function directly,
//            e.g. char *s = "1234";
//                 NumberString(s);    is forbidden!
// ========================================================================
void NumberString(char *s)
{
	static char cDecimal = 0;
	static char sThousand[2];

	size_t len, begin, tocopy;
	char *p, tmp[128];

	if (!s)
		return;

	strcpy(tmp, s);

	if (!cDecimal)
	{
		cDecimal = *Locale.mon_decimal_point;
		sThousand[0] = *Locale.mon_thousands_sep;
		sThousand[1] = '\0';
	}

	// On some systems the setlocale() call does not make (s)printf() format the double value as desired,
	// so replace decimal point:
	p = tmp;
	while (*p)
	{
		if (*p == '.')
		{
			*p = cDecimal;
			break;         // there is only 1 decimal point max. --> exit loop
		}
		p++;
	}

	// split fractional part off:
	// ==========================
	p = strchr(tmp, cDecimal);
	len = strlen(tmp);
	if (p)
		len -= len - (p - tmp);

	p = tmp;
	begin = TRUE;
	*s = '\0';

	// insert thousand-points:
	// ======================= 
	while (len > 0)
	{
		tocopy = len % 3;
		if (tocopy == 0)
		{
			if (!begin)
				strcat(s, sThousand);
			tocopy = 3;
		}
		strncat(s, p, tocopy);
		p += tocopy;
		len -= tocopy;
		begin = FALSE;
	}

	strcat(s, p);
}



void double2NumberString(double d, char *s)
{
	sprintf(s, "%.2f", d);
	NumberString(s);
}


// ========================================================================
//                            striplf
// ========================================================================
void striplf(char *s)
{
	size_t i;

	for (i=strlen(s)-1; i>=0 && (s[i]=='\n' || s[i]=='\r'); i--)
		s[i] = 0;
}


// ========================================================================
//                              xfgets
//
// Strips Line-Feed characters from the string
// ========================================================================
char *xfgets(char *buf, int size, FILE *fh)
{
	char *rc;

	rc = fgets(buf, size, fh);
	striplf(buf);

	return rc;
}


// ========================================================================
//                               Welcome
// ========================================================================
void welcome()
{
   WinMsg("This demo shows images in true-color, which might not look that good in other color resolutions.");
 #ifdef CREATE_SWAPFILES
   #pragma message("*** CREATE SWAP FILES ***")
   unlink("welcm_sf.vpe");
   VpeHandle hDoc = VpeOpenDocFile(hMainWindow, "welcm_sf.vpe", "Welcome", VPE_NO_HELPBUTTON + VPE_NO_INFOBUTTON);
 #else
   VpeHandle hDoc = VpeOpenDoc(hMainWindow, "Welcome", VPE_NO_HELPBUTTON + VPE_NO_INFOBUTTON /* + VPE_NO_TOOLBAR + VPE_NO_RULERS); // + VPE_NO_STATBAR */ );
 #endif

   if (!hDoc)
   {
      Msg("The Document could not be created!");
      return;
   }

   Welcome = hDoc;
#ifdef _WINDOWS
   VpeSetGUITheme(hDoc, Theme);
   VpeSetRulersMeasure(hDoc, Units);
   VpePreviewDoc(hDoc, NULL, VPE_SHOW_MAXIMIZED);
#endif

   // Set PenSize = 0 to avoid frames drawn around the images:
   // ========================================================
   ParentBookmark = VpeAddBookmark(hDoc, NULL, "Welcome");
   VpeNoPen(hDoc);

   VpePicture(hDoc, 0, 0, 19.00, 13.50, WOOD);
   VpeSetPenSize(hDoc, 0.03);
   if (VpeGetEdition() > VEDITION_COMMUNITY)	// the Community Edition can not rotate text
      VpePrint(hDoc, 0.50, 8.00, "[N '"TIMES"' S 32 I U Rot 2700]The Story");

   VpePicture(hDoc, 1.00, 1.00, VFREE, VFREE, SAND);
   VpePrint(hDoc, 1.20, 0.50, "['"TIMES"' S 12]Most of the time, I walk around like that.");
   
   VpePicture(hDoc, 10.00, 3.50, VFREE, VFREE, BAU);
   VpePrint(hDoc, 10.20, 3.00, "My tools and components look like this.");

   VpePicture(hDoc, 2.50, 7.50, VFREE, VFREE, SILENCE);
   VpeWrite(hDoc, VLEFT, VTOP, VRIGHT, VBOTTOM, " Yes, I wish I could be there...\n Everything would be so easy.");
   VpeWrite(hDoc, 11.00, 11.00, -6.50, VFREE, "[N 'Helvetica' S 10 C White]Well, we can't solve all of your problems, but in case of printing and on-screen presentations... (please turn the page)");

   VpePageBreak(hDoc);
   VpeAddBookmark(hDoc, ParentBookmark, "Welcome - The Solution");
   VpeNoPen(hDoc);
   VpePicture(hDoc, 0, 0, 19.00, 17.50, WOOD);
   VpeSetPenSize(hDoc, 0.03);
   VpeWrite(hDoc, 0, 1.00, 19.00, VFREE, "['"TIMES"' FontSize 36 Italic Underline Center]Welcome to a New Software:");
   VpePicture(hDoc, 3.50, 3.00, VFREE, VFREE, VPELOGO);

#ifdef CREATE_FILES
	#pragma message("*** CREATE FILES ***")
	#ifdef CREATE_SWAPFILES
		unlink("welcm_sw.vpe");
		VpeWriteDoc(hDoc, "welcm_sw.vpe");
	#else
		unlink("welcome.vpe");
		VpeWriteDoc(hDoc, "welcome.vpe");
	#endif
#endif

	ExportDocument(hDoc, _T("Welcome.pdf"));

#ifndef _WINDOWS
	VpeCloseDoc(hDoc);
#endif
}


//----------------------------------------------------------------------------
//                               Precision
//----------------------------------------------------------------------------
#define HEADLINE 1      // ordinal for storing a setting
#define BODYTEXT 2      // ordinal for storing a setting


void Draw3DFrame(VpeHandle hDoc, VpeCoord x, VpeCoord y, VpeCoord x2, VpeCoord y2, int gap=TRUE, int sunken=TRUE, VpeCoord pen_size=0.03)
{
   if (gap)
   {
      x -= 0.15;
      y -= 0.15;
      x2 += 0.15;
      y2 += 0.15;
   }
   
   VpeSetPen(hDoc, pen_size, PS_SOLID, sunken ? COLOR_DKGRAY : COLOR_WHITE);
   VpeLine(hDoc, x, y, x2, y);
   VpeLine(hDoc, x, y, x, y2);

   VpeSetPenColor(hDoc, sunken ? COLOR_WHITE : COLOR_DKGRAY);
   VpeLine(hDoc, x2, y, x2, y2);
   VpeLine(hDoc, x, y2, x2, y2);
}


// Page 1 of Precision demo
// ========================
void page1(VpeHandle hDoc)
{
   VpeCoord y, oldy, node_x, node_y;

// Enable the following line, if you are exporting to Adobe Acrobat 5 or higher
//   VpeSetPDFVersion(hDoc, VPE_PDF_ACROBAT_5);
   VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_BOLD + VBOOKMARK_STYLE_ITALIC + VBOOKMARK_STYLE_OPEN);	// Bold and Italic styles will work only with PDF 1.4
   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FIT, 0, 0, 0, 0, 1);
   ParentBookmark = VpeAddBookmark(hDoc, NULL, "Introduction");

   VpeSetBkgMode(hDoc, VBKG_SOLID);
   VpeSetBkgColor(hDoc, COLOR_LTGRAY);
   VpeBox(hDoc, 1.00, 1.00, 20.00, 27.00);
   VpeSetBkgColor(hDoc, COLOR_BLACK);
   VpeBox(hDoc, 1.06, 27.00, 20.06, 27.06);
   VpeBox(hDoc, 20.00, 1.06, 20.06, 27.06);
   VpeSetBkgMode(hDoc, VBKG_TRANSPARENT);

   VpePicture(hDoc, 3.00, 2.00, 18.00, -6.50, MARBLE);

   VpeWriteBox(hDoc, VLEFT, VTOP, VRIGHT, VFREE, "['"TIMES"' S 26 B U CE PS 0]Capabilities and Precision");
   VpeWriteBox(hDoc, VpeGet(hDoc, VLEFT) + 0.30, VBOTTOM, VpeGet(hDoc, VRIGHT) - 0.30, VFREE,
   "[S 18 UO IO C Black]This demo shows, what you can do with VPE - the Virtual Print Engine.\n"
   "VPE is a hammer! It solves all your printing problems, cooperates perfectly with your development tools "
   "and has a real big advantage in contrast to most available components:\n"
   "It is easy, fast and robust!");

   VpeWriteBox(hDoc, VpeGet(hDoc, VLEFT) - 0.03, VpeGet(hDoc, VTOP) - 0.03, VpeGet(hDoc, VRIGHT) - 0.03, VFREE,
   "[C White]This demo shows, what you can do with VPE - the Virtual Print Engine.\n"
   "VPE is a hammer! It solves all your printing problems, cooperates perfectly with your development tools "
   "and has a real big advantage in contrast to most available components:\n"
   "It is easy, fast and robust!");

   Draw3DFrame(hDoc, 3.00, 2.00, 18.00, 8.50, TRUE, FALSE, 0.06);

   VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_NONE);
   VpeSetBookmarkColor(hDoc, COLOR_LTRED);	// will work only with PDF 1.4
   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITR, 2.50, 1.50, 18.50, -7.00, 0);
   VpeAddBookmark(hDoc, ParentBookmark, "Capabilities and Precision");


   oldy = 9.75;
   y = VpePrint(hDoc, 4.50, oldy, "[S 12 B U L C Blue]What is the Virtual Print Engine?");

   VpeSetBookmarkColor(hDoc, COLOR_BLACK);	// will work only with PDF 1.4
   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FIT, 0, 0, 0, 0, 0);
   
   VpeStoreSet(hDoc, HEADLINE);
   y = VpeWrite(hDoc, VLEFT, y + 0.20, -12.00, VFREE,
            "[S 11 BO UO C Black]You're still watching it. "
            "VPE is a completely new way to solve your printing and presentation problems.\n"
            "It allows you the dynamic generation of absolutely free screen and printer output "
            "by calling functions during the runtime of your applications.\n"
            "The concept behind it makes it as easy, as if you were using "
            "a flat DOS screen, whilst offering you the full power of Windows.");

   VpeSetPen(hDoc, 0.03, PS_SOLID, COLOR_BLACK);
   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITH, 0, 0, 0, 0, 0);
   VpeStoreSet(hDoc, BODYTEXT);

   Draw3DFrame(hDoc, 4.50, oldy, VpeGet(hDoc, VRIGHT), y);

   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITR, 4.00, oldy - 0.50, VpeGet(hDoc, VRIGHT) + 0.50, y + 0.50, 0);
   VpeSetBookmarkColor(hDoc, COLOR_LTRED);	// will work only with PDF 1.4
   VpeAddBookmark(hDoc, ParentBookmark, "What is the Virtual Print Engine?");

   oldy = y + 2.00;
   VpeSetPen(hDoc, 0.06, PS_SOLID, COLOR_LTRED);
   VpeLine(hDoc, 10.50, y + 0.20, 10.50, y + 1.00);
   VpeLine(hDoc, 10.50, y + 1.00, 16.00, y + 1.00);
   VpeLine(hDoc, 16.00, y + 1.00, 16.00, oldy - 0.25);
   VpeLine(hDoc, 10.50, y + 1.00, 7.00, y + 1.00);
   VpeLine(hDoc, 7.00, y + 1.00, 7.00, oldy - 0.25);
   node_x = 10.50; node_y = y + 1.00;
   
   VpeUseSet(hDoc, HEADLINE);
   y = VpePrint(hDoc, 12.00, oldy, "Can I try it?");
   VpeUseSet(hDoc, BODYTEXT);
   y = VpeWrite(hDoc, VLEFT, y + 0.20, -6.91, VFREE,
            "[J]Of course! This demo contains the complete VPE-SDK, including the manuals "
            "as helpfiles, the DLL, ActiveX, VCL, .NET components and three megabytes of "
            "demo source codes for the most common programming languages / tools.");
   
   Draw3DFrame(hDoc, 12.00, oldy, VpeGet(hDoc, VRIGHT), y);
   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITR, 12.00, oldy - 0.50, VpeGet(hDoc, VRIGHT) + 0.50, y + 0.50, 0);
   VpeSetBookmarkColor(hDoc, COLOR_LTRED);	// will work only with PDF 1.4
   VpeAddBookmark(hDoc, ParentBookmark, "Can I try it?");

   VpeUseSet(hDoc, HEADLINE);
   y = VpePrint(hDoc, 2.00, oldy, "When and why should I use VPE?");
   VpeUseSet(hDoc, BODYTEXT);
   y = VpeWrite(hDoc, VLEFT, y + 0.20, -8.00, VFREE,
            "In all cases of printing and data presentation, like:\n"
            " complex dynamic documents and reports\n"
            " drawing plans, graphs, diagrams, etc.\n"
            " filling-in forms exactly and printer-independent\n"
            " high performance printing\n"
            " using images and barcodes of all common types");

   Draw3DFrame(hDoc, 2.00, oldy, VpeGet(hDoc, VRIGHT), y);
   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITR, 1.50, oldy - 0.50, VpeGet(hDoc, VRIGHT) + 0.50, y + 0.50, 0);
   VpeSetBookmarkColor(hDoc, COLOR_LTRED);	// will work only with PDF 1.4
   VpeAddBookmark(hDoc, ParentBookmark, "When and why should I use VPE?");

   oldy = y + 1.25;
   VpeSetPen(hDoc, 0.06, PS_SOLID, COLOR_LTRED);
   VpeLine(hDoc, node_x, node_y, node_x, oldy - 0.25);
   VpeUseSet(hDoc, HEADLINE);
   y = VpePrint(hDoc, 5.00, oldy, "What are the key features of VPE?");
   VpeUseSet(hDoc, BODYTEXT);
   y = VpeWrite(hDoc, VLEFT, y + 0.20, -12.00, VFREE,
            " Exact and printer independent positioning, all coordinates in cm or inches\n"
            " Supports lines, polylines, polygons, ellipses, boxes, frames, text, true-color,\n"
            "  hatching, BMP, WMF, EMF, JPEG, PNG, TIFF, GIF, PCX and\n"
            "  38 1D-barcode types plus four 2D-barcode types\n"
            " Fast vector graphics, easy to use\n"
            " Unlimited number of simultaneously open documents and pages\n"
            " Optional free scalable WYSIWYG preview\n"
            " The preview can be embedded into your own application's windows (like\n"
            "  this one), or handled automatically by VPE (like the 'Welcome' demo)\n"
            " Intelligent printer setup\n"
            " Toolbar, statusbar, rulers, measuring: everything can be modified\n"
            " Powerful dynamic layout functions\n"
            " and many more...");

   Draw3DFrame(hDoc, 5.00, oldy, VpeGet(hDoc, VRIGHT), y);
   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITR, 4.50, oldy - 0.50, VpeGet(hDoc, VRIGHT) + 0.50, y + 0.50, 0);
   VpeSetBookmarkColor(hDoc, COLOR_LTRED);	// will work only with PDF 1.4
   VpeAddBookmark(hDoc, ParentBookmark, "What are the key features of VPE?");
   VpeSetBookmarkColor(hDoc, COLOR_BLACK);	// will work only with PDF 1.4
   
#ifdef _WINDOWS
   VpeSetProgressBar(hDoc, 8);
#endif
}


// Page 2 of Precision demo
// ========================
void page2(VpeHandle hDoc)
{
   VpeHandle p;
   
   VpePageBreak(hDoc);
   VpeUseSet(hDoc, HEADLINE);
   VpePrint(hDoc, 1.00, 1.00, "[N S 16 I C Black]Overview and Tutorial");
   VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_BOLD + VBOOKMARK_STYLE_ITALIC + VBOOKMARK_STYLE_OPEN);	// Bold and Italic styles will work only with PDF 1.4
   ParentBookmark = VpeAddBookmark(hDoc, NULL, "Overview and Tutorial");
   VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_NONE);
   
   VpePrint(hDoc, 1.00, 2.00, "I. Drawing Primitives (Note: Colors and colored Text might not print with some drivers!)");
   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITH, 1.00, 2.00, 0, 0, 1);
   VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_OPEN);
   VpeAddBookmark(hDoc, ParentBookmark, "I. Drawing Primitives");

   VpeUseSet(hDoc, BODYTEXT);
   VpeLine(hDoc, 1.00, 3.00, 3.00, 3.30);
   VpePrint(hDoc, 3.50, 3.00,
            "This line was drawn with the command 'Line(1.00, 3.00, 3.00, 3.30)'.\n"
            "It means, that the starting coordinate (x, y) is 1cm from the right and 3cm from the top\n"
            "and the ending coordinate (x2, y2) is 3cm from the right and 3.3cm from the top.\n"
            "The structure of a positioning rectangle (x, y, x2, y2) is used for most of VPE's output functions.");


   VpeSetPen(hDoc, 0.03, PS_SOLID, COLOR_BLACK);
   VpeLine(hDoc, 1.00, 5.00, -2.00, -0.30);
   VpePrint(hDoc, 3.50, 5.00,
            "This line looks the same as above, but it was drawn with: Line(1.00, 5.00, -2.00, -0.30)\n"
            "The negative values mean a relative position to the starting coordinate.\n"
            "In the example the ending coordinate is 2cm to the right and 0.3cm to the top of\n"
            "the starting coordinate. This mechanism is usable for ALL drawing functions of VPE.\n\n"
            "The line thickness and color can be modified as you want. This was done with:\n"
            "'PenColor = COLOR_LTRED' and 'PenSize = 0.06' [= 0.6mm]\n"
            "(for the DLL you would enter: 'VpeSetPenColor(hDoc, COLOR_LTRED)' and 'VpeSetPenSize(hDoc, 0.06)')\n"
            "Note: you need to set the attributes, before inserting an object into the document.\n"
            "All VPE-objects use their related current attribute settings, at the moment being inserted.");
   VpeSetPenColor(hDoc, COLOR_LTRED);
   VpeSetPenSize(hDoc, 0.06);
   VpeLine(hDoc, 1.00, 7.50, 3.00, 7.50);
   
   VpeLine(hDoc, 0.80, 9.80, 2.20, 10.80);
   VpeBox(hDoc, 1.00, 10.00, 2.00, 11.00);
   VpePrint(hDoc, 3.50, 10.00,
            "The box was drawn with the command 'Box(1.00, 10.00, 2.00, 11.00)'\n"
            "As you realize, the current pencolor and pensize is automatically used.\n"
            "The inner side of the rectangle is transparent, so that drawings beneath aren't overpainted.");
   VpeSetBkgMode(hDoc, FALSE);
   VpeSetBkgColor(hDoc, COLOR_BLUE);
   VpeLine(hDoc, 0.80, 11.30, 2.20, 12.30);
   VpeBox(hDoc, 1.00, 11.50, 2.00, 12.50);
   VpeSetBkgMode(hDoc, TRUE);
   VpePrint(hDoc, 3.50, 11.50,
            "You can turn off the transparent mode and specify a background color with:\n"
            "'BkgMode = VBKG_SOLID' and 'BkgColor = COLOR_BLUE'");
   VpeLine(hDoc, 0.80, 12.80, 2.20, 13.80);
   VpeSetHatchStyle(hDoc, HS_BDIAGONAL);
   VpeBox(hDoc, 1.00, 13.00, 2.00, 14.00);
   VpeSetHatchStyle(hDoc, HS_NONE);
   VpePrint(hDoc, 3.50, 13.00, "Different hatchstyles are also possible with: 'HatchStyle = HS_BDIAGONAL'");

   VpeSetPenSize(hDoc, 0);
   VpeSetHatchStyle(hDoc, HS_FDIAGONAL);
   VpeSetHatchColor(hDoc, COLOR_LTYELLOW);
   VpeSetBkgMode(hDoc, FALSE);
   VpeBox(hDoc, 1.00, 14.50, 2.00, 15.50);
   VpeSetHatchStyle(hDoc, HS_NONE);
   VpeSetBkgMode(hDoc, TRUE);
   VpePrint(hDoc, 3.50, 14.50,
            "If you set the pensize = 0, the borders of the box aren't drawn.\n"
            "Background color and hatching can be combined. 'HatchStyle = HS_FDIAGONAL'");

   VpeSetPenSize(hDoc, 0.06);
   VpeSetHatchStyle(hDoc, HS_FDIAGONAL);
   VpeSetHatchColor(hDoc, COLOR_LTYELLOW);
   VpeSetBkgMode(hDoc, FALSE);
   VpeEllipse(hDoc, 1.00, 16.00, 2.00, 17.00);
   VpeSetHatchStyle(hDoc, HS_NONE);
   VpeSetBkgMode(hDoc, TRUE);
   VpePrint(hDoc, 3.50, 16.00,
            "This circle was drawn with: 'Ellipse(1.00, 16.00, 2.00, 17.00)'\n"
            "All styles for boxes also apply to ellipses.");
   VpeEllipse(hDoc, 1.00, 17.50, 3.00, 18.50);
   VpePrint(hDoc, 3.50, 17.50,
            "This ellipse was drawn with: 'Ellipse(1.00, 17.50, 3.00, 18.50)'");

   VpeSetPen(hDoc, 0.03, PS_SOLID, COLOR_BLUE);
   p = VpePolyLine(hDoc, 4);
   VpeAddPolyPoint(hDoc, p, 1.00, 19.50);
   VpeAddPolyPoint(hDoc, p, 3.00, 20.00);
   VpeAddPolyPoint(hDoc, p,  0.70,  19.75);
   VpeAddPolyPoint(hDoc, p, 1.00, 20.50);
   VpeCoord y = VpePrint(hDoc, 3.50, 19.50,
            "If you want to draw a lot of lines, use polylines to save memory and increase performance.\n"
            "long p = PolyLine(4)   [ reserve space for 4 points ]\n");
   VpePrint(hDoc, 3.50, y,
             "AddPolyPoint(p, 1.00, 19.50)\n"
             "AddPolyPoint(p, 0.70, 19.75)");

   VpePrint(hDoc, 9.00, y,
            "AddPolyPoint(p, 3.00, 20.00)\n"
            "AddPolyPoint(p, 1.00, 20.50)");

   VpeSetPenSize(hDoc, 0.06);
   p = VpePolyLine(hDoc, 6);
   VpeAddPolyPoint(hDoc, p, 1.00, 22.00);
   VpeAddPolyPoint(hDoc, p, 3.00, 22.50);
   VpeAddPolyPoint(hDoc, p,  0.70, 22.25);
   VpeAddPolyPoint(hDoc, p,  -1,   -1);
   VpeAddPolyPoint(hDoc, p, 1.00, 23.00);
   VpeAddPolyPoint(hDoc, p, 2.50, 22.00);
   y = VpePrint(hDoc, 3.50, 22.00,
            "All penstyles also apply to polylines.\n"
            "You can start a new line at a completely new position by inserting a -1, -1 pair, e.g.:\n"
            "p = PolyLine(6)   [ reserve space for 6 points ]\n");
   VpePrint(hDoc, 3.50, y,
            "AddPolyPoint(p, 1.00, 22.00)\n"
            "AddPolyPoint(p, 0.70, 22.25)\n"
            "AddPolyPoint(p, 1.00, 23.00)");

   VpePrint(hDoc, 9.00, y,
            "AddPolyPoint(p, 3.00, 22.50)\n"
            "AddPolyPoint(p, -1, -1)\n"
            "AddPolyPoint(p, 2.50, 22.00)");

   VpeSetHatchStyle(hDoc, HS_FDIAGONAL);
   VpeSetHatchColor(hDoc, COLOR_LTYELLOW);
   VpeSetBkgMode(hDoc, FALSE);
   VpeSetBkgColor(hDoc, COLOR_LTRED);
   p = VpePolygon(hDoc, 4);
   VpeAddPolygonPoint(hDoc, p, 2.00, 25.00);
   VpeAddPolygonPoint(hDoc, p, 3.00, 25.70);
   VpeAddPolygonPoint(hDoc, p, 2.00, 26.00);
   VpeAddPolygonPoint(hDoc, p, 1.00, 27.00);
   VpeSetHatchStyle(hDoc, HS_NONE);
   VpeSetBkgMode(hDoc, TRUE);
   y = VpePrint(hDoc, 3.50, 25.00,
            "All styles for boxes also apply to polygons.\n"
            "The closing line from the last polygon point to the starting point is drawn automatically.\n"
            "p = Polygon(4)   [ reserve space for 4 points ]\n");

   VpePrint(hDoc, 3.50, y,
            "AddPolygonPoint(p, 2.00, 25.00)\n"
            "AddPolygonPoint(p, 2.00, 26.00)");

   VpePrint(hDoc, 9.00, y,
            "AddPolygonPoint(p, 3.00, 25.70)\n"
            "AddPolygonPoint(p, 1.00, 27.00)");

#ifdef _WINDOWS
   VpeSetProgressBar(hDoc, 16);
#endif
}


// Page 3 and 4 of Precision demo
// ==============================
void page34(VpeHandle hDoc)
{
   VpePageBreak(hDoc);
   VpeUseSet(hDoc, HEADLINE);
   VpePrint(hDoc, 1.00, 1.00, "II. Text Output");

   VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_OPEN);
   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITH, 1.00, 1.00, 0, 0, 1);
   VpeHandle bookmark = VpeAddBookmark(hDoc, ParentBookmark, "II. Text Output");

   VpeUseSet(hDoc, BODYTEXT);
   VpeWrite(hDoc, 1.00, 2.00, 3.00, 2.60, "Hi folks!");
   VpePrint(hDoc, 3.50, 2.00,
            "Text is drawn as easy as a line. This text was inserted with:\n"
            "'Write(1.00, 2.00, 3.00, 2.60, \"Hi folks!\")'");

   VpeSetPenColor(hDoc, COLOR_BLUE);
   VpeWriteBox(hDoc, 1.00, 3.00, 3.00, 3.60, "Hi folks!");
   VpePrint(hDoc, 3.50, 3.00,
            "If you use 'WriteBox' instead of 'Write', a box is automatically drawn around the text.\n"
            "'WriteBox(1.00, 3.00, 3.00, 3.60, \"Hi folks!\")'");

   VpeSetBkgMode(hDoc, FALSE);
   VpeSetBkgColor(hDoc, COLOR_LTRED);
   VpeSetTextColor(hDoc, COLOR_WHITE);
   VpeWriteBox(hDoc, 1.00, 4.00, 3.00, 4.60, "Hi folks!");
   VpeSetBkgMode(hDoc, TRUE);
   VpeSetTextColor(hDoc, COLOR_BLACK);
   VpePrint(hDoc, 3.50, 4.00,
            "Of course, all current styles and settings for the box are used.\n"
            "(pen size, pen color, background mode, background color, etc.)");

   VpePrint(hDoc, 1.00, 5.50,
            "Well, the above is very static. You always supply the rectangle (x, y, x2, y2), "
            "but if you use a different font size or different text lengths, how can "
            "you know about the size of the rectangle, so that the text will fit into it?\n"
            "You don't. VPE does it for you!\n"
            "There is a special constant called VFREE, you can use for y2. "
            "This means, you specify the starting coordinate (x, y) and the right margin (x2) "
            "and VPE computes the bottom coordinate y2 itself. Look at the following example:");
            
   VpeSetPenColor(hDoc, COLOR_BLACK);
   VpeWriteBox(hDoc, 1.00, 8.00, 5.00, VFREE,
               "Hi folks! I have been in cinema yesterday. The film was very good.");
   VpePrint(hDoc, 6.00, 8.00,
            "This output was created with the statement:\n"
            "'WriteBox(1.00, 8.00, 5.00, VFREE, \"Hi folks! I have been in cinema yesterday. The film was very good.\")'");

   VpePrint(hDoc, 1.00, 10.00,
            "VPE computed the bottom of the rectangle (y2) itself, using the current font size "
            "and text length.\n"
            "Note, that VPE does also the automatic word break. Now the same command, "
            "but with another font size:");
   VpeSetFont(hDoc, HELVETICA, 14);
   VpeWriteBox(hDoc, 1.00, 11.50, 5.00, VFREE,
               "Hi folks! I have been in cinema yesterday. The film was very good.");
   VpeSetFont(hDoc, HELVETICA, 10);
   VpePrint(hDoc, 6.00, 11.50,
            "The font (Arial, 14pt) was selected with: 'SelectFont(\"Arial\", 14)'\n"
            "Note, that the rectangle is much higher than before.");

   VpePrint(hDoc, 1.00, 15.00,
            "You can specify the text alignment by setting the property 'Alignment' to one of "
            "the predefined constants ALIGN_LEFT, ALIGN_RIGHT, ALIGN_CENTER and ALIGN_JUSTIFIED "
            "(e.g. 'Alignment = ALIGN_RIGHT'):");

   VpeWriteBox(hDoc, 1.00, 16.00, -4.00, VFREE,
               "Hi folks! I have been in cinema yesterday. The film was very good. (left aligned)");
   VpeSetTextAlignment(hDoc, ALIGN_RIGHT);
   VpeWriteBox(hDoc, 6.00, 16.00, -4.00, VFREE,
               "Hi folks! I have been in cinema yesterday. The film was very good. (right aligned)");
   VpeSetTextAlignment(hDoc, ALIGN_CENTER);
   VpeWriteBox(hDoc, 11.00, 16.00, -4.00, VFREE,
               "Hi folks! I have been in cinema yesterday. The film was very good. (centered)");
   VpeSetTextAlignment(hDoc, ALIGN_JUSTIFIED);
   VpeWriteBox(hDoc, 16.00, 16.00, -4.00, VFREE,
               "Hi folks! I have been in cinema yesterday. The film was very good. (justified)");

   VpePrint(hDoc, 1.00, 18.50, "The same without the frames, but some attributes:");
   VpeSetTextAlignment(hDoc, ALIGN_LEFT);
   VpeSetBold(hDoc, TRUE);
   VpeWrite(hDoc, 1.00, 19.00, -4.00, VFREE,
               "Hi folks! I have been in cinema yesterday. The film was very good. (left aligned, bold)");
   VpeSetTextAlignment(hDoc, ALIGN_RIGHT);
   VpeSetBold(hDoc, FALSE);
   VpeSetItalic(hDoc, TRUE);
   VpeWrite(hDoc, 6.00, 19.00, -4.00, VFREE,
               "Hi folks! I have been in cinema yesterday. The film was very good. (right aligned, italic)");
   VpeSetTextAlignment(hDoc, ALIGN_CENTER);
   VpeSetItalic(hDoc, FALSE);
   VpeSetUnderlined(hDoc, TRUE);
   VpeWrite(hDoc, 11.00, 19.00, -4.00, VFREE,
               "Hi folks! I have been in cinema yesterday. The film was very good. (centered, underlined)");
   VpeSetTextAlignment(hDoc, ALIGN_JUSTIFIED);
   VpeSetBold(hDoc, TRUE);
   VpeSetItalic(hDoc, TRUE);
   VpeWrite(hDoc, 16.00, 19.00, -4.00, VFREE,
               "Hi folks! I have been in cinema yesterday. The film was very good. (justified, all attributes)");

   VpeUseSet(hDoc, BODYTEXT);
   VpePrint(hDoc, 1.00, 21.00,
            "To set the different attributes, VPE offers 3 boolean properties: "
            "TextBold, TextItalic and TextUnderlined\n"
            "Example: with 'TextBold = TRUE' you activate the bold-attribute, and with 'TextBold = FALSE' you deactivate it.");

   VpePrint(hDoc, 1.00, 22.50, "To modify the text color, just use the property 'TextColor':");
   VpeSetTextColor(hDoc, COLOR_BLUE);
   VpePrint(hDoc, 1.00, 23.00,
            "This text color was set by the statement: 'TextColor = COLOR_BLUE'.");

   VpeSetTextColor(hDoc, COLOR_BLACK);
   VpePrint(hDoc, 1.00, 24.00,
            "If text doesn't fit onto the current page, VPE is able to break it "
            "automatically over multiple pages. "
            "The behaviour of VPE is controlled by the property 'AutoBreakMode'. "
            "By default it is activated, and you can see it working in the following text:");

   VpeWriteBox(hDoc, 5.00, 26.00, 15.00, VFREE,
               "Hi folks! I have been in cinema yesterday. The film was very good. "
               "Also I had been eating popcorn there, drank a coke and felt very fine. "
               "I don't want to tell you, in what film I've been, because it might "
               "lead into trouble, making advertising here for films.");
#ifdef _WINDOWS
   VpeSetProgressBar(hDoc, 25);
#endif

   VpePrint(hDoc, 1.00, 3.00,
   "The command for the output of the box above - with its beginning at the "
   "bottom of the previous page - was:\n\n"
   "WriteBox(5.00, 26.00, 15.00, VFREE,");
   VpePrint(hDoc, 2.50, VBOTTOM,
   "\"Hi folks! I have been in cinema yesterday. The film was very good. \"\n"
   "\"Also I had been eating popcorn there, drank a coke and felt very fine. \"\n"
   "\"I don't want to tell you, in what film I've been, because it might \"\n"
   "\"lead into trouble, making advertising here for films.\")\n\n");

   VpePrint(hDoc, 1.00, VBOTTOM,
   "As the text reached the bottom of the page, VPE automatically did add a new page and "
   "inserted the rest of the text (including the surrounding box) on the new one."
   "This is called AUTOMATIC TEXT BREAK. "
   "Note, that VPE is able to do this auto break over multiple pages "
   "with just one output command. See the 'Auto Page Break' demo (see in the menu of this application), which generates the whole listing "
   "by just ONE single call to 'WriteBox()'.\n"
   "By the way: you can generate 'manually' page breaks (e.g. add new blank pages to the "
   "current end of the document) by calling the method 'PageBreak'.");

   VpeUseSet(hDoc, HEADLINE);
   VpePrint(hDoc, 1.00, 10.40, "II.1 Margins");
   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITH, 0, 10.40, 0, 0, 0);
   VpeAddBookmark(hDoc, bookmark, "II.1 Margins");

   VpeUseSet(hDoc, BODYTEXT);
   VpePrint(hDoc, 1.00, 11.00,
   "The margins you define in VPE are virtual margins. That means, you can place text "
   "and other objects outside of them. "
   "But you can access (read / write) the margins with the properties nLeftMargin, nTopMargin, "
   "nRightMargin and nBottomMargin. So they can work as placeholders for you, containing "
   "the current document definitions. If you need to change the margins later during the "
   "development process, and your calls to VPE are considering them, you will "
   "have no problems with the new layout.\n\n"
   "Nevertheless the margins play an important role for the behaviour of some methods:\n"
   " The bottom margin tells VPE, where to start with the Automatic Text Break, but you can "
   "change this by\n"
   "  modifying the property 'AutoBreakMode' (see documentation).\n"
   " The top margin tells VPE, where to start with automatic broken text on successive pages.\n"
   " The right margin is important for the method 'Print', which will be explained below.\n\n");


   VpeUseSet(hDoc, HEADLINE);
   VpePrint(hDoc, 1.00, 16.70, "II.2 More About Text Output");
   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITH, 0, 16.70, 0, 0, 0);
   VpeAddBookmark(hDoc, bookmark, "II.2 More About Text Output");

   VpeUseSet(hDoc, BODYTEXT);
   VpePrint(hDoc, 1.00, 17.30,
            "Until now, you've only used the methods 'Write' and 'WriteBox'.\nBut there are "
            "two other methods: 'Print' and 'PrintBox'.\n"
            "As you notice, they have no right margin (x2 is missing) and no bottom "
            "margin (y2 is missing).\n"
            "Both methods are intended for 'quick usage', you don't need to care about "
            "the border coordinates, because VPE does.\n\n"
            "'Print' outputs text until the whole text is processed, or the right page "
            "margin is reached. "
            "Then the remaining text is broken to the next line, starting "
            "again at the x-coordinate. "
            "This is done until the bottom margin of the page is reached. "
            "Depending on the setting for 'AutoBreakMode', text is then broken onto "
            "succeeding pages (or not). "
            "Note, that - regardless of the settings for the text alignment - the method "
            "'Print' always outputs text left aligned (otherwise the function wouldn't "
            "make any sense).\n\n"
            "Examples:");

   VpePrintBox(hDoc, 5.00, 22.50,
               "Hi folks! I have been in cinema yesterday. The film was very good.");
   VpePrint(hDoc, 1.00, 23.30,
               "command: 'PrintBox(5.00, 22.50, \"Hi folks! I have been in cinema yesterday. The film was very good.\")'");

   VpePrintBox(hDoc, 12.00, 24.00,
               "Hi folks! I have been in cinema yesterday. The film was very good.");

   VpePrint(hDoc, 1.00, 25.20,
               "command: 'PrintBox(12.00, 24.00, \"Hi folks! I have been in cinema yesterday. The film was very good.\")'\n\n"
               "The first example did the output in a single line. In the second example you can see, "
               "that VPE did break the text into two lines, because it reached the right margin.");

#ifdef _WINDOWS
   VpeSetProgressBar(hDoc, 33);
#endif
}


// Page 5 of Precision demo
// ========================
void page5(VpeHandle hDoc)
{
   VpePageBreak(hDoc);
   VpeUseSet(hDoc, HEADLINE);
   VpePrint(hDoc, 1.00, 1.00, "III. Images");
   VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_OPEN);
   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITH, 1.00, 1.00, 0, 0, 1);
   VpeAddBookmark(hDoc, ParentBookmark, "III. Images");

   VpeUseSet(hDoc, BODYTEXT);
   VpePrint(hDoc, 1.00, 2.00, "[N U]VPE imports the following graphics file formats:");
   VpePrint(hDoc, 1.50, VBOTTOM,
       " Windows and OS/2 Bitmaps (BMP: 2 / 16 / 256 / True Color)\n"
       " Windows Placeable Metafiles (WMF) and Enhanced Metafiles (EMF - 32-bit version only)\n"
       " PNG (the new Portable Network Graphics format!)\n"
       " JPG (256 / True Color)\n"
       " TIFF 5.0 (2 / 16 / 256 / True Color, LZW / PackBits / Fax G3 & G4, Multipage)\n"
       " GIF (2 / 16 / 256 Colors, Multipage)\n"
       " PCX (2 / 16 / 256 / True Color)\n"
       " plus ICO, JNG, KOA, IFF/LBM, MNG, PBM, PCD, PGM, PPM, RAS, TGA,\n"
       "  WAP/WBMP/WBM, PSD, CUT, XBM, XPM, DDS, HDR, G3, SGI"
   );
       
   VpePrint(hDoc, 1.00, 6.90, //VpeGet(hDoc, VBOTTOM) + 80,
       "[N B]Images are easily inserted with the method 'Picture'.");

   VpePicture(hDoc, 1.00, 7.50, VFREE, VFREE, IDEALLOGO);
   VpePrint(hDoc, 6.00, 7.50,
      "Picture(1.00, 7.50, VFREE, VFREE, \"..\\Images\\logo.bmp\")\n\n"
      "This is all you need to enter. The first 4 parameters are the position rectangle. Note, that x2 and y2 are VFREE, which "
      "means, that VPE computes the size of the image by retrieving the DPI resolution "
      "from the image and rendering it to its device independent metric coordinate system.");

   VpePicture(hDoc, 1.00, 13.00, VFREE, VFREE, SAND);
   VpePrint(hDoc, 9.00, 13.00,
      "Don't care for the filetypes of the images. VPE automatically determines the right "
      "filetype by examining the suffix of the filename (e.g. .BMP / .TIF / .JPG / ...)\n\n"
      "This image is a JPEG file and was created with:\n\n"
      "Picture(1.00, 13.00, VFREE, VFREE, \"sand.jpg\")\n\n");

   VpePrint(hDoc, 1.00, 19.00,
      "[N B]Of course you can scale images (undistorted and distorted):");
   
   VpePicture(hDoc, 1.50, 20.00, 2.00, VFREE, IDEALLOGO);
   VpePicture(hDoc, VpeGet(hDoc, VRIGHT) + 1.00, 20.00, VpeGet(hDoc, VRIGHT) + 2.50, VFREE, IDEALLOGO);
   VpePicture(hDoc, VpeGet(hDoc, VRIGHT) + 1.00, 20.00, VpeGet(hDoc, VRIGHT) + 7.50, VFREE, IDEALLOGO);
   VpePicture(hDoc, VpeGet(hDoc, VRIGHT) + 1.00, 20.00, VRIGHTMARGIN, -3.00, IDEALLOGO);

#ifdef _WINDOWS
   VpeSetProgressBar(hDoc, 41);
#endif
}


// Page 6 of Precision demo
// ========================
void page6(VpeHandle hDoc)
{
   VpeCoord y;
   
   VpePageBreak(hDoc);
   VpeUseSet(hDoc, HEADLINE);
   VpePrint(hDoc, 1.00, 1.00, "IV. Dynamic Layout");
   VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_OPEN);
   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITH, 1.00, 1.00, 0, 0, 1);
   VpeAddBookmark(hDoc, ParentBookmark, "IV. Dynamic Layout");

   VpeUseSet(hDoc, BODYTEXT);
   VpePrint(hDoc, 1.00, 2.00,
      "So far you have learned about the different graphical objects offered by VPE, and "
      "the basic techniques of using them. "
      "Now it is time, to introduce one of the most powerful concepts: the Dynamic Layout.\n\n"
      "You have seen, that objects can be placed static (e.g. with absolute coordinates) and that "
      "text and images can be rendered. Also you will expirience during a development cycle, "
      "that layouts change, because even static placed objects need to be moved or resized and "
      "that dependend objects need to be moved also.\n\n"
      "VPE offers some constants and properties to gain precise control over this task. "
      "First of all you have the four properties nLeftMargin, nRightMargin, nTopMargin and nBottomMargin. "
      "They contain the current margin settings, so you can place objects relative to them, "
      "which is very useful, if the margins change later during the development process.");
      
   VpePrint(hDoc, VLEFTMARGIN, 7.20,
      "For example this output was created with: Print(nLeftMargin, 7.00, \"text\")");
   VpePrint(hDoc, VpeGet(hDoc, VLEFTMARGIN) + 1.00, 7.90,
      "This output was created with: Print(nLeftMargin + 1.00, 7.70, \"text\")");
   VpeWriteBox(hDoc, 10.00, 8.60, VpeGet(hDoc, VRIGHTMARGIN), VFREE, 
      "And this output was created with: WriteBox(10.00, 8.50, nRightMargin, VFREE, \"text\")");

   VpePrint(hDoc, 1.00, 10.50,
      "This is easy, and the next thing you are going to learn is as easy. "
      "VPE keeps track of the last inserted object, and stores its coordinates in the four "
      "properties nLeft, nRight, nTop and nBottom.\n\n"
      "If you insert for example a text object with y2=VFREE, so that you don't know its bottom "
      "coordinate, but you want to place another object attached to this bottom coordinate, you do "
      "the following:");

   VpePrintBox(hDoc, 1.00, 13.00, "This is text A");
   y = VpePrintBox(hDoc, 1.00, VBOTTOM, "This is text B");

   VpePrint(hDoc, 4.50, 13.00,
      "PrintBox(1.00, 13.00, \"This is text A\")\n"
      "PrintBox(1.00, nBottom, \"This is text B\")\n");

   VpePrint(hDoc, 1.00, y + 0.20,
      "The example means nothing else, than that the starting y-coordinate of text B is "
      "the bottom coordinate of text A.\n\n"
      "In the next example the same thing, but with some offsets:");

   VpePrintBox(hDoc, 1.00, 16.00, "This is text A");
   y = VpePrintBox(hDoc, VpeGet(hDoc, VLEFT) + 0.50, VpeGet(hDoc, VBOTTOM) + 0.50, "This is text B");

   VpePrint(hDoc, 4.50, 16.00,
      "PrintBox(1.00, 16.00, \"This is text A\")\n"
      "PrintBox(nLeft + 0.50, nBottom + 0.50, \"This is text B\")\n");

   y = VpePrint(hDoc, 1.00, y + 0.20,
      "Imagine, what you can do with this powerful concept! You don't need to care about the "
      "size or position of objects, when placing others depended on them. "
      "Even if your source code is a big project with hundreds of nested functions, which might "
      "change dynamically the layout depending on the data they process, you nowhere need to "
      "care about anything, just use the n-properties!\n\n");

   VpePrint(hDoc, 1.00, y + 0.30,
      "[N B]VPE can also compute the dimensions of text, RTF and image objects in advance without inserting "
      "them into a document. See 'Rendering Objects' in the help file titled 'Programmer's Manual'.");

   VpePrint(hDoc, 1.00, 22.00,
      "Note: The examples are 100% correct, but the syntax was simplified.\n\n"
      "For the OCX and VCL you access methods and properties with the syntax: <object>.method or "
      "<object>.property.\n"
      "So if you had named your OCX / VCL 'Doc', you would for example need to enter:\n"
      "     Doc.PrintBox(Doc.nLeft + 0.50, Doc.nBottom + 0.50, \"This is text B\")\n\n"
      "Since the DLL cannot offer objects or properties, you create handles for each document "
      "by calling 'VpeOpenDoc'. After retrieving a handle to a document, each function of the DLL "
      "expects this handle as parameter. To read or write the n-properties, the DLL offers the functions "
      "'VpeGet' and 'VpeSet'. So for the DLL the example would be:\n"
      "     VpePrintBox(hDoc, VpeGet(hDoc, VLEFT) + 0.50, VpeGet(hDoc, VBOTTOM) + 0.50, \"This is text B\")");

#ifdef _WINDOWS
   VpeSetProgressBar(hDoc, 50);
#endif
}


// Page 7 of Precision demo
// ========================
void page7(VpeHandle hDoc)
{
   VpeCoord y;
   
   VpePageBreak(hDoc);
   VpeUseSet(hDoc, HEADLINE);
   y = VpeWrite(hDoc, 0, 1.00, 21.00, VFREE, "[C Black BO CE I S 36]Congratulations!");
   
   VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_ITALIC + VBOOKMARK_STYLE_BOLD + VBOOKMARK_STYLE_OPEN);	// Bold and Italic styles will work only with PDF 1.4
   VpeHandle DemoListBmk = VpeAddBookmark(hDoc, NULL, "Demo List");

   VpeSetFont(hDoc, HELVETICA, 14);
   VpeStoreSet(hDoc, HEADLINE);
   VpeUseSet(hDoc, BODYTEXT);
   y = VpePrint(hDoc, 2.00, y + 0.50,
      "You have learned how to use 90% of VPE's powerful capabilities by just studying "
      "a five-page tutorial! "
      "If you want to learn more about VPE, we recommend to read the chapter "
      "'Programming Techniques' in the manual / helpfile named 'Programmer's Manual'.\n\n"
      "Please note, that all documents in this demo are layouted to fit as well onto "
      "DIN A4, as onto US-Letter format.\n\n"
      "Now, after so much theory, let's see what we can do with it. The following pages "
      "will show some good examples about the precision and capabilities of VPE. "
      "There are also some other demos in the main menu, which show other features. "
      "A short description of each demo follows now.\n\n\n");

   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITH, 2.00, y, 0, 0, 0);
   VpeAddBookmark(hDoc, DemoListBmk, "Welcome");

   y = VpePrint(hDoc, 2.00, y, "[N 'Helvetica' B]Welcome");
   y = VpePrint(hDoc, 2.00, y,
       "An introduction. Note, that the images are in true-color, they don't look very nice in 16 or 256 color video mode (this applies also to the \"Capabilities + Precision\" demo).\n"
       "\n"
       "\n");

   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITH, 2.00, y, 0, 0, 0);
   VpeAddBookmark(hDoc, DemoListBmk, "Capabilities + Precision");
   y = VpePrint(hDoc, 2.00, y, "[N 'Helvetica' B]Capabilities + Precision");
   y = VpePrint(hDoc, 2.00, y,
       "This demo shows text formatting features, drawing features, bitmap handling, form filling and of course printing. "
       "Important: the VPE-DLL  \"docks\" its view inside of the window owned by vpedemo.exe!  This is very easily done by a few lines of code! "
       "The menu-entry \"Background\" shows how to print without showing a preview and no setup-dialog (default printer is used). "
       "The preview window sends the VPE_HELP message to the calling application instead of showing the standard help dialog, so you see the message box \"User requested help\" generated by vpedemo.exe on the screen.\n"
       "\n"
       "\n");

   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITH, 2.00, y, 0, 0, 0);
   VpeAddBookmark(hDoc, DemoListBmk, "Speed + Tables");
   y = VpePrint(hDoc, 2.00, y, "[N 'Helvetica' B]Speed + Tables");
   y = VpePrint(hDoc, 2.00, y,
       "Here you can see how fast VPE builds a report with a size of about 110-130 pages:\n"
       "A text file with random data is generated (journal.rpt). vpedemo.exe reads the text file line by line, interpreting it and instructing VPE how to build the report.\n"
       "Since it is random data, the number of pages differs from 110-130 pages.  Note, that this demo prints the number of generated pages finally on the FIRST page of the report in the upper left corner.  This is done by the virtual processing of the document, where you can move to any page at any time to draw on it.  In this case the demo generates all pages and then jumps to the first page to draw the message.\n"
       "\n"
       "\n");

   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITH, 2.00, y, 0, 0, 0);
   VpeAddBookmark(hDoc, DemoListBmk, "Colors");
   y = VpePrint(hDoc, 2.00, y, "[N 'Helvetica' B]Colors");
   y = VpePrint(hDoc, 2.00, y,
       "There you can see a fixed scaled window.  Also, the toolbar has only the print and the e-mail button, and the status bar is hidden.  The user cannot close the document; it can only be closed through vpedemo.exe by selecting the \"Close Preview\" menu entry. If you print the page to a color printer, you will get a true-color result.\n"
       "\n"
       "\n");

   VpeSetBookmarkDestination(hDoc, VBOOKMARK_DEST_FITH, 2.00, y, 0, 0, 0);
   VpeAddBookmark(hDoc, DemoListBmk, "Report");
   y = VpePrint(hDoc, 2.00, y, "[N 'Helvetica' B]Report");
   y = VpePrint(hDoc, 2.00, y,
       "This is another report, showing various colors and a pie chart on the second page. The source code shows very fine, how easy creating reports is by encapsulating the different parts of the report into functions.\n"
       "\n"
       "\n");

#ifdef _WINDOWS
   VpeSetProgressBar(hDoc, 58);
#endif
}


// Page 8 of Precision demo
// ========================
void page8(VpeHandle hDoc)
{
   VpeCoord y, oldy;
   
   VpePageBreak(hDoc);
   VpeUseSet(hDoc, HEADLINE);
   VpePrint(hDoc, 2.00, 2.00, "An example of drawing (better to turn the grid off here):");
   
   VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_ITALIC + VBOOKMARK_STYLE_BOLD + VBOOKMARK_STYLE_OPEN);	// Bold and Italic styles will work only with PDF 1.4
   ParentBookmark = VpeAddBookmark(hDoc, NULL, "Examples");

   VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_OPEN);
   VpeAddBookmark(hDoc, ParentBookmark, "Drawing Speed");

   VpeSetPen(hDoc, 0.08, PS_SOLID, COLOR_BLACK);
   VpeBox(hDoc, 2.00, 3.00, 17.00, 18.00);
   VpeSetPen(hDoc, 0.03, PS_SOLID, COLOR_BLACK);

   double xr = 6, xstep = xr / 750;
   double yr = 18;
   double x;
   char s[160];
   int skip, first, xx, segments=0;

   // The following graph is created with VpeAddPolyPoint().
   // Note, that all routines contain complex handling of clipping.
   x = -xr;
   skip = 0;
   VpeSetPen(hDoc, 0.02, PS_SOLID, COLOR_BLUE);
   VpeHandle p = VpePolyLine(hDoc, 1500);
        
   for (xx = 200; xx < 1700; xx++)
   {
      y = 1050 - (pow(x, 3) - 2*pow(x, 2) - 8*x) / ( yr / 750);
      x += xstep;
      
      if (y < 300)
      {
         skip = 1;
         continue;
      }
      else if (y > 1800)
      {
         skip = 1;
         continue;
      }
      
      if (skip == 1)
      {
         VpeAddPolyPoint(hDoc, p, -0.01, -0.01);
         oldy = y;
         skip = 2;
      }
      else
      {
         if (skip == 2)
         {
            VpeAddPolyPoint(hDoc, p, (xx-1)/100.00, oldy/100.00);
            skip = 0;
         }
         VpeAddPolyPoint(hDoc, p, xx/100.00, y/100.00);
         segments++;
      }
   }

   // The following graph is created with VpeAddPolyPoint().
   x = -xr;
   first = 1;
   VpeSetPen(hDoc, 0.02, PS_SOLID, COLOR_LTRED);
   p = VpePolyLine(hDoc, 1500);

   for (xx = 200; xx < 1700; xx++)
   {
      y = 1050 - (3*pow(x, 2) - 4*x - 8) / ( yr / 750);
      x += xstep;
      
      if (y < 300)
      {
         skip = 1;
         continue;
      }
      else if (y > 1800)
      {
         skip = 1;
         continue;
      }
      
      if (skip == 1)
      {
         VpeAddPolyPoint(hDoc, p, -1, -1);
         oldy = y;
         skip = 2;
      }
      else
      {
         if (skip == 2)
         {
            VpeAddPolyPoint(hDoc, p, (xx-1)/100.00, oldy/100.00);
            skip = 0;
         }
         VpeAddPolyPoint(hDoc, p, xx/100.00, y/100.00);
         segments++;
      }
   }

   // The following graph is created with VpeAddPolyPoint().
   x = -xr;
   first = 1;
   VpeSetPen(hDoc, 0.02, PS_SOLID, COLOR_GREEN);
   p = VpePolyLine(hDoc, 1500);
   for (xx = 200; xx < 1700; xx++)
   {
      y = 1050 - (3*x - 04) / ( yr / 750);
      x += xstep;
      
      if (y < 300)
      {
         skip = 1;
         continue;
      }
      else if (y > 1800)
      {
         skip = 1;
         continue;
      }
      
      if (skip == 1)
      {
         VpeAddPolyPoint(hDoc, p, -1, -1);
         oldy = y;
         skip = 2;
      }
      else
      {
         if (skip == 2)
         {
            VpeAddPolyPoint(hDoc, p, (xx-1)/100.00, oldy/100.00);
            skip = 0;
         }
         VpeAddPolyPoint(hDoc, p, xx/100.00, y/100.00);
         segments++;
      }
   }

   VpeSetPen(hDoc, 0.03, PS_SOLID, COLOR_BLACK);
   VpeLine(hDoc, 2.00, 10.50, 17.00, 10.50);
   VpeLine(hDoc, 9.50, 3.00, 9.50, 18.00);

   VpeSetFont(hDoc, HELVETICA, 10);
   VpeSetFontAttr(hDoc, ALIGN_LEFT, 0, 0, 0, 0);
   VpeSetPen(hDoc, 0.01, PS_DOT, COLOR_BLACK);
   for (xx = 1; xx < xr; xx++)
   {
      VpeLine(hDoc, 9.50 + xx * 7.50 / 6, 3.00, 9.50 + xx * 7.50 / 6, 18.00);
      wsprintf(s, "%d", (int)xx);
      VpePrint(hDoc, 9.60 + xx * 7.50 / 6, 10.50, s);
      VpeLine(hDoc, 9.50 - xx * 7.50 / 6, 3.00, 9.50 - xx * 7.50 / 6, 18.00);
      wsprintf(s, "%d", (int)-xx);
      VpePrint(hDoc, 9.60 - xx * 7.50 / 6, 10.50, s);
   }

   for (y = 2; y < yr; y += 2)
   {
      VpeLine(hDoc, 2.00, 10.50 + y * 7.50 / yr, 17.00, 10.50 + y * 7.50 / yr);
      wsprintf(s, "%d", (int)y);
      VpePrint(hDoc, 9.60, 10.50 - y * 7.50 / yr, s);
      VpeLine(hDoc, 2.00, 10.50 - y * 7.50 / yr, 17.00, 10.50 - y * 7.50 / yr);
      wsprintf(s, "%d", (int)-y);
      VpePrint(hDoc, 9.60, 10.50 + y * 7.50 / yr, s);
   }

   y = 18.50;
   wsprintf(s, "[S 14]The three graphs together consist of %d (number determined during runtime) single lines!\n\nVPE manages this data bulk for you FAST!", segments);
   VpeWrite(hDoc, 2.00, y, 20.00, VFREE, s);

#ifdef _WINDOWS
   VpeSetProgressBar(hDoc, 66);
#endif
}


inline double CompX(VpeCoord w, int numcols, int col)
{
   return w / numcols * col + 2.50;
}


inline double CompY(VpeCoord h, int col, int row)
{
   return h / pow(2.0, col) * row + (h / pow(2.0, col)) / 2 + 2.00;
}


// Page 9 of Precision demo
// ========================
void page9(VpeHandle hDoc)
{
   VpeCoord x, y, w, h, offset;
   double boxwidth, boxheight;
   int item = 1;
   double gap = 1.00;

   char s[80];

   VpePageBreak(hDoc);
   VpeSetPageOrientation(hDoc, VORIENT_LANDSCAPE);
   VpeUseSet(hDoc, HEADLINE);
   VpePrint(hDoc, 2.00, 1.50, "A Diagram on a Landscape Oriented Page in the Middle of the Document:");
   
   VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_OPEN);
   VpeAddBookmark(hDoc, ParentBookmark, "Page Orientation");

   VpeUseSet(hDoc, BODYTEXT);
   VpeSetAutoBreak(hDoc, AUTO_BREAK_NO_LIMITS);
   VpeSetFontName(hDoc, HELVETICA);
   VpePrint(hDoc, 2.00, VpeGet(hDoc, VBOTTOMMARGIN) + 0.75,
      "Note: if this page is not printed in landscape orientation, your printer driver "
      "is not capable of switching the orientation in the middle of a print job.");
   VpeSetBold(hDoc, TRUE);

   w = VpeGetPageWidth(hDoc) - 5.00;
   h = VpeGetPageHeight(hDoc) - 5.00;

   VpeSetBkgMode(hDoc, VBKG_SOLID);
   VpeSetBkgColor(hDoc, RGB(190, 255, 190));

   int numcols = 5;
   for (int col = 0; col < numcols; col++)
   {
      for (int row = 0; row < pow(2.0, col); row++)
      {
         // draw the textbox:
         x = CompX(w, numcols, col);
         y = CompY(h, col, row);

         if (col < numcols -1)
         {
            boxwidth = 3.00;
            boxheight = 1.50;
            offset = 0;
         }
         else
         {
            boxwidth = 4.00;
            boxheight = 0.75;
            offset =  (1.50 - boxheight) / 2.00;
         }

         if (col < numcols - 1)
         {
            // Draw connecting lines to both succeeding objects:
            VpeSetPenSize(hDoc, 0.05);
            VpeLine(hDoc, x + boxwidth, y + boxheight / 2, x + boxwidth + gap, y + boxheight / 2);
            VpeLine(hDoc, VRIGHT, VTOP, VRIGHT, CompY(h, col + 1, row * 2) + boxheight / 2);
            VpeLine(hDoc, VRIGHT, VTOP, CompX(w, numcols, col + 1), VTOP);
            VpeLine(hDoc, x + boxwidth + gap, y + boxheight / 2,
                          x + boxwidth + gap, CompY(h, col + 1, row * 2 + 1) + boxheight / 2);
            VpeLine(hDoc, VRIGHT, VBOTTOM, CompX(w, numcols, col + 1), VBOTTOM);
            VpeSetPenSize(hDoc, 0.03);
         }

         wsprintf(s, "Diagram Item #%d", item++);
         VpeWriteBox(hDoc, x, y + offset, -boxwidth, -boxheight, s);
      }
   }

   VpePageBreak(hDoc);
   VpeSetPageOrientation(hDoc, VORIENT_PORTRAIT);
}


// Page 10 and 11 of Precision demo
// ================================
void page10_11(VpeHandle hDoc)
{
   VpeCoord y = 1.00;
   VpeUseSet(hDoc, BODYTEXT);

   if (VpeGetEdition() > VEDITION_STANDARD)
   {
      VpeWriteBox(hDoc, 1.00, 1.00, -4.50, VFREE, "If you are viewing this in Acrobat Reader, zoom in to see the correct barcodes!");
      VpeUseSet(hDoc, HEADLINE);
      VpeNoPen(hDoc);
      VpeSetFontAttr(hDoc, ALIGN_CENTER, 0, 1, 0, 0);
      VpeSetFont(hDoc, HELVETICA, 18);
      VpeWrite(hDoc, 0, 1.50, 21.00, 3.00, "The supported barcode-types:");

      VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_OPEN);
      VpeAddBookmark(hDoc, ParentBookmark, "Supported Barcode Types");

      VpeSetFont(hDoc, HELVETICA, 10);
      VpeSetBold(hDoc, 1);

      VpeWrite(hDoc, 2.00, 3.00, 5.50, 4.00, "2 of 5:");
      VpeBarcode(hDoc, 2.00, 3.60, 5.50, 5.60, BCT_2OF5, "123789", NULL);

      VpeWrite(hDoc, 7.00, 3.00, 12.00, 4.00, "Interleaved 2 of 5:");
      VpeBarcode(hDoc, 7.00, 3.60, 12.00, 5.60, BCT_INTERLEAVED2OF5, "12389578348", NULL);

      VpeWrite(hDoc, 13.50, 3.00, 17.50, 4.00, "Code 39 (text on top):");
      VpeSetBarcodeParms(hDoc, 1, 0);
      VpeBarcode(hDoc, 13.50, 3.60, 17.50, 5.60, BCT_CODE39, "ABC123", NULL);

      VpeWrite(hDoc, 2.00, 7.00, 5.50, 8.00, "Code 93 (rotated):");
      VpeSetBarcodeParms(hDoc, 0, 0);
      VpeSetRotation(hDoc, 900);
      VpeBarcode(hDoc, 2.75, 7.60, -3.00, -2.00, BCT_CODE93, "DEF987", NULL);

      VpeWrite(hDoc, 7.00, 7.00, 12.00, 8.00, "[Rot 0]Codabar (rotated):");
      VpeSetRotation(hDoc, 1800);
      VpeBarcode(hDoc, 7.00, 7.60, -5.00, -2.00, BCT_CODABAR, "A123456B", NULL);

      VpeWrite(hDoc, 14.00, 7.00, 17.00, 8.00, "[Rot 0]EAN-8 (rotated):");
      VpeSetRotation(hDoc, 2700);
      VpeBarcode(hDoc, 14.50, 7.60, -3.00, -2.00, BCT_EAN8, "40167794", NULL);

      VpeWrite(hDoc, 2.00, 12.00, 5.00, 14.00, "[Rot 0]EAN-8 + 2:");
      VpeSetBarcodeParms(hDoc, 0, 1);
      VpeBarcode(hDoc, 2.00, 12.60, 5.00, 14.60, BCT_EAN8_2, "12345670", "12");

      VpeWrite(hDoc, 7.00, 12.00, 12.00, 14.00, "EAN-8 + 5:");
      VpeSetBarcodeParms(hDoc, 0, 0);
      VpeBarcode(hDoc, 7.00, 12.60, 12.00, 14.60, BCT_EAN8_5, "98765430", "12345");

      VpeWrite(hDoc, 13.50, 12.00, 17.50, 14.00, "EAN-13:");
      VpeBarcode(hDoc, 13.50, 12.60, 17.50, 14.60, BCT_EAN13, "9781556153952", NULL);

      VpeWrite(hDoc, 2.00, 16.00, 6.00, 18.00, "EAN-13 + 2:");
      VpeBarcode(hDoc, 2.00, 16.60, 6.00, 18.60, BCT_EAN13_2, "4501645096787", "12");

      VpeWrite(hDoc, 7.00, 16.00, 12.00, 18.00, "EAN-13 + 5:");
      VpeSetBarcodeParms(hDoc, 0, 1);
      VpeBarcode(hDoc, 7.00, 16.60, 12.00, 18.60, BCT_EAN13_5, "9781556153952", "12345");

      VpeWrite(hDoc, 13.50, 16.00, 17.50, 18.00, "EAN-128 A:");
      VpeSetBarcodeParms(hDoc, 0, 0);
      VpeBarcode(hDoc, 13.50, 16.60, 17.50, 18.60, BCT_EAN128, "EAN-128 A", NULL);

      VpeWrite(hDoc, 2.00, 20.00, 6.00, 22.00, "EAN-128 B:");
      VpeBarcode(hDoc, 2.00, 20.60, 6.00, 22.60, BCT_EAN128, "ean-128 b", NULL);

      VpeWrite(hDoc, 7.00, 20.00, 12.00, 22.00, "EAN-128 A/B/C:");
      VpeBarcode(hDoc, 7.00, 20.60, 12.00, 22.60, BCT_EAN128, "Ean128902", NULL);

      VpeSetBarcodeParms(hDoc, BCP_DEFAULT, BCP_DEFAULT);   
      VpeWrite(hDoc, 13.50, 20.00, 18.50, 22.00, "POSTNET (1.20) 5/9/11 digits:");
      VpeBarcode(hDoc, 13.50, 20.60, 16.28, 21.20, BCT_POSTNET, "12345", NULL);
      VpeBarcode(hDoc, 13.50, 21.50, 18.50, 22.10, BCT_POSTNET, "414649623", NULL);
      VpeBarcode(hDoc, 13.50, 22.40, 19.50, 23.00, BCT_POSTNET, "41464962339", NULL);

      VpeWrite(hDoc, 2.00, 24.00, 6.00, 25.00, "Code-128 A:");
      VpeBarcode(hDoc, 2.00, 24.60, 6.00, 26.60, BCT_CODE128, "CODE-128 A", NULL);

      VpeWrite(hDoc, 7.00, 24.00, 12.00, 25.00, "Code-128 B:");
      VpeBarcode(hDoc, 7.50, 24.60, 11.50, 26.60, BCT_CODE128, "code-128 b", NULL);

      VpeWrite(hDoc, 13.50, 24.00, 18.00, 25.00, "Code-128 A/B/C:");
      VpeBarcode(hDoc, 13.50, 24.60, 18.00, 26.60, BCT_CODE128, "Code128902", NULL);

#ifdef _WINDOWS
	  VpeSetProgressBar(hDoc, 75);
#endif
      VpePageBreak(hDoc);

      VpeSetFontAttr(hDoc, ALIGN_CENTER, 0, 1, 0, 0);
      VpeSetFont(hDoc, HELVETICA, 18);
      VpeWrite(hDoc, 0, 1.50, 21.00, 3.00, "The supported barcode-types (continued):");
      VpeSetFont(hDoc, HELVETICA, 10);
      VpeSetBold(hDoc, 1);

      VpeWrite(hDoc, 2.00, 3.00, 5.50, 4.00, "UPC-A:");
      VpeBarcode(hDoc, 2.00, 3.60, 5.50, 5.60, BCT_UPCA, "07447079382", NULL);

      VpeWrite(hDoc, 7.00, 3.00, 11.00, 4.00, "UPC-A + 2:");
      VpeBarcode(hDoc, 7.00, 3.60, 11.00, 5.60, BCT_UPCA_2, "07447079382", "01");

      VpeWrite(hDoc, 13.50, 3.00, 18.00, 4.00, "UPC-A + 5:");
      VpeBarcode(hDoc, 13.50, 3.60, 18.00, 5.60, BCT_UPCA_5, "03126764825", "94687");

      VpeWrite(hDoc, 2.00, 7.00, 5.50, 9.00, "UPC-E:");
      VpeBarcode(hDoc, 2.00, 7.60, 5.50, 9.60, BCT_UPCE, "0378492", NULL);

      VpeWrite(hDoc, 7.00, 7.00, 11.00, 9.00, "UPC-E + 2:");
      VpeBarcode(hDoc, 7.00, 7.60, 11.00, 9.60, BCT_UPCE_2, "0378492", "14");

      VpeWrite(hDoc, 13.50, 7.00, 18.00, 9.00, "UPC-E + 5:");
      VpeBarcode(hDoc, 13.50, 7.60, 18.00, 9.60, BCT_UPCE_5, "0364825", "79462");

      VpeSetBarcodeAlignment(hDoc, ALIGN_CENTER);
      VpeSetBarcodeParms(hDoc, BCP_DEFAULT, BCP_DEFAULT);
      VpeWrite(hDoc, 2.00, 11.00, 5.50, 12.00, "EAN 2:");
      VpeBarcode(hDoc, 3.50, 11.60, 4.00, 13.60, BCT_EAN2, "12", NULL);

      VpeWrite(hDoc, 7.00, 11.00, 10.50, 12.00, "EAN 5:");
      VpeBarcode(hDoc, 8.00, 11.60, 9.50, 13.60, BCT_EAN5, "12345", NULL);

      VpeWrite(hDoc, 13.50, 11.00, 18.00, 12.00, "Code 39 Extended:");
      VpeBarcode(hDoc, 12.50, 11.60, 19.00, 13.60, BCT_CODE39EXT, "Code~39", NULL);

      VpeWrite(hDoc, 1.50, 14.50, 6.00, 15.50, "RM4SCC - Royal Mail:");
      VpeBarcode(hDoc, 2.00, 15.10, 5.50, 15.90, BCT_RM4SCC, "SN34RD1A", NULL);

      VpeWrite(hDoc, 1.50, 16.70, 6.00, 17.70, "ISBN / ISBN+EAN5:");
      VpeBarcode(hDoc, 2.00, 17.30, 5.50, 19.20, BCT_ISBN, "3-518-10012-2", NULL);

      VpeWrite(hDoc, 7.00, 15.00, 10.50, 16.00, "MSI:");
      VpeBarcode(hDoc, 8.00, 15.60, 9.50, 17.60, BCT_MSI, "12345", NULL);

      VpeWrite(hDoc, 13.50, 15.00, 18.00, 16.00, "Code 93 Extended:");
      VpeBarcode(hDoc, 13.50, 15.60, 18.00, 17.60, BCT_CODE93EXT, "Code~93", NULL);

      VpeBarcode(hDoc, 1.50, 19.60, 6.00, 21.60, BCT_ISBN_5, "3-518-10012-2", "09980");

      VpeWrite(hDoc, 6.00, 19.00, 11.50, 20.50, "Identcode (Deutsche Post AG):");
      VpeBarcode(hDoc, 7.00, 19.60, 10.50, 21.60, BCT_IDENTCODE, "56.310 243031", NULL);

      VpeWrite(hDoc, 12.50, 19.00, 18.50, 20.50, "Leitcode (Deutsche Post AG):");
      VpeBarcode(hDoc, 13.00, 19.60, 18.00, 21.60, BCT_LEITCODE, "21348.075.016.40", NULL);

      VpeWrite(hDoc, 2.00, 23.00, 5.50, 24.00, "PZN (Pharma):");
      VpeBarcode(hDoc, 2.00, 23.60, 5.50, 24.60, BCT_PZN, "PZN-123456", NULL);

      VpeWrite(hDoc, 2.00, 25.10, 5.50, 26.00, "Code 11:");
      VpeBarcode(hDoc, 2.00, 25.70, 5.50, 26.70, BCT_CODE11, "12-34569", NULL);

      VpePrint(hDoc, 6.50, 23.00,
         "[N UO L]Barcodes are as easily generated as images. An example:\n\n"
         "Barcode(7.00, 16.60, 12.00, 18.60, BCT_EAN13_5, \"9781556153952\", \"12345\")\n\n"
         "VPE supports 38 barcode types. Check digits can be auto-generated for all types if wanted. "
         "The barcodes can be rotated in 90 degree steps, the font of the descriptive text can be specified and the "
         "text can be hidden. Text and add-on text may be drawn on the bottom "
         "or top of the barcode and the ratio between the modules can be specified.");

      VpePageBreak(hDoc);
   }

	if (VpeGetEdition() > VEDITION_COMMUNITY)
	{
		// The Community Edition has no Gradients and rounded Corners
		VpeCoord x;
		char *s;
		VpeSetFontName(hDoc, TIMES);
		VpeSetBkgMode(hDoc, VBKG_GRD_LINE);
		VpeSetBkgGradientTriColor(hDoc, TRUE);
		VpeSetBkgGradientMiddleColorPosition(hDoc, 35);
		VpeSetBkgGradientStartColor(hDoc, COLOR_LTGRAY);
		VpeSetBkgGradientMiddleColor(hDoc, COLOR_WHITE);
		VpeSetBkgGradientEndColor(hDoc, COLOR_DKGRAY);
		VpeSetPenStyle(hDoc, PS_SOLID);
		VpeSetUnderline(hDoc, FALSE);
		s = "[PS 0.03 S 20 C Black]   How about some gradients?   ";
		VpeRenderPrintBox(hDoc, 2.00, y + 1.00, s);

		VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_OPEN);
		VpeAddBookmark(hDoc, ParentBookmark, "Gradients");

		x = 10.00 - VpeGet(hDoc, VRENDERWIDTH) / 2;
		y = VpePrintBox(hDoc, x, y + 0.70, s);
		VpeSetBkgGradientTriColor(hDoc, FALSE);

		y += 0.75;
		VpeSetBkgMode(hDoc, VBKG_GRD_ELLIPSE);
		VpeSetBkgGradientStartColor(hDoc, COLOR_DKORANGE);
		VpeSetBkgGradientEndColor(hDoc, COLOR_LTYELLOW);
		y = VpeWriteBox(hDoc, 1.00, y, -6.00, -6.00,"[CE S20 B C Blue]\n\nVirtual\nPrint\nEngine");

		VpeSetBkgGradientStartColor(hDoc, COLOR_BLUE);
		VpeSetBkgGradientEndColor(hDoc, COLOR_CYAN);
		VpeSetBkgGradientRotation(hDoc, 900);
		VpeSetBkgMode(hDoc, VBKG_GRD_LINE);
		VpePrintBox(hDoc, 8.00, VTOP, "[C White] The printing and presentation ");

		VpeSetBkgMode(hDoc, VBKG_GRD_RECT);
		VpeSetBkgGradientStartColor(hDoc, COLOR_LTBLUE);
		VpeSetBkgGradientEndColor(hDoc, COLOR_YELLOW);
		x = VpeGet(hDoc, VWIDTH);
		VpeBox(hDoc, 8.00, VpeGet(hDoc, VBOTTOM) + 1.00, -x, y);
		VpeCoord bottom = VpeGet(hDoc, VBOTTOM);

		s = "[T C Black S48 I]Solution!";
		VpeRenderPrint(hDoc, 0, 0, s);
		x = 8.00 + (x - VpeGet(hDoc, VRENDERWIDTH)) / 2;
		y = VpeGet(hDoc, VTOP) + (VpeGet(hDoc, VHEIGHT) - VpeGet(hDoc, VRENDERHEIGHT)) / 2;
		VpePrint(hDoc, x,  y, s);

		y = bottom - 0.25;
		VpeSetBkgMode(hDoc, VBKG_GRD_LINE);
		VpeSetBkgGradientRotation(hDoc, 0);
		VpeSetBkgGradientStartColor(hDoc, COLOR_LTBLUE);
		VpeSetBkgGradientEndColor(hDoc, COLOR_BLACK);
		s = "[S 20 U C White]Text and images";
		VpeRenderPrint(hDoc, 7.50, y+0.75, s);
		x = 10.00 - VpeGet(hDoc, VRENDERWIDTH) / 2;
		y = VpePrintBox(hDoc, x, y+0.75, s);
		VpeSetBkgGradientRotation(hDoc, 900);
		VpeSetBkgGradientStartColor(hDoc, COLOR_LTRED);
		VpePrintBox(hDoc, VRIGHT, VBOTTOM, "[Rot 900] can be freely ");
		VpeSetBkgGradientRotation(hDoc, 1800);
		VpeSetBkgGradientStartColor(hDoc, COLOR_LTYELLOW);
		VpeWriteBox(hDoc, x, VBOTTOM, VLEFT, VFREE, "[Rot 1800 CE]rotated in 90");

		// The WIDTH (after rotation it's the height) is the top of the last inserted text
		// minus the bottom of the first inserted object.
		VpeSetBkgGradientRotation(hDoc, 2700);
		VpeSetBkgGradientStartColor(hDoc, COLOR_GREEN);
		VpeRenderWriteBox(hDoc, 0, 0, -5.00, VFREE, "[Rot 2700]degree steps");
		VpeWriteBox(hDoc, x - VpeGet(hDoc, VRENDERWIDTH), y, -(VpeGet(hDoc, VTOP) - y), VFREE, "[Rot 2700]degree steps");

		VpeSetRotation(hDoc, 0);
		VpeSetUnderlined(hDoc, 0);

		VpeSetBkgGradientRotation(hDoc, 0);
		VpeSetTextColor(hDoc, COLOR_BLACK);
		VpeSetBkgGradientTriColor(hDoc, TRUE);
		VpeSetBkgGradientMiddleColorPosition(hDoc, 40);
		VpeSetBkgGradientStartColor(hDoc, COLOR_BLUE);
		VpeSetBkgGradientMiddleColor(hDoc, COLOR_WHITE);
		VpeSetBkgGradientEndColor(hDoc, COLOR_DKBLUE);
		VpeWriteBox(hDoc, 1.00, 20.00, VFREE, VFREE, "   Three-Dimensional Gradient   ");

		VpeSetCornerRadius(hDoc, 0.50);
		VpeSetBkgColor(hDoc, RGB(190, 255, 190));
		VpeSetHatchStyle(hDoc, HS_FDIAGONAL);
		VpeSetBkgMode(hDoc, VBKG_SOLID);
		VpeWriteBox(hDoc, 1.00, 22.00, VFREE, VFREE, "   Rounded   ");

		VpeSetBkgMode(hDoc, VBKG_GRD_LINE);
		VpeSetBkgGradientMiddleColorPosition(hDoc, 50);
		VpeSetBkgGradientStartColor(hDoc, COLOR_ORANGE);
		VpeSetBkgGradientEndColor(hDoc, COLOR_ORANGE);
		VpeWriteBox(hDoc, 7.00, 22.00, VFREE, VFREE, "   Rounded   ");

		VpeSetCornerRadius(hDoc, 0);
		VpeSetBkgMode(hDoc, VBKG_TRANSPARENT);
		VpeSetHatchStyle(hDoc, HS_NONE);
		VpeWriteBox(hDoc, 13.00, 22.00, VFREE, VFREE, "[S 10]NOTE: Rounded rectangles with gradient fills do not work on Win9x/Me.");

		VpePageBreak(hDoc);
	}

#ifdef _WINDOWS
	VpeSetProgressBar(hDoc, 83);
#endif
}


// Page 12 of Precision demo
// =========================
void page12(VpeHandle hDoc)
{
   VpeNoPen(hDoc);
   VpeSetFont(hDoc, HELVETICA, 10);

   VpeAddBookmark(hDoc, ParentBookmark, "Filled Form");
   VpePicture(hDoc, 0, 0, VFREE, VFREE, GEW);
   if (VpeGetEdition() >= VEDITION_PROFESSIONAL)
   {
      VpeWrite(hDoc, 11.00, 15.20, 19.50, VFREE,
         "[S 10 J I C LtRed]"
         "The bitmap form has a resolution of 300 DPI. Therefore the preview is not very nice.\n"
         "Try the Professional Edition Demo, to see the Scale-to-Gray Technology!");
   }
   else
   {
      VpeWrite(hDoc, 11.00, 15.20, 19.50, VFREE,
         "[S 10 J I C LtRed]"
         "It is very important to mention here, that the bitmap form has a resolution of 96 "
         "DPI only. The print will not be very nice. Just try a 300 DPI bitmap on your own!");
   }

   VpeWrite(hDoc, 2.50, 1.10, 12.00, VFREE, "[S 24 C LtRed L PS 0 IO]Freie Bank");
   VpeWrite(hDoc, 2.50, 5.00, 10.00, VFREE, "[S 14 C Blue]Mustermann & Co.\nFeinkost Im- und Export");
   VpeWrite(hDoc, 12.00, 4.60, 17.00, VFREE, "[S 11 B]Dsseldorf");
   VpeWrite(hDoc, 2.50, 6.80, 7.50, VFREE, "Schmidt");
   VpeWrite(hDoc, 2.50, 8.45, 7.50, VFREE, "24.7. 1947");
   VpeWrite(hDoc, 5.00, 8.45, 10.00, VFREE, "Krefeld");
   VpeWrite(hDoc, 2.50, 10.15, 7.50, VFREE, "Landstr. 93");
   VpeWrite(hDoc, 10.00, 10.10, 17.50, VFREE, "0 27 84 / 16 45 98");
   VpeWrite(hDoc, 2.50, 12.65, 17.50, VFREE, "Parkstr. 17");                   
   VpeWrite(hDoc, 10.00, 12.60, 17.50, VFREE, "0 27 84 / 23 54 90");
   VpeWrite(hDoc, 12.20, 6.70, 17.50, VFREE, "Peter - Thomas");

#ifdef _WINDOWS
   VpeSetProgressBar(hDoc, 91);
#endif
}


// Page 13 of Precision demo
// ========================
void page13(VpeHandle hDoc)
{
   VpeCoord y;
   
   VpePageBreak(hDoc);
   VpeUseSet(hDoc, HEADLINE);
   
   VpeSetBookmarkStyle(hDoc, VBOOKMARK_STYLE_ITALIC + VBOOKMARK_STYLE_BOLD + VBOOKMARK_STYLE_OPEN);	// Bold and Italic styles will work only with PDF 1.4
   VpeSetBookmarkColor(hDoc, RGB(0, 0, 255));	// will work only with PDF 1.4
   VpeAddBookmark(hDoc, NULL, "Contact");

   y = VpeWrite(hDoc, VLEFTMARGIN, 2.50, VRIGHTMARGIN, VFREE, "[CE I S 26]VPE is Unlimited Printing Power");

   VpeUseSet(hDoc, BODYTEXT);
   y = VpeWrite(hDoc, 2.50, y + 1.00, 17.50, VFREE,
      "[S 14 J]The well engineered set of dynamic layout functions and the detailed manual / help "
      "will never leave you alone in the dark. It allows the rapid solution of all "
      "printing and presentation problems.\n\nThe Virtual Print Engine is the answer to the "
      "nightmare of printing.");

   VpeSetPenSize(hDoc, 0.06);
   VpeLine(hDoc, 6.00, y + 2.00, 14.00, y + 2.00);
   
   y = VpeWrite(hDoc, VLEFTMARGIN, y + 4.00, VRIGHTMARGIN, VFREE, "[BO CE I S 22]Order Today");
   VpeUseSet(hDoc, BODYTEXT);
   VpeNoPen(hDoc);
   VpePicture(hDoc, 8.75, y + 0.50, VFREE, VFREE, PROD);
   y = VpeWrite(hDoc, VLEFTMARGIN, VpeGet(hDoc, VBOTTOM) + 1.00, VRIGHTMARGIN, VFREE,
       "[B CE]For more information please visit us at:");
   y = VpeWrite(hDoc, 2.50, y + 0.50, 17.50, VFREE, "[N 'Helvetica' I S 20]www.IdealSoftware.com");
   y = VpeWrite(hDoc, 2.50, y + 0.50, 17.50, VFREE,
      "There you will find up-to-date information and the latest version for free download.\n\n"
      "Online ordering is also possible, get your license key 24/7 within 5 minutes!\n\n"
      "(German and English contents)");

#ifdef _WINDOWS
   VpeSetProgressBar(hDoc, 100);
#endif
}


// Precision demo
// ==============
void precision(int mode = 0)
{
	VpeHandle hDoc;
	char s[128];

#ifdef _WINDOWS
	if (mode == 0)
	{
		if (VpeGetEdition() >= VEDITION_STANDARD)
			WinMsg("The following preview will be embedded as child window into the application's main window.\nYou just need a few lines of code!");

#ifdef CREATE_SWAPFILES
		unlink("cap_sf.vpe");
		hDoc = VpeOpenDocFile(hMainWindow, "cap_sf.vpe", "Precision + Capabilities", VPE_EMBEDDED + VPE_GRIDBUTTON);
#else
		hDoc = VpeOpenDoc(hMainWindow, "Precision + Capabilities", VPE_EMBEDDED + VPE_GRIDBUTTON);
#endif
 		if (!hDoc)
		{
			Msg("The Document could not be created!");
			return;
		}

		VpeSetGUITheme(hDoc, Theme);
		VpeSetRulersMeasure(hDoc, Units);

		Precision = hDoc;
		VpePreviewDoc(hDoc, NULL, VPE_SHOW_NORMAL);

		// this is an embedded window, forward the focus to the preview, so
		// it receives keyboard and mouse-wheel messages
		SetFocus(VpeGetWindowHandle(hDoc));	
		VpeEnableHelpRouting(hDoc, TRUE);
		VpeOpenProgressBar(hDoc);
	}
	else
#endif
	{
		WinMsg("The document is generated and printed invisibly in background, i.e. without showing a preview.\nNo printer-setup is done - the settings of the default printer are used.");
		hDoc = VpeOpenDoc(hMainWindow, "Precision + Capabilities",  0);
		if (!hDoc)
		{
			Msg("The Document could not be created!");
			return;
		}
		PBackGnd = hDoc;
	}

	VpeSet(hDoc, VRIGHTMARGIN, 20.00);
	VpeSetDefOutRect(hDoc, 1.00, 1.00, 19.00, 27.30);

	page1(hDoc);
	page2(hDoc);
	page34(hDoc);
	page5(hDoc);
	page6(hDoc);
	page7(hDoc);
	page8(hDoc);
	page9(hDoc);
	page10_11(hDoc);
	page12(hDoc);
	page13(hDoc);

	// Insert page numbers in the style "Page n of <total>":
	// =====================================================
	VpeUseSet(hDoc, BODYTEXT);
	VpeSetTextAlignment(hDoc, ALIGN_RIGHT);
	VpeSetItalic(hDoc, TRUE);

	for (int page = 2; page <= VpeGetPageCount(hDoc); page++)
	{
		VpeGotoPage(hDoc, page);
		sprintf(s, "Page %d of %d", page, VpeGetPageCount(hDoc));
		VpeWrite(hDoc, VpeGet(hDoc, VRIGHTMARGIN) - 5.00, VTOPMARGIN, VpeGet(hDoc, VRIGHTMARGIN) + 1.00, VFREE, s);
	}

	VpeRemoveSet(hDoc, HEADLINE);
	VpeRemoveSet(hDoc, BODYTEXT);

#ifdef _WINDOWS
	VpeCloseProgressBar(hDoc);
#endif

#ifdef CREATE_FILES
	#ifdef CREATE_SWAPFILES
		unlink("cap_sw.vpe");
		VpeWriteDoc(hDoc, "cap_sw.vpe");
	#else
		unlink("cap.vpe");
		VpeWriteDoc(hDoc, "cap.vpe");
	#endif
#endif

	if (mode != 0)
	{
#ifdef _WINDOWS
		VpePrintDoc(hDoc, FALSE);
#else
		ExportDocument(hDoc, _T("Capabilities and Precision.pdf"));
#endif
		VpeCloseDoc(hDoc);
	}
	else 
		ExportDocument(hDoc, _T("Capabilities and Precision.pdf"));

	return;
}


//----------------------------------------------------------------------------
// Report Demo
//----------------------------------------------------------------------------
#define FACTOR  49

void GroupHeader(VpeHandle hDoc, char *Product)
{
   VpeCoord y;
   char s[80];

   VpeSetTextAlignment(hDoc, ALIGN_LEFT);
   wsprintf(s, "[N S 18 C Blue]Product: %s", Product);
   y = VpePrint(hDoc, 1.50, VpeGet(hDoc, VBOTTOM), s);
   y = y + 0.10;
   VpePrint(hDoc, 1.50, y, "[S 16 C Purple]Country");
   VpeWrite(hDoc, 6.50, y, 9.00, VFREE, "[R]Quantity");
   VpeWrite(hDoc, 11.50, y, 14.50, VFREE, "Value (in $)");
}



void DetailLine(VpeHandle hDoc, COLORREF Color, char *Country, double quantity)
{
   VpeCoord y;
   char s[80];

   y = VpeGet(hDoc, VBOTTOM);
   VpeSetBkgMode(hDoc, VBKG_SOLID);
   VpeSetBkgColor(hDoc, Color);
   VpeSetPenColor(hDoc, Color);
   VpeBox(hDoc, 1.50, y, 15.50, LineHeight);
   VpeSetBkgMode(hDoc, VBKG_TRANSPARENT);

   wsprintf(s, "[S 14 C Black L]%s", Country);
   VpeWriteBox(hDoc, 1.50, y, -4.00, LineHeight, s);
   VpeSetTextAlignment(hDoc, ALIGN_RIGHT);
   double2NumberString(quantity, s);
   VpeWriteBox(hDoc, 6.50, y, 9.00, LineHeight, s);
   double2NumberString(quantity * FACTOR, s);
   VpeWriteBox(hDoc, 11.50, y, 14.50, LineHeight, s);
}



void GroupFooter(VpeHandle hDoc, double quantity)
{
   VpeCoord y;
   char s[80];

   // PrintBox and WriteBox are used here, because in the Detail-Section they are used, too.
   // If we wouldn't use them, the Footer-Section would be misaligned, because of
   // the invisible surrounding rectangle of the Detail-Section.
   y = VpeGet(hDoc, VBOTTOM) + VpeGetPenSize(hDoc) + 0.02;
                // + Pensize, because the white frame would otherwise overlap the Detail-Section's frame
                // + 0.02, to avoid rounding problems we use a bit more
   VpeSetPenColor(hDoc, COLOR_WHITE);
   VpePrintBox(hDoc, 1.50, y, "Total");
   double2NumberString(quantity, s);
   VpeWriteBox(hDoc, 6.50, y, 9.00, LineHeight, s);
   double2NumberString(quantity * FACTOR, s);
   VpeWriteBox(hDoc, 11.50, y, 14.50, LineHeight, s);
   VpeSet(hDoc, VBOTTOM, y + 1.80);
}


void reporttest(int mode)
{
   VpeHandle hDoc;
   VpeCoord y;

 #ifdef CREATE_SWAPFILES
   unlink("reprt_sf.vpe");
   hDoc = VpeOpenDocFile(hMainWindow, "reprt_sf.vpe", "Report",  0);
 #else
   hDoc = VpeOpenDoc(hMainWindow, "Report",  0);
 #endif
   if (!hDoc)
   {
      Msg("The Document could not be created!");
      return;
   }

#ifdef _WINDOWS
   VpeSetGUITheme(hDoc, Theme);
   VpeSetRulersMeasure(hDoc, Units);
#endif

   if (mode == 0)
      Report = hDoc;
   else
   {
      if (mode == 2)
         WinMsg("This option does not work with some MAPI clients (dialog will be shown).\n"
                "This also depends on the MAPI client configuration!");
         
      WinMsg("The document will be generated and e-mailed invisibly in background.\n\n"
          "By code, the mail subject is set to 'VPE Demo'.\n\n"
          "By code, the following mail recipients are defined:\n"
          "- MrX@dummyxyz.com\n"
          "- MrY@dummyxyz.com\n\n"
          "By default, the VPE document is automatically added as attachment.\n"
          "By code, the attachment 'c++\\vpedemo.cpp' is added.\n"
          "Note: You can also remove the VPE document attachment by code.");
   }
  
   VpeRenderPrintBox(hDoc, 0, 0, "[S 14 C Black]x");
   LineHeight = -VpeGet(hDoc, VRENDERHEIGHT);

   VpeSetAutoBreak(hDoc, AUTO_BREAK_NO_LIMITS);
   VpeSetPen(hDoc, 0.05, PS_SOLID, COLOR_BLACK);
   VpePicture(hDoc, 16.50, 1.50, VFREE, VFREE, FRUITS);
   VpeWriteBox(hDoc, 16.50, VpeGet(hDoc, VBOTTOM), VpeGet(hDoc, VRIGHT), VFREE, "[N S 9 CE I C Blue BC Gray TO]Better Fruits Software");
   y = VpeGet(hDoc, VBOTTOM) + 1.00;
   VpePrint(hDoc, 1.50, 2.00, "[N S 26 U]Year End Results");
   VpePrint(hDoc, 1.50, 4.00, "[N S 32]Better Fruits Software");

   VpeLine(hDoc, 1.50, y, 20.00, y);
   VpeSet(hDoc, VBOTTOM, y + 0.50);

   struct
   {
      char   *product;
      double germany;
      double france;
      double usa;
      double australia;
   } table[3] =
   {
      "Apples",  2450, 1700, 3200, 1080,
      "Oranges", 2830, 2240, 3650, 1470,
      "Bananas", 1560,  925, 2130,  520,
   };

   double group_total, total = 0;
   char s[80], s2[80];

   for (int n = 0; n < 3; n++)
   {
      GroupHeader(hDoc, table[n].product);
      DetailLine(hDoc, COLOR_GRAY, "Germany", table[n].germany);
      DetailLine(hDoc, COLOR_HIBLUE, "France", table[n].france);
      DetailLine(hDoc, COLOR_GRAY, "USA", table[n].usa);
      DetailLine(hDoc, COLOR_HIBLUE, "Australia", table[n].australia);
      group_total = table[n].germany +
              table[n].france +
              table[n].usa +
              table[n].australia;
      GroupFooter(hDoc, group_total);
      total += group_total;
   }

   double2NumberString(total * FACTOR, s);
   sprintf(s2, "[N S 20 U]Yearly Country Sales Total: $%s", s);
   VpePrint(hDoc, 1.50, VpeGet(hDoc, VBOTTOM), s2);

   // Draw a pie
   // use the VpeGet()-stuff to easily position this anywhere on the paper
   // ====================================================================
   VpePageBreak(hDoc);
   VpeSetFont(hDoc, TIMES, 12);
   VpePrint(hDoc, 2.00, VBOTTOM, "[N S 18 U]Analysis of Paradise:");
   VpeSetPen(hDoc, 0.03, PS_SOLID, COLOR_BLACK);
   VpeSetBkgMode(hDoc, FALSE);
   VpeSetBkgColor(hDoc, COLOR_RED);

   VpePie(hDoc, 2.00, VpeGet(hDoc, VBOTTOM) + 1.00, -6.00, -6.00, 0, 300);
   VpeStorePos(hDoc);
   VpeBox(hDoc, VpeGet(hDoc, VRIGHT) + 2.00, VpeGet(hDoc, VTOP) + 0.20, -0.30, -0.30);
   VpePrint(hDoc, VpeGet(hDoc, VRIGHT) + 0.30, VpeGet(hDoc, VTOP) - 0.10, "[N T]Apples");

   VpeRestorePos(hDoc);
   VpeSetBkgColor(hDoc, COLOR_BLUE);
   VpePie(hDoc, VLEFT, VTOP, VRIGHT, VBOTTOM, 300, 750);
   VpeStorePos(hDoc);
   VpeBox(hDoc, VpeGet(hDoc, VRIGHT) + 2.00, VpeGet(hDoc, VTOP) + 0.70, -0.30, -0.30);
   VpePrint(hDoc, VpeGet(hDoc, VRIGHT) + 0.30, VpeGet(hDoc, VTOP) - 0.10, "[N T]Oranges");

   VpeRestorePos(hDoc);
   VpeSetBkgColor(hDoc, COLOR_LTYELLOW);
   VpePie(hDoc, VLEFT, VTOP, VRIGHT, VBOTTOM, 750, 1500);
   VpeStorePos(hDoc);
   VpeBox(hDoc, VpeGet(hDoc, VRIGHT) + 2.00, VpeGet(hDoc, VTOP) + 1.20, -0.30, -0.30);
   VpePrint(hDoc, VpeGet(hDoc, VRIGHT) + 0.30, VpeGet(hDoc, VTOP) - 0.10, "[N T]Bananas");

   VpeRestorePos(hDoc);
   VpeSetBkgColor(hDoc, COLOR_GREEN);
   VpePie(hDoc, VLEFT, VTOP, VRIGHT, VBOTTOM, 1500, 2900);
   VpeStorePos(hDoc);
   VpeBox(hDoc, VpeGet(hDoc, VRIGHT) + 2.00, VpeGet(hDoc, VTOP) + 1.70, -0.30, -0.30);
   VpePrint(hDoc, VpeGet(hDoc, VRIGHT) + 0.30, VpeGet(hDoc, VTOP) - 0.10, "[N T]Cherries");

   VpeRestorePos(hDoc);
   VpeSetBkgColor(hDoc, COLOR_CYAN);
   VpePie(hDoc, VLEFT, VTOP, VRIGHT, VBOTTOM, 2900, 0);
   VpeStorePos(hDoc);
   VpeBox(hDoc, VpeGet(hDoc, VRIGHT) + 2.00, VpeGet(hDoc, VTOP) + 2.20, -0.30, -0.30);
   VpePrint(hDoc, VpeGet(hDoc, VRIGHT) + 0.30, VpeGet(hDoc, VTOP) - 0.10, "[N T]Coconuts");

#ifdef CREATE_FILES
   #ifdef CREATE_SWAPFILES
      unlink("reprt_sw.vpe");
      VpeWriteDoc(hDoc, "reprt_sw.vpe");
   #else
      unlink("report.vpe");
      VpeWriteDoc(hDoc, "report.vpe");
   #endif
#endif


#ifndef _WINDOWS
      ExportDocument(hDoc, _T("Report.pdf"));
      VpeCloseDoc(hDoc);
#else
   // E-Mail functions:
   // =================
   if (mode == 0)
   {
      VpePreviewDoc(hDoc, NULL, VPE_SHOW_NORMAL);
      ExportDocument(hDoc, _T("Report.pdf"));
   }
   else
   {
      char full_path[_MAX_PATH + 1];

      getcwd(full_path, sizeof(full_path));

      // Cut last directory from path:
      size_t i;
      for (i = strlen(full_path) - 1; i >= 0 && full_path[i] != '\\'; i--);
      if (i < 0)
         i = 0;
      full_path[i] = '\0';

      strcat(full_path, "\\c++\\vpedemo.cpp");
      if (mode == 2)
         VpeSetMailWithDialog(hDoc, FALSE);
      VpeSetMailSubject(hDoc, "VPE Demo");
      VpeAddMailReceiver(hDoc, "MrX@dummyxyz.com", VMAIL_TO);
      VpeAddMailReceiver(hDoc, "MrY@dummyxyz.com", VMAIL_CC);
      VpeAddMailAttachment(hDoc, full_path, NULL);

      if (VpeMailDoc(hDoc))
         WinMsg("e-mailing finished without error.");
      else
      {
         char s[80];
         wsprintf(s, "Document could not be e-mailed.\n\nVPE Error-Code: %ld", VpeGetLastError(hDoc));
         WinMsg(s);
      }

      VpeCloseDoc(hDoc);
   }
#endif
}



//----------------------------------------------------------------------------
//                               Colortest
//----------------------------------------------------------------------------
void colortest()
{
   const int range = 1400;
   const int step = 1;
   const int color_step = 2;
   const int min_color = 0;
   const int max_color = 255;
   
   VpeHandle hDoc;
   RECT rc;
   int x;
   VpeCoord y;
   int r, g, b;
   int delta_r, delta_g, delta_b;
   double xx, factor;
   
 #ifdef CREATE_SWAPFILES
   unlink("color_sf.vpe");
   hDoc = VpeOpenDocFile(hMainWindow, "color_sf.vpe", "Colors", VPE_NO_SCALEBTNS + VPE_NO_MOVEBTNS +
                                            VPE_NO_USER_CLOSE + VPE_NO_STATBAR +
                                            VPE_NO_RULERS + VPE_NO_HELPBUTTON +
                                            VPE_NO_INFOBUTTON);
 #else
   hDoc = VpeOpenDoc(hMainWindow, "Colors", VPE_NO_SCALEBTNS + VPE_NO_MOVEBTNS +
                                            VPE_NO_USER_CLOSE + VPE_NO_STATBAR +
                                            VPE_NO_RULERS + VPE_NO_HELPBUTTON +
                                            VPE_NO_INFOBUTTON);
 #endif
   if (!hDoc)
   {
      Msg("The Document could not be created!");
      return;
   }

#ifdef _WINDOWS
   VpeSetGUITheme(hDoc, Theme);
   VpeSetRulersMeasure(hDoc, Units);
   VpeSetPaperView(hDoc, FALSE);
   VpeSetScale(hDoc, 0.25);
#endif
   VpeSetAutoBreak(hDoc, AUTO_BREAK_NO_LIMITS);
   Colors = hDoc;
  
   VpeSetPen(hDoc, 0, PS_SOLID, COLOR_BLACK);
   VpeSetFont(hDoc, HELVETICA, 30);

   xx = -3.1415;
   factor = 2 * fabs(xx) / (double)range * (double)step;

   r = 192;
   delta_r = color_step;
   g = min_color+1;
   delta_g = color_step;
   b = min_color+1;
   delta_b = color_step;
   
   for (x = 100; x < range+100; x+=step)
   {
     y = sin(xx) * 500.0 + 500;
     VpeSetTextColor(hDoc, RGB( r, g, b));
     VpeWriteBox(hDoc, x/100.00, y/100.00, (x + 800)/100.00, (y + 130)/100.00, "Color Test");

     if (x % 10 == 0)
     {
        xx += factor;
        x += step;
        y = sin(xx) * 500.0 + 500;
        VpeSetTextColor(hDoc, COLOR_BLACK);
        VpeWriteBox(hDoc, x/100.00, y/100.00, (x + 800)/100.00, (y + 130)/100.00, "Color Test");
     }
     
     xx += factor;
     if (r > min_color && r < max_color)
     {
        r += delta_r;
        if (r < min_color)
           r = min_color;
        if (r > max_color)
           r = max_color;
     }
     else if (g > min_color && g < max_color)
     {
        g += delta_g;
        if (g < min_color)
           g = min_color;
        if (g > max_color)
           g = max_color;
     }
     else if (b > min_color && b < max_color)
     {
        b += delta_b;
        if (b < min_color)
           b = min_color;
        if (b > max_color)
           b = max_color;
     }

     if (r >= max_color && b >= max_color)
     {
        delta_r = -color_step;
        r = max_color-1;
     }
     if (r >= min_color && g >= max_color)
     {
        delta_g = -color_step;
        g = max_color-1;
     }
     if (g == min_color && b >= max_color)
     {
        delta_b = -color_step;
        b = max_color-1;
     }

     if (r==min_color && g==min_color && b==min_color)
     {
        r = min_color+1; g=min_color+1; b=min_color+1;
        delta_r = delta_g = delta_b = color_step;
     }
   }

   rc.left = 0;
   rc.top = 0;
   rc.right = 280;
   rc.bottom =200;

#ifdef CREATE_FILES
   #ifdef CREATE_SWAPFILES
      unlink("color_sw.vpe");
      VpeWriteDoc(hDoc, "color_sw.vpe");
   #else
      unlink("colors.vpe");
      VpeWriteDoc(hDoc, "colors.vpe");
   #endif
#endif

#ifdef _WINDOWS
   VpePreviewDoc(hDoc, &rc, VPE_SHOW_NORMAL);
   ExportDocument(hDoc, _T("Colors.pdf"));
#else
   ExportDocument(hDoc, _T("Colors.pdf"));
   VpeCloseDoc(hDoc);
#endif
}



// ========================================================================
//                            SpeedTest
// ========================================================================

#define RPT_FILE        "journal.rpt"


// ========================================================================
//                            Globals
// ========================================================================
double sum_amount=0, sum_prorated=0, sum_tax=0;


// ========================================================================
//                              PrintJournal
//
// Structure of input-file
// =======================
// Document-Name
// Year
// <@>Table-Name --> start a new table!!!
// No.
// Date
// Amount
// Prorated Amount
// Tax
// Remark
// Remark
//
// ========================================================================

VpeCoord headline_height = 0;
VpeCoord group_title_height = 0;
VpeCoord detail_height = 0;


// ========================================================================
//                              GenerateReport
//
// A bit more complex, because it generates date-formats depending on
// the setting of ShortDateFormat in the control panel, but always using
// 4 digits for the year ("Y2K-compliance").
// ========================================================================
void GenerateReport()
{
   FILE *fh;
   size_t i, z;
   int stepper, min, position = 0;
   long count = 0, table = 1;
   struct tm dt;
   TCHAR dir[4096];

   srand((int)clock());
#ifdef _WINDOWS
   sprintf(dir, _T("%s\\%s"), GetTmpDir(), RPT_FILE);
#else
   sprintf(dir, _T("%s/%s"), GetTmpDir(), RPT_FILE);
#endif
   fh = fopen(dir, "wt");
   if (!fh)
      return;
   fprintf(fh, "Test-Document\n");
   fprintf(fh, "2008\n");
   z = 0;
   
   while (count < 20000L)
   {
      if (z++ < (size_t)(rand() % 5 + 10))
      {
         stepper = 4;
         min = 3;
      }
      else
      {
         stepper = 21;
         min = 20;
         z = 0;
      }
      
      fprintf(fh, "@Table %ld\n", table++);
      for (i=rand() % stepper + min; i > 0; i--)
      {
         dt.tm_mday = rand() % 28 + 1;
         dt.tm_mon = rand() % 12;
         dt.tm_year = rand() % 4 + 105;

         fprintf(fh, "%ld\n", count);
         strftime(dir, sizeof(dir), "%x", &dt);
         fprintf(fh, "%s\n", dir);
         fprintf(fh, "%d.%02d\n", rand() % 99999, rand() % 100);
         fprintf(fh, "%d.%02d\n", rand() % 99999, rand() % 100);
         fprintf(fh, "%d.%02d\n\n\n", rand() % 99999, rand() % 100);
         count += 7;
      }
   }

   fclose(fh);
   Msg("Pseudo report-file for 'Speed + Tables' demo generated.");
}



// ========================================================================
//          Start of encapsulating functions for Report-Layout
// ========================================================================

// ========================================================================
//                              PrintHeader
// ========================================================================
void PrintHeader(VpeHandle hDoc, char *table)
{
   VpeSetTextAlignment(hDoc, ALIGN_CENTER);
   VpeSetFont(hDoc, HELVETICA, 14);
   VpeSetBkgColor(hDoc, COLOR_LTGRAY);
   if (headline_height == 0)
   {
      // determine height of headline only once for better performance:
      VpeRenderPrintBox(hDoc, 0, 0, "x");
      headline_height = -VpeGet(hDoc, VRENDERHEIGHT);
   }
   VpeWriteBox(hDoc, VLEFTMARGIN, VBOTTOM, VRIGHTMARGIN, headline_height, table);

   VpeSetBold(hDoc, TRUE);
   VpeSetFont(hDoc, HELVETICA, 11);
   if (group_title_height == 0)
   {
      // determine height of titles only once for better performance:
      VpeRenderPrintBox(hDoc, 0, 0, "x");
      group_title_height = -VpeGet(hDoc, VRENDERHEIGHT);
   }
   VpeWriteBox(hDoc, VLEFTMARGIN, VBOTTOM, -2.00, group_title_height, "No.");
   VpeWriteBox(hDoc, VRIGHT, VTOP, -2.20, VBOTTOM, "Date");
   VpeWriteBox(hDoc, VRIGHT, VTOP, -2.50, VBOTTOM, "Amount");
   VpeWriteBox(hDoc, VRIGHT, VTOP, -2.50, VBOTTOM, "Prorated");
   VpeWriteBox(hDoc, VRIGHT, VTOP, -2.50, VBOTTOM, "Tax");
   VpeWriteBox(hDoc, VRIGHT, VTOP, VRIGHTMARGIN, VBOTTOM, "Remark");
   VpeSetFontAttr(hDoc, ALIGN_LEFT, 0, 0, 0, 0);
   VpeSetBkgColor(hDoc, COLOR_WHITE);
}



// ========================================================================
//                              PrintFooter
// ========================================================================
void PrintFooter(VpeHandle hDoc)
{
   char s[20];
 
   VpeSetFontAttr(hDoc, ALIGN_CENTER, 1, 0, 0, 0);
   VpeSetBkgColor(hDoc, COLOR_LTGRAY);
   VpeWriteBox(hDoc, VLEFTMARGIN, VBOTTOM, -4.20, group_title_height, "Sum");
   VpeSetTextAlignment(hDoc, ALIGN_RIGHT);
   double2NumberString(sum_amount, s);
   VpeWriteBox(hDoc, VRIGHT, VTOP, -2.50, VBOTTOM, s);
   double2NumberString(sum_prorated, s);
   VpeWriteBox(hDoc, VRIGHT, VTOP, -2.50, VBOTTOM, s);
   double2NumberString(sum_tax, s);
   VpeWriteBox(hDoc, VRIGHT, VTOP, -2.50, VBOTTOM, s);
   VpeBox(hDoc, VRIGHT, VTOP, VRIGHTMARGIN, VBOTTOM);
   VpeSetFontAttr(hDoc, ALIGN_LEFT, 0, 0, 0, 0);
   VpeSetBkgColor(hDoc, COLOR_WHITE);
}



// ========================================================================
//                              PrintPageFooter
// ========================================================================
void PrintPageFooter(VpeHandle hDoc, char *name)
{
   VpeStorePos(hDoc);
   VpeWrite(hDoc, VLEFTMARGIN, VBOTTOMMARGIN, VpeGet(hDoc, VRIGHTMARGIN) - 4.00, detail_height, name);
   VpeRestorePos(hDoc);
}



// ========================================================================
//                              PrintJournal
// ========================================================================
bool PrintJournal()
{
   VpeHandle hDoc;
   FILE *fh;
   char s[256], buf[514];
   char name[80], year[8], table[128];
   int  footer_ok = FALSE, read_ahead = FALSE, page = 0;
 
   TCHAR dir[4096];
#ifdef _WINDOWS
   sprintf(dir, _T("%s\\%s"), GetTmpDir(), RPT_FILE);
#else
   sprintf(dir, _T("%s/%s"), GetTmpDir(), RPT_FILE);
#endif
   if ((fh=fopen(dir, "rt")) == NULL)
   {
      GenerateReport();
      if ((fh=fopen(dir, "rt")) == NULL)
      {
         Msg("ERROR: Report-file could not be generated!");
         return false;
      }
   }
 
 #ifdef CREATE_SWAPFILES
   unlink("speed_sf.vpe");
   hDoc = VpeOpenDocFile(hMainWindow, "speed_sf.vpe", "Speed + Tables",  VPE_GRIDBUTTON);
 #else
   hDoc = VpeOpenDoc(hMainWindow, "Speed + Tables",  VPE_GRIDBUTTON);
 #endif
   if (!hDoc)
   {
      Msg("The Document could not be created!");
      return false;
   }

#ifdef _WINDOWS
   VpeSetGUITheme(hDoc, Theme);
   VpeSetRulersMeasure(hDoc, Units);
   VpeSetDevJobName(hDoc, "My own Job Name");
   VpeOpenProgressBar(hDoc);
   VpePreviewDoc(hDoc, NULL, VPE_SHOW_MAXIMIZED);
#endif

   Speed = hDoc;

   // Set the bottom margin, so the report will fit
   // onto A4 as well as onto US-Letter paper:
   // =============================================
   VpeSetOutRect(hDoc, 2.00, 2.00, 19.00, 26.50);

   VpeSetAutoBreak(hDoc, AUTO_BREAK_NO_LIMITS);
   VpeSetBkgMode(hDoc, VBKG_SOLID);


   // ===================
   // Read Report-Header:
   // ===================
   xfgets(name, sizeof(name), fh);
   xfgets(year, sizeof(year), fh);

   sprintf(s, "Journal %s", year);
   VpeNoPen(hDoc);
   VpeSetFontAttr(hDoc, ALIGN_CENTER, 0, 1, 0, 0);
   VpeSetFont(hDoc, HELVETICA, 16);
   VpeWriteBox(hDoc, VLEFTMARGIN, VTOPMARGIN, VRIGHTMARGIN, VFREE, s);

   VpeSetPenSize(hDoc, 0.03);
   VpeSetFontAttr(hDoc, ALIGN_LEFT, 0, 0, 0, 0);
   VpeSetFont(hDoc, HELVETICA, 11);
   if (detail_height == 0)
   {
      // determine height of one detail line only once for better performance:
      // =====================================================================
      VpeRenderPrintBox(hDoc, 0, 0, "x");
      detail_height = -VpeGet(hDoc, VRENDERHEIGHT);
   }

   // Process variable tables:
   // ========================
   while (!feof(fh))
   {
#ifdef _WINDOWS
      // The user is browsing this report while we are generating it.
      // Call VpeDispatchAllMessages() to hold the Preview "alive".
      // ============================================================
      if (VpeDispatchAllMessages(hDoc))
      {
         // The Preview or the Parent-Window was closed by the user,
         // therefore exit this function immediately:
         // ========================================================
         fclose(fh);
         return false;
      }
#endif

      // Because you will normally process a database and therefore know,
      // how many rows of a table you need to print, the following does
      // not apply to you.
      //
      // But because we are reading a serial text file in this example,
      // we need to do a few tricky things - don't care about.
      //
      // This is, what we do here:
      // When a page break happened, we read ahead the serial file
      // to see if data is still available. (see bottom of this function)
      // Therefore we will only read a line from the serial file now,
      // if it was not read ahead:
      // ================================================================
      if (!read_ahead)
      {
         if (!xfgets(s, sizeof(s), fh))
            break;
      }
      read_ahead = FALSE;

      if (*s == '@')
      {
         // This is the beginning of a new table.
         // Print the Group-Footer for the previously printed table:
         // ========================================================
         if (footer_ok)
            PrintFooter(hDoc);  // Don't print a footer before any data from a table was printed

         // Reset sums:
         // ===========
         sum_amount = sum_prorated = sum_tax = 0;

         // Is the room to the page bottom large enough for a new table ?
         // To print a new table on the same page, we need the following space available:
         // 1 cm distance to the previous table, PLUS the Group-Title height, PLUS
         // the Group-Header height PLUS the Group-Footer height PLUS the height for
         // at least 2 detail-lines PLUS the height for another detail-line as gap to
         // the page footer. Check, if enough space is available:
         // We use negative signs, because we had set detail_height and group_title_height
         // to negative values.
         // ==============================================================================
         if (VpeGet(hDoc, VBOTTOMMARGIN) - VpeGet(hDoc, VBOTTOM) <
             -(-1.00 + headline_height + 2 * group_title_height + 3 * detail_height))
         {
            // No, add a new page:
            // ===================
            PrintPageFooter(hDoc, name);
            VpePageBreak(hDoc);
            page++;
         }
         else
         {
            // Yes, there is enough room, the beginning of the new table
            // shall be 1 cm below the previous table, so "forge" the
            // VBOTTOM coordinate.
            //
            // We only need this gap, if a previous table was printed
            // before on the same page. Check this condition by
            // comparing VBOTTOM with VTOP.
            // (VBOTTOM is always bigger than VTOP, except on a newly
            // generated page, where they are set equal by VPE.)
            // =========================================================
            if (VpeGet(hDoc, VBOTTOM) > VpeGet(hDoc, VTOP))
               VpeSet(hDoc, VBOTTOM, VpeGet(hDoc, VBOTTOM) + 1.00);
         }
         strcpy(table, s+1);

         // Print the Group-Header for the new table:
         // =========================================
         PrintHeader(hDoc, table);
         footer_ok = FALSE;
      }
      else
      {
         // Detail part:
         // ============
         footer_ok = TRUE;
         VpeSetBkgMode(hDoc, VBKG_TRANSPARENT);
         VpeSetTextAlignment(hDoc, ALIGN_RIGHT);

         // No.
         VpeWriteBox(hDoc, VLEFTMARGIN, VBOTTOM, -2.00, detail_height, s);

         // Date
         xfgets(s, sizeof(s), fh);
         VpeWriteBox(hDoc, VRIGHT, VTOP, -2.20, VBOTTOM, s);

         // Amount
         xfgets(s, sizeof(s), fh);
         sum_amount += atof(s);
         NumberString(s);
         VpeWriteBox(hDoc, VRIGHT, VTOP, -2.50, VBOTTOM, s);

         // Prorated
         xfgets(s, sizeof(s), fh);
         sum_prorated += atof(s);
         NumberString(s);
         VpeWriteBox(hDoc, VRIGHT, VTOP, -2.50, VBOTTOM, s);

         // Tax
         xfgets(s, sizeof(s), fh);
         sum_tax += atof(s);
         NumberString(s);
         VpeWriteBox(hDoc, VRIGHT, VTOP, -2.50, VBOTTOM, s);

         // Remarks
         xfgets(s, sizeof(s), fh);
         strcpy(buf, s);
         strcat(buf, " ");
         xfgets(s, sizeof(s), fh);
         strcat(buf, s);
         VpeSetTextAlignment(hDoc, ALIGN_LEFT);
         VpeSetFont(hDoc, HELVETICA, 6);
         VpeWriteBox(hDoc, VRIGHT, VTOP, VRIGHTMARGIN, VBOTTOM, buf);
         VpeSetFont(hDoc, HELVETICA, 11);
         VpeSetBkgMode(hDoc, VBKG_SOLID);

         // To print another row from the current table we need the space for one detail-line
         // PLUS the Group-Footer height PLUS the height for another detail-line as gap
         // to the page footer. Check, if enough space is available:
         // We use subtraction, because we had set detail_height and group_title_height
         // to negative values.
         // ==================================================================================
         if (VpeGet(hDoc, VBOTTOM) - 2 * detail_height - group_title_height > VpeGet(hDoc, VBOTTOMMARGIN))
         {
            // No, not enough space, bottom of page is reached:
            // ================================================
            footer_ok = FALSE;
            PrintFooter(hDoc);
            PrintPageFooter(hDoc, name);

            // Read ahead from the data source, to see
            // if there is still data to print:
            // =======================================
            if (!feof(fh) && xfgets(s, sizeof(s), fh))
            {
               read_ahead = TRUE;
               
               // Yes, there is still data to print:
               // ==================================
               VpePageBreak(hDoc);
               page++;

               // Print the Group-Header again, if data
               // from the current table is still available:
               // ==========================================
               if (*s != '@')
                  PrintHeader(hDoc, table);
            }
#ifdef _WINDOWS
            VpeSetProgressBar(hDoc, MulDiv(page, 100, 120));    // we know, that about 120 pages equal 100 %
#endif
         }
      } // else
   } // while
 
   fclose(fh);
   
   if (footer_ok)
   {
      PrintFooter(hDoc);
      PrintPageFooter(hDoc, name);
   }

   // Insert total number of generated pages on first page:
   // ===================================================== 
   VpeGotoPage(hDoc, 1);
   
   wsprintf(s, "[N BC LtRed B]Generated %d page report from serial text file as data source!", VpeGetPageCount(hDoc));
   VpePrintBox(hDoc, 2.00, 1.00, s);

   // Insert page numbers in the style "Page n of <total>":
   // =====================================================
   VpeSetTextAlignment(hDoc, ALIGN_RIGHT);
   VpeSetItalic(hDoc, TRUE);
   for (page = 1; page <= VpeGetPageCount(hDoc); page++)
   {
      VpeGotoPage(hDoc, page);
      sprintf(buf, "Journal Page %d of %d", page, VpeGetPageCount(hDoc));
      VpeWrite(hDoc, VpeGet(hDoc, VRIGHTMARGIN) - 5.00, VBOTTOMMARGIN, VRIGHTMARGIN, detail_height, buf);
   }

#ifdef _WINDOWS
   VpeRefreshDoc(hDoc);
   VpeCloseProgressBar(hDoc);
#endif

#ifdef CREATE_FILES
   #ifdef CREATE_SWAPFILES
      unlink("speed_sw.vpe");
      VpeWriteDoc(hDoc, "speed_sw.vpe");
   #else
      unlink("speed.vpe");
      VpeWriteDoc(hDoc, "speed.vpe");
   #endif
#endif

   ExportDocument(hDoc, _T("Speed and Tables.pdf"));

#ifndef _WINDOWS
   VpeCloseDoc(hDoc);
#endif

   return true;
}



// ===============================================================
//							Color Table
// ===============================================================

typedef struct
{
   COLORREF color;
   char    *name;
} COLORTABLE;

COLORTABLE ColorTable[] =
{
   COLOR_BLACK        , "COLOR_BLACK",
   COLOR_DKGRAY       , "COLOR_DKGRAY",
   COLOR_GRAY         , "COLOR_GRAY",
   COLOR_LTGRAY       , "COLOR_LTGRAY",
   COLOR_WHITE        , "COLOR_WHITE",
                      
   COLOR_DKRED        , "COLOR_DKRED",
   COLOR_RED          , "COLOR_RED",
   COLOR_LTRED        , "COLOR_LTRED",
                      
   COLOR_DKORANGE     , "COLOR_DKORANGE",
   COLOR_ORANGE       , "COLOR_ORANGE",
   COLOR_LTORANGE     , "COLOR_LTORANGE",
                      
   COLOR_DKYELLOW     , "COLOR_DKYELLOW",
   COLOR_YELLOW       , "COLOR_YELLOW",
   COLOR_LTYELLOW     , "COLOR_LTYELLOW",
                      
   COLOR_DKGREEN      , "COLOR_DKGREEN",
   COLOR_GREEN        , "COLOR_GREEN",
   COLOR_LTGREEN      , "COLOR_LTGREEN",
   COLOR_HIGREEN      , "COLOR_HIGREEN",
   COLOR_BLUEGREEN    , "COLOR_BLUEGREEN",
   COLOR_OLIVE        , "COLOR_OLIVE",
   COLOR_BROWN        , "COLOR_BROWN",
                      
   COLOR_DKBLUE       , "COLOR_DKBLUE",
   COLOR_BLUE         , "COLOR_BLUE",
   COLOR_LTBLUE       , "COLOR_LTBLUE",
   COLOR_LTLTBLUE     , "COLOR_LTLTBLUE",
   COLOR_HIBLUE       , "COLOR_HIBLUE",
   COLOR_CYAN         , "COLOR_CYAN",
                      
   COLOR_DKPURPLE     , "COLOR_DKPURPLE",
   COLOR_PURPLE       , "COLOR_PURPLE",
   COLOR_MAGENTA      , "COLOR_MAGENTA",
};


void color_table()
{
   #define CWIDTH       3.00
   #define CHEIGHT      3.00
   #define CSTARTX      1.00

   int n;
   double x = CSTARTX, y = 2.50;

 #ifdef CREATE_SWAPFILES
   unlink("coltb_sw.vpe");
   VpeHandle hDoc = VpeOpenDocFile(hMainWindow, "coltb_sw.vpe", "Color Table", VPE_NO_PAGESCROLLER);
 #else
   VpeHandle hDoc = VpeOpenDoc(hMainWindow, "Color Table", VPE_NO_PAGESCROLLER);
 #endif
   if (!hDoc)
   {
      Msg("The Document could not be created!");
      return;
   }

#ifdef _WINDOWS
   VpeSetGUITheme(hDoc, Theme);
   VpeSetRulersMeasure(hDoc, Units);
#endif
   ColorTableGrad = hDoc;

   VpeWrite(hDoc, VLEFTMARGIN, 1.00, VRIGHTMARGIN, VFREE, "[N S 20 U CE]Predefined Colors");
   VpeSet(hDoc, VRIGHTMARGIN, 21.00);     // Avoid VpePrint() to clip at the right margin
   
   for (n = 0; n < sizeof(ColorTable) / sizeof(COLORTABLE); n++)
   {
      VpeSetBkgMode(hDoc, VBKG_SOLID);
      VpeSetBkgColor(hDoc, ColorTable[n].color);
      VpeSetPenSize(hDoc, 0.03);
      VpeSetPenColor(hDoc, COLOR_BLACK);
      VpeBox(hDoc, x, y, -CWIDTH, -CHEIGHT);
      
      VpeSetBkgMode(hDoc, VBKG_TRANSPARENT);
      VpeSetPenColor(hDoc, COLOR_WHITE);
      
      // Use PrintBox() with invisible Pen, because of trouble
      // with some HP drivers (see manual / help):
      VpePrintBox(hDoc, VLEFT, nBottom(hDoc) + 0.05, ColorTable[n].name);
      
      x += CWIDTH + 1.00;
      if (x + CWIDTH > 20.00)
      {
         x = CSTARTX;
         y += CHEIGHT + 1.00;
      }
   }

#ifdef CREATE_FILES
   #ifdef CREATE_SWAPFILES
      unlink("coltb_sf.vpe");
      VpeWriteDoc(hDoc, "coltb_sf.vpe");
   #else
      unlink("coltab.vpe");
      VpeWriteDoc(hDoc, "coltab.vpe");
   #endif
#endif

#ifdef _WINDOWS
   VpePreviewDoc(hDoc, NULL, VPE_SHOW_MAXIMIZED);
   ExportDocument(hDoc, _T("Color Table.pdf"));
#else
   ExportDocument(hDoc, _T("Color Table.pdf"));
   VpeCloseDoc(hDoc);
#endif
}



#ifdef _WINDOWS
				// ===============================================================
				//					Windows GUI Specific Stuff
				// ===============================================================

// ===============================================================
//							UnitsMsg
// ===============================================================
void UnitsMsg()
{
	static bool msg_shown = false;
	
	if (!msg_shown)
	{
		Msg("The selected units for the rulers will be applied to the next newly created document.\n\n"
			"The units display of the rulers is independent of the internal coordinate system,\n"
			"which can be either centimeters or inches.");
		msg_shown = true;
	}
}


// ===============================================================
//							ThemeMsg
// ===============================================================
void ThemeMsg()
{
	static bool msg_shown = false;
	
	if (!msg_shown)
	{
		Msg("The selected theme will be applied to the next newly created document.\n\n"
			"VPE can also assign a theme to an existing document, but we would like\n"
			"to demonstrate that VPE can handle multiple themes simultaneously.\n\n"
			"The Whidbey and the Office 2003 themes will only differ on Windows XP,\n"
			"because VPE uses the color theme of Win XP.\n\n"
			"For the Community Edition the theme can not be changed.");
		msg_shown = true;
	}
}


// ===============================================================
//                     DrawGradientRect
// ===============================================================
typedef struct
{
   LONG red, green, blue, deltared, deltagreen, deltablue;
   int bandcount;
} GRADIENT_SCHEME;

#define STEPDIVISOR 1024
#define GRADIENTDIVISOR 1024

void DrawGradientRect(HDC hdc, RECT &rectClient)
{
   RECT rectFill;
   HBRUSH hBrush;
   int fStep, iOnBand;
   GRADIENT_SCHEME scheme = { 0, 0, 261120,  0,  0, -1024, 255 };

   fStep = MulDiv(rectClient.bottom - rectClient.top, STEPDIVISOR, scheme.bandcount + 1);
   for (iOnBand = 0; iOnBand < scheme.bandcount + 1; iOnBand++) 
   {
         // Set the location of the current band
         SetRect(&rectFill,
               rectClient.left,                                           // Upper left X
               MulDiv(iOnBand, fStep, STEPDIVISOR) + rectClient.top,      // Upper left Y
               rectClient.right,                                          // Lower right X
               MulDiv(iOnBand + 1, fStep, STEPDIVISOR) + rectClient.top); // Lower right Y

         // Create a brush with the appropriate color for this band
         hBrush = CreateSolidBrush(RGB(scheme.red / GRADIENTDIVISOR, scheme.green / GRADIENTDIVISOR, scheme.blue / GRADIENTDIVISOR));
      scheme.red   += scheme.deltared;
      scheme.green += scheme.deltagreen;
      scheme.blue  += scheme.deltablue;
      FillRect(hdc, &rectFill, hBrush);
      DeleteObject(hBrush);
   }
}




// ========================================================================
//                              Paint
// ========================================================================
HFONT create_font(char *name, int pt_size, BYTE italic)
{
   return CreateFont(pt_size, 0,
               0, 0,
               FW_DONTCARE,     // bold ? FW_BOLD : FW_DONTCARE,
               italic,
               0,               // underlined,
               0, 0, 0, 0, 0, 0, name);
}


void Paint(HWND hwnd)
{
   PAINTSTRUCT ps;
   HDC hdc;
   RECT rectClient;
   HFONT hfont, holdfont;
   char *s, *SUBTITLE;
   int w, h;
   char Product[80];

   wsprintf(Product, PRODUCT, VpeGetVersion() >> 8, VpeGetVersion() & 0xff);
   
   hdc = BeginPaint(hwnd, &ps);
   GetClientRect(hwnd, &rectClient);
   DrawGradientRect(hdc, rectClient);

   SetTextColor(hdc, 0xffffff);
   SetBkMode(hdc, TRANSPARENT);

   hfont = create_font(TIMES, -36, 1);
   holdfont = SelectFont(hdc, hfont);
   SetTextColor(hdc, 0);
   TextOut(hdc, 17, 17, Product, (int)strlen(Product));
   SetTextColor(hdc, 0xffffff);
   TextOut(hdc, 15, 15, Product, (int)strlen(Product));
   SelectFont(hdc, holdfont);
   DeleteFont(hfont);

   switch (VpeGetEdition())
   {
    case VEDITION_COMMUNITY: SUBTITLE = "Community Edition"; break;
    case VEDITION_STANDARD: SUBTITLE = "Standard Edition"; break;
    case VEDITION_ENHANCED: SUBTITLE = "Enhanced Edition"; break;
    case VEDITION_PROFESSIONAL: SUBTITLE = "Professional Edition"; break;
    case VEDITION_ENTERPRISE: SUBTITLE = "Enterprise Edition"; break;
    case VEDITION_INTERACTIVE: SUBTITLE = "Interactive Edition"; break;
   }

   hfont = create_font(HELVETICA, -18, 0);
   holdfont = SelectFont(hdc, hfont);
   SetTextColor(hdc, 0);
   TextOut(hdc, 16, 61, SUBTITLE, (int)strlen(SUBTITLE));
   SetTextColor(hdc, 0xffffff);
   TextOut(hdc, 15, 60, SUBTITLE, (int)strlen(SUBTITLE));
   SelectFont(hdc, holdfont);
   DeleteFont(hfont);
         

   hfont = create_font(HELVETICA, -18, 0);
   holdfont = SelectFont(hdc, hfont);
   s = "Copyright  2008 IDEAL Software";
 #if defined(__WIN32__) || defined(WIN32) || defined(__NT__)
      SIZE p;
      GetTextExtentPoint32(hdc, s, (int)strlen(s), &p);
      w = p.cx;
      h = p.cy;
 #else
      DWORD wh = GetTextExtent(hdc, s, strlen(s));
      w = LOWORD(wh);
      h = HIWORD(wh);
 #endif
   TextOut(hdc, 15, rectClient.bottom - 3*h, s, (int)strlen(s));
   s = "";
   TextOut(hdc, 15+w, rectClient.bottom - 3*h - h/3, s, (int)strlen(s));
   s = "All rights reserved.";
   TextOut(hdc, 15, rectClient.bottom - 2*h, s, (int)strlen(s));
   SelectFont(hdc, holdfont);
   DeleteFont(hfont);

   EndPaint(hwnd, &ps);
}


// ========================================================================
//                              WndProc
// ========================================================================
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   static HINSTANCE  hInstance;
   
   switch (message)
   {
    case WM_CREATE:
       hMainWindow = hwnd;
       hInstance = ((LPCREATESTRUCT) lParam)->hInstance;
       return 0;


    case WM_PAINT:
       if (!Precision || VpeGetEdition() == VEDITION_COMMUNITY)	// for the Community Edition the Preview can not be embedded
       {
          Paint(hwnd);
          return 0;
       }
       break;


    case WM_COMMAND:
       switch (wParam)
       {
        case CM_GENREPORT:
           GenerateReport();
           break;
 
        case CM_DELREPORT:
           {
              TCHAR dir[MAX_PATH];
              wsprintf(dir, _T("%s\\%s"), GetTmpDir(), RPT_FILE);
              remove(dir);
           }
           break;

        case CM_WELCOME:
           welcome();
           EnableMenuItem(hMenu, CM_WELCOME, MF_GRAYED);
           break;
           
        case CM_PRECISION:
           precision();
           EnableMenuItem(hMenu, CM_PRECISION, MF_GRAYED);
           break;

        case CM_BACKGROUND:
           if (VpeGetEdition() == VEDITION_COMMUNITY)
              Msg("This is not possible with the Community Edition.");
           else
           {
              EnableMenuItem(hMenu, CM_BACKGROUND, MF_GRAYED);
              precision(1);
           }
           break;

        case CM_SPEED:
           EnableMenuItem(hMenu, CM_SPEED, MF_GRAYED);
           PrintJournal();
           break;

        case CM_REPORT:
           reporttest(0);
           EnableMenuItem(hMenu, CM_REPORT, MF_GRAYED);
           break;

        case CM_REP_EMAIL:
           if (VpeGetEdition() == VEDITION_COMMUNITY)
              Msg("This is not possible with the Community Edition.");
		   else
              reporttest(1);
           break;

        case CM_REP_EMAIL_HIDE:
           if (VpeGetEdition() == VEDITION_COMMUNITY)
              Msg("This is not possible with the Community Edition.");
		   else
              reporttest(2);
           break;

        case CM_COLORS:
           colortest();
           EnableMenuItem(hMenu, CM_COLORS, MF_GRAYED);
           EnableMenuItem(hMenu, CM_CLOSECOLORS, MF_ENABLED);
           break;

        case CM_CLOSECOLORS:
           VpeCloseDoc(Colors);
           break;

        case CM_PREDEFINEDCOLORS:
           color_table();
           EnableMenuItem(hMenu, CM_PREDEFINEDCOLORS, MF_GRAYED);
           break;

        case CM_EXIT:
           PostMessage(hwnd, WM_CLOSE, 0, 0);
           break;

        case CM_OFFICE2000:
           ThemeMsg();
           Theme = VGUI_THEME_OFFICE2000;
           CheckMenuRadioItem(hMenu, CM_OFFICE2000, CM_WHIDBEY, CM_OFFICE2000, MF_BYCOMMAND);
           break;

        case CM_OFFICE2003:
           ThemeMsg();
           Theme = VGUI_THEME_OFFICE2003;
           CheckMenuRadioItem(hMenu, CM_OFFICE2000, CM_WHIDBEY, CM_OFFICE2003, MF_BYCOMMAND);
           break;

        case CM_WHIDBEY:
           ThemeMsg();
           Theme = VGUI_THEME_WHIDBEY;
           CheckMenuRadioItem(hMenu, CM_OFFICE2000, CM_WHIDBEY, CM_WHIDBEY, MF_BYCOMMAND);
           break;

        case CM_CENTIMETERS:
           UnitsMsg();
           Units = CM;
           CheckMenuRadioItem(hMenu, CM_CENTIMETERS, CM_INCH, CM_CENTIMETERS, MF_BYCOMMAND);
           break;

        case CM_INCH:
           UnitsMsg();
           Units = INCH;
           CheckMenuRadioItem(hMenu, CM_CENTIMETERS, CM_INCH, CM_INCH, MF_BYCOMMAND);
           break;
       }
       return 0;


    case WM_SIZE:
       if (Precision)   // this is an embedded window
       {
           if (VpeGetEdition() != VEDITION_COMMUNITY)	// for the Community Edition the Preview can not be embedded
           {
              MoveWindow(VpeGetWindowHandle(Precision), 0, 0, LOWORD(lParam), HIWORD(lParam), TRUE);
           }
       }
       return 0;


    case WM_SETFOCUS:
       if (Precision)
       {
           if (VpeGetEdition() != VEDITION_COMMUNITY)	// for the Community Edition the Preview can not be embedded
           {
              // this is an embedded window, forward the focus to the preview, so
              // it receives keyboard and mouse-wheel messages
              SetFocus(VpeGetWindowHandle(Precision));
           }
       }
       return 0;


    case WM_CLOSE:
       if (!VpeCloseDoc(Welcome) ||     // VpeCloseDoc() allows that the hDoc parameter is NULL
           !VpeCloseDoc(Precision) ||
           !VpeCloseDoc(PBackGnd) ||
           !VpeCloseDoc(Speed) ||
           !VpeCloseDoc(Colors) ||
           !VpeCloseDoc(Report) ||
           !VpeCloseDoc(ColorTableGrad))
       {
          Msg("The application cannot terminate until all jobs have finished printing.");
          return 0;
       }
       DestroyWindow(hwnd);
       return 1;
              
           
    case WM_DESTROY:
       PostQuitMessage(0);
       return 0;
   }


   // Check messages for VPE document:
   // ================================
   switch (VpeMapMessage(NULL, message))        // we use NULL, because all documents are
   {                                             // created with dynamic message mapping, i.e. VPE_FIXED_MESSAGES is not used
    case VPE_DESTROYWINDOW:
       if (Welcome == (VpeHandle)lParam)
       {
          Welcome = 0;
          EnableMenuItem(hMenu, CM_WELCOME, MF_ENABLED);
       }
       else if (Precision == (VpeHandle)lParam)
       {
          Precision = 0;
          EnableMenuItem(hMenu, CM_PRECISION, MF_ENABLED);
       }
       else if (PBackGnd == (VpeHandle)lParam)
       {
          PBackGnd = 0;
          EnableMenuItem(hMenu, CM_BACKGROUND, MF_ENABLED);
       }
       else if (Speed == (VpeHandle)lParam)
       {
          Speed = 0;
          EnableMenuItem(hMenu, CM_SPEED, MF_ENABLED);
       }
       else if (Colors == (VpeHandle)lParam)
       {
          Colors = 0;
          EnableMenuItem(hMenu, CM_COLORS, MF_ENABLED);
          EnableMenuItem(hMenu, CM_CLOSECOLORS, MF_GRAYED);
       }
       else if (Report == (VpeHandle)lParam)
       {
          Report = 0;
          EnableMenuItem(hMenu, CM_REPORT, MF_ENABLED);
       }
       else if (ColorTableGrad == (VpeHandle)lParam)
       {
          ColorTableGrad = 0;
          EnableMenuItem(hMenu, CM_PREDEFINEDCOLORS, MF_ENABLED);
       }
       break;

                            
    case VPE_PRINT:
       switch (wParam)
       {
        case PRINT_MSG_SETUPABORT:  // User aborted Setup-Dialog
           Msg("Caught event from VPE: Setup aborted.");
           break;
           
        case PRINT_MSG_SETUPSTART: // Setup-Dialog started
           Msg("Caught event from VPE: Setup started.\n\nNow you could change some printer properties by code,\nor send ESCAPE-Sequences to the printer.");
           break;
           
        case PRINT_MSG_SETUPEND:   // Setup-Dialog ended
           Msg("Caught event from VPE: Setup successful finished.");
           break;
           
        case PRINT_MSG_START:
           Msg("Caught event from VPE: Print-Job started.");
           break;

       default:
           if (wParam == PRINT_MSG_END)
              Msg("Caught event from VPE: --- Successful End of Print-Job ---");
           else
              Msg("Caught event from VPE: *** Print-Job aborted ***");
           break;
       }
       return 0;


    case VPE_BEFORE_MAIL:
       Msg("Caught event from VPE: --- BEFORE Mail ---\n\nNow you could invisibly add recipients, attachments, text, etc. by code.");
       return 0;

       
    case VPE_HELP:
       MessageBox(hwnd, "User requested help!", "Note:", MB_OK);
       break;
   }

   return DefWindowProc (hwnd, message, wParam, lParam) ;
}


// ========================================================================
//                              Windows WinMain
// ========================================================================
int PASCAL WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPSTR , int nCmdShow)
{
	MSG         msg ;
	HWND        hwnd ;
	WNDCLASS    wndclass ;

	setlocale(LC_ALL, "");
	Locale = *localeconv();

	FRUITS = "..\\images\\FRUITS.BMP";
	IDEALLOGO = "..\\images\\LOGO.BMP";

	if (VpeGetEdition() <= VEDITION_STANDARD)
	{
		// The Standard Edition is delivered with all example images as BMP's,
		// because it is not able to read JPG, TIF, PNG, etc.
		// ===================================================================
		WOOD = "..\\images\\WOOD.BMP";
		SAND = "..\\images\\SAND.BMP";
		BAU = "..\\images\\BAU.BMP";
		SILENCE = "..\\images\\SILENCE.BMP";
		MARBLE = "..\\images\\MARBLE.BMP";
		PROD = "..\\images\\PRODUCT.BMP";
		VPELOGO = "..\\images\\VPELOGO.BMP";
		GEW = "..\\images\\GEW.BMP";
	}
	else
	{
		WOOD = "..\\images\\WOOD.JPG";
		SAND = "..\\images\\SAND.JPG";
		BAU = "..\\images\\BAU.JPG";
		SILENCE = "..\\images\\SILENCE.JPG";
		MARBLE = "..\\images\\MARBLE.JPG";
		PROD = "..\\images\\PRODUCT.PNG";
		VPELOGO = "..\\images\\VPELOGO.PNG";
		if (VpeGetEdition() < VEDITION_PROFESSIONAL)
			GEW = "..\\images\\GEW.PNG";
		else
			GEW = "..\\images\\GEW300G4.TIF";
	}

	if (!hPrevInstance)
	{
		wndclass.style         = CS_HREDRAW | CS_VREDRAW;
		wndclass.lpfnWndProc   = WndProc ;
		wndclass.cbClsExtra    = 0 ;
		wndclass.cbWndExtra    = 0 ;
		wndclass.hInstance     = hInstance ;
		wndclass.hIcon         = LoadIcon (hInstance, "APP_ICON");
		wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
		wndclass.hbrBackground = GetStockBrush (NULL_BRUSH) ;
		wndclass.lpszMenuName  = NULL;
		wndclass.lpszClassName = szAppName;
		RegisterClass(&wndclass) ;
	}

	hMenu = LoadMenu(hInstance, MAKEINTRESOURCE(MENU_1));
	CheckMenuRadioItem(hMenu, CM_OFFICE2000, CM_WHIDBEY, CM_WHIDBEY, MF_BYCOMMAND);

	if (GetDefaultMetricSystem() == 0)
	{
		Units = CM;
		CheckMenuRadioItem(hMenu, CM_CENTIMETERS, CM_INCH, CM_CENTIMETERS, MF_BYCOMMAND);
	}
	else
	{
		Units = INCH;
		CheckMenuRadioItem(hMenu, CM_CENTIMETERS, CM_INCH, CM_INCH, MF_BYCOMMAND);
	}

	hwnd = CreateWindow(szAppName, szAppName,
						WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
						CW_USEDEFAULT, CW_USEDEFAULT,
						CW_USEDEFAULT, CW_USEDEFAULT,
						NULL, hMenu, hInstance, NULL) ;

	ShowWindow (hwnd, nCmdShow) ;
	UpdateWindow (hwnd) ;

	while (GetMessage (&msg, NULL, 0, 0))
	{
		TranslateMessage (&msg) ;
		DispatchMessage (&msg) ;
	}

	return (int)msg.wParam;
}

#else

// ========================================================================
//                              Non-Windows main()
// ========================================================================
int main(int argc, const char **argv)
{
	setlocale(LC_ALL, "");
	Locale = *localeconv();

	FRUITS = "../images/FRUITS.BMP";
	IDEALLOGO = "../images/LOGO.BMP";

	if (VpeGetEdition() <= VEDITION_STANDARD)
	{
		// The Standard Edition is delivered with all example images as BMP's,
		// because it is not able to read JPG, TIF, PNG, etc.
		// ===================================================================
		WOOD = "../images/WOOD.BMP";
		SAND = "../images/SAND.BMP";
		BAU = "../images/BAU.BMP";
		SILENCE = "../images/SILENCE.BMP";
		MARBLE = "../images/MARBLE.BMP";
		PROD = "../images/PRODUCT.BMP";
		VPELOGO = "../images/VPELOGO.BMP";
		GEW = "../images/GEW.BMP";
	}
	else
	{
		WOOD = "../images/WOOD.JPG";
		SAND = "../images/SAND.JPG";
		BAU = "../images/BAU.JPG";
		SILENCE = "../images/SILENCE.JPG";
		MARBLE = "../images/MARBLE.JPG";
		PROD = "../images/PRODUCT.PNG";
		VPELOGO = "../images/VPELOGO.PNG";
		if (VpeGetEdition() < VEDITION_PROFESSIONAL)
			GEW = "../images/GEW.PNG";
		else
			GEW = "../images/GEW300G4.TIF";
	}

	struct stat st_buf;
	if (stat(FRUITS, &st_buf) != 0)
	{
		const char *exe_name = "";
		int i = strlen(argv[0]);
		while (i >= 0 && argv[0][i] != '/')
			i--;
		if (i >= 0)
			exe_name = &argv[0][i + 1];
		printf("\nPlease run this demo from within its original directory,\ni.e. with './%s'\n\n", exe_name);
		return 1;
	}

	welcome();
	precision(1);
	reporttest(0);
	colortest();
	PrintJournal();
	color_table();

	return 0;
}
#endif
