//
//              VPECOMON.H
//              ==========
//
// Common definitions for the "Virtual Print Engine"
//
// Copyright  1993 - 2008 IDEAL Software. All rights reserved.
//


#ifndef _MODULE_VPECOMON
#define _MODULE_VPECOMON

// $ +av skip
// $ +n transform


// ========================================================================
//                      Editions returned by VpeGetEdition():
// ========================================================================
// $ +n enum Edition (VEDITION_) {
#define VEDITION_COMMUNITY       500
#define VEDITION_STANDARD       1000
#define VEDITION_ENHANCED       2000
#define VEDITION_PROFESSIONAL   3000
#define VEDITION_ENTERPRISE     4000
#define VEDITION_INTERACTIVE    5000
// $ }


// ========================================================================
//                      VpeOpenDoc() Flag Parameters:
// ========================================================================
// $ +n enum OpenDocFlag {
#define VPE_NO_TOOLBAR              1L   // Toolbar NOT visible
#define VPE_NO_PRINTBUTTON          8L   // Print-Button invisible, VpePrintDoc() works
#define VPE_NO_MAILBUTTON          16L   // Mail-Button invisible
#define VPE_NO_SCALEBTNS           32L   // No Scale Buttons in Toolbar
#define VPE_GRIDBUTTON             64L   // Grid Toolbar-Button visible
#define VPE_NO_MOVEBTNS           128L   // Move Buttons invisible
#define VPE_NO_HELPBUTTON         256L   // Help-Button invisible
#define VPE_NO_INFOBUTTON         512L   // Info-Button invisible
#define VPE_NO_USER_CLOSE        1024L   // User can't close VPE:
                                         // Close-Button INVISIBLE and Sys-Menu
                                         // disabled (if not embedded)
                                         // VpeCloseDoc() works!

#define VPE_NO_STATBAR           2048L   // Statusbar invisible
#define VPE_NO_PAGESCROLLER      4096L   // Page Scroller in Statusbar invisible
#define VPE_NO_STATUSSEG         8192L   // Status Segment in Statusbar invisible

#define VPE_NO_RULERS           16384L   // Rulers invisible
#define VPE_EMBEDDED            32768L   // Preview is made child window of
                                         // parent window specified in VpeOpenDoc()
#define VPE_DOCFILE_READONLY    65536L   // Document file is opened with read-only permission
#define VPE_FIXED_MESSAGES     131072L   // Messages are not registered by the Windows System

#define VPE_NO_OPEN_BUTTON     262144L   // No File Open Button
#define VPE_NO_SAVE_BUTTON     524288L   // No File Save Button
// $ }


// ========================================================================
//                    VpePreviewDoc() Flag Parameters:
// ========================================================================
// $ +n Type PreviewState (VPE_SHOW_)
enum
{
   VPE_SHOW_NORMAL = 1,
   VPE_SHOW_MAXIMIZED,
   VPE_SHOW_HIDE,
};


// ========================================================================
//                            Background Modes:
// ========================================================================
// $ +n enum BkgMode (VBKG_) {
#define VBKG_SOLID       0      // solid background color
#define VBKG_TRANSPARENT 1      // transparent background
// $ +n subst GradientLine
#define VBKG_GRD_LINE    2      // line gradient background
// $ +n subst GradientRectangle
#define VBKG_GRD_RECT    3      // rectangular gradient background
// $ +n subst GradientEllipse
#define VBKG_GRD_ELLIPSE 4      // elliptic gradient background
// $ }


// ========================================================================
//                            Modes for GradientPrint:
// ========================================================================
// $ +n enum BkgGradientPrint (VGRD_PRINT_) {
#define VGRD_PRINT_AUTO     0   // if printer is a color printer, the gradient
                                // is printed, otherwise the alternative solid color
#define VGRD_PRINT_GRADIENT 1   // the gradient is always printed
#define VGRD_PRINT_SOLID    2   // the alternative solid color is always printed
// $ }


// ========================================================================
//                            Pen Styles:
//
// All PS_xxx styles are defined in windows.h:
// ========================================================================
// $ +n enum PenStyle (PS_) {
// $ #define PS_SOLID            0
// $ #define PS_DASH             1       /* -------  */
// $ #define PS_DOT              2       /* .......  */
// $ +n subst DashDot
// $ #define PS_DASHDOT          3       /* _._._._  */
// $ +n subst DashDotDot
// $ #define PS_DASHDOTDOT       4       /* _.._.._  */
// $ }


// ========================================================================
//                            Hatch Styles:
// ========================================================================
// $ +n enum HatchStyle (HS_)
// $ {
#define HS_NONE         -1
// All other HS_xxx styles are defined in windows.h:
// $ #define HS_HORIZONTAL       0
// $ #define HS_VERTICAL         1
// $ +n subst ForwardDiagonal
// $ #define HS_FDIAGONAL        2
// $ +n subst BackwardDiagonal
// $ #define HS_BDIAGONAL        3
// $ #define HS_CROSS            4
// $ +n subst DiagonalCross
// $ #define HS_DIAGCROSS        5
// $ }


// ========================================================================
//                      Text-Formatting Attributes:
// ========================================================================
// $ +n Type TextAlignment (ALIGN_)
enum
{
    ALIGN_LEFT,
    ALIGN_RIGHT,
    ALIGN_CENTER,
    ALIGN_JUSTIFIED,
    ALIGN_JUSTIFIED_AB = 5,  // Text will be aligned justified. In contrast to ALIGN_JUSTIFIED which aligns
                             // the last line left aligned, the last line of the text will also be aligned
                             // justified - if it is not ending with a CR / LF character.
};


// ========================================================================
//                              Auto-Break Options:
// ========================================================================
// $ +n Type AutoBreakMode (AUTO_BREAK_)
enum
{
    AUTO_BREAK_ON,              // Auto Break on; top and bottom of DefOutRect are used
    AUTO_BREAK_OFF,             // Same behaviour as AUTO_BREAK_ON (limited positioning /
                                // rendering), but no Auto Break performed 
    AUTO_BREAK_NO_LIMITS,       // No Auto Break performed, no limited positioning / rendering
    AUTO_BREAK_FULL,            // Auto Break on; complete DefOutRect is used
};


// ========================================================================
//                            Frame Attributes:
// ========================================================================
// $ skip
// $ Type FrameStyle (FRAME_)
enum
{
    FRAME_INSIDE,       // - ONLY for internal usage: DO NOT USE
    FRAME_OUTSIDE,      // - ONLY for internal usage: DO NOT USE
// $ +n subst OnCenter
    FRAME_ONCENTER,     // - ONLY for internal usage: DO NOT USE
};
// $ endskip


// Flags for VpeSetPictureType()
// =============================
// $ +n enum PictureType (PIC_TYPE_) {
#define PIC_TYPE_AUTO   255     // Auto determination by filename-suffix (default) or - if none present - file-content (the latter is unsafe)
// $ +n endtransform
#define PIC_TYPE_BMP      0     // Windows-Bitmap
#define PIC_TYPE_WMF      5     // WMF
#define PIC_TYPE_EMF      6     // EMF
#define PIC_TYPE_TIFF    64     // TIF/TIFF
#define PIC_TYPE_GIF     65     // GIF
#define PIC_TYPE_PCX     66     // PCX
#define PIC_TYPE_JPEG    68     // JPG/JIF/JPEG/JPE
#define PIC_TYPE_PNG     69     // PNG (Portable Network Graphic)
#define PIC_TYPE_ICO     70     // ICO (Windows Icon)
#define PIC_TYPE_JNG     71     // JNG (JPEG Network Graphics)
#define PIC_TYPE_KOALA   72     // KOA (C64 Koala Graphics)
#define PIC_TYPE_IFF     73     // IFF/LBM (Interchangeable File Format - Amiga/Deluxe Paint)
#define PIC_TYPE_MNG     74     // MNG (Multiple-Image Network Graphics)
#define PIC_TYPE_PBM     75     // PBM (Portable Bitmap [ASCII])
#define PIC_TYPE_PBM_RAW 76     // PBM (Portable Bitmap [RAW])
#define PIC_TYPE_PCD     77     // PCD (Kodak PhotoCD)
#define PIC_TYPE_PGM     78     // PGM (Portable Greymap [ASCII])
#define PIC_TYPE_PGM_RAW 79     // PGM (Portable Greymap [RAW])
#define PIC_TYPE_PPM     80     // PPM (Portable Pixelmap [ASCII])
#define PIC_TYPE_PPM_RAW 81     // PPM (Portable Pixelmap [RAW])
#define PIC_TYPE_RAS     82     // RAS (Sun Raster Image)
#define PIC_TYPE_TARGA   83     // TGA/TARGA (Truevision Targa)
#define PIC_TYPE_WBMP    84     // WAP/WBMP/WBM (Wireless Bitmap)
#define PIC_TYPE_PSD     85     // PSD (Adobe Photoshop)
#define PIC_TYPE_CUT     86     // CUT (Dr. Halo)
#define PIC_TYPE_XBM     87     // XBM (X11 Bitmap Format)
#define PIC_TYPE_XPM     88     // XPM (X11 Pixmap Format)
#define PIC_TYPE_DDS     89     // DDS (DirectX Surface)
#define PIC_TYPE_HDR     90     // HDR (High Dynamic Range Image)
#define PIC_TYPE_FAX_G3  91     // G3  (Raw fax format CCITT G.3)
#define PIC_TYPE_SGI     92     // SGI (SGI Image Format)
// $ +n transform
// $ }


// ========================================================================
//                          Printer- / Setup-Flags:
// ========================================================================

// Flags for VpeSetupPrinter():
// $ +n enum PrintDialogControl (PRINTDLG_) {
#define PRINTDLG_NEVER      0   // never show setup-dialog (if file_name is NULL, last setting 
                                // or the setting of the default-printer will be taken)
// $ +n subst OnFail
#define PRINTDLG_ONFAIL     1   // show setup-dialog only, if file-read fails
#define PRINTDLG_ALWAYS     2   // show setup-dialog always
#define PRINTDLG_FULL       4   // show FULL Dialog (with page range, collation, etc.)
                                // This is a flag, i.e. add it to one of the other PRINTDLG_xyz flags
                                // (example: PRINTDLG_ALWAYS +  PRINTDLG_FULL)
                                // The flag is especially useful, if you hide the Toolbar or the Print-Button.
// $ }

// Flags for VpeSetPrintOptions():
// $ +n enum PrintOptions {
#define PRINT_ALL                0   // print all pages (default)
#define PRINT_EVEN               1   // print only even pages
#define PRINT_ODD                2   // print only odd pages
// $ +n subst NoAbortDialog
#define PRINT_NOABORTDLG         4   // show no abort / progress dialog during printing
// $ exclude
#define PRINT_WAIT_COMPLETION    8   // not supported. wait until print job has been transfered completely to the printer
// $ endexclude
// $ +n subst NoAutoPageDims
#define PRINT_NO_AUTO_PAGE_DIMS 16   // normally, VPE instructs the printer during printing, to switch the paper dimensions
                                     // automatically (some printers will therefore select another input PaperBin) according to
                                     // the page dimensions of the VPE Document's currently printed page. This flag stops VPE from doing so.
// $ }

// Flags for VpeSetPrintPosMode():
// $ +n enum PrintPosMode (PRINTPOS_) {
#define PRINTPOS_ABSOLUTE   0   // coordinates are absolute to the paper-edges (default)
#define PRINTPOS_RELATIVE   1   // coordinates are relative to the first printable positon of the printer
// $ }


// ========================================================================
//                        Notification Messages:
// ========================================================================
// $ +n Type VpeMessages
enum
{
   VPE_DESTROYWINDOW = 3330,
                           // WM_USER + 2306
                           // VPE was destroyed - lParam = Doc-Handle
                           // Possible return codes: your application should return zero when processing this message

   VPE_PRINT,              // printing: wParam = one of the PRINT_MSG_xyz constants
                           //           lParam = Doc-Handle
                           // Possible return codes: PRINT_ACTION_OK or
                           //                        (only for wParam = PRINT_MSG_SETUPSTART) PRINT_ACTION_ABORT
                             
   VPE_AUTOPAGEBREAK,      // Auto Page Break occured. Engine is on the new page. This is either
                           // a newly generated page, or an already existing page (if the text
                           // was inserted on a page, that has already pages following) - lParam = Doc-Handle
                           // Possible return codes: your application should return zero when processing this message
                           
   VPE_PRINT_NEWPAGE,      // Event is sent (only while printing) exactly before printing a new page.
                           // The event is useful to change the device properties on-the-fly during printing.
                           // wParam = page no.
                           // lParam = Doc-Handle
                           // Possible return codes: PRINT_ACTION_OK or PRINT_ACTION_CHANGE
                           
   VPE_PRINT_DEVDATA,      // Event is sent (only while printing) exactly before printing a new page and 
                           // immediately after VPE_PRINT_NEWPAGE has been sent.
                           // The event is useful to call VpeDevSendData() to send Escape sequences and other
                           // binary data directly to the output device during printing.
                           // wParam = page no.
                           // lParam = Doc-Handle
                           // Possible return codes: PRINT_ACTION_OK or PRINT_ACTION_CHANGE
   
   VPE_HELP,               // User requests help - lParam = Doc-Handle
                           // Possible return codes: your application should return zero when processing this message

   VPE_CLOSEWINDOW,        // Preview was closed, but document not destroyed - lParam = Doc-Handle
                           // Possible return codes: your application should return zero when processing this message

   VPE_OBJECTCLICKED,      // An object with an assigned ObjectID has been clicked - lParam = Doc-Handle
                           // retrieve the ObjectID with VpeGetObjectID() [Professional Edition and above]
                           // Possible return codes: your application should return zero when processing this message

   VPE_UDO_PAINT,          // Notification for User Defined Object - lParam = Doc-Handle [Professional Edition and above]
                           // Possible return codes: your application should return zero when processing this message

   VPE_BEFORE_MAIL,        // User pushed eMail-button, but email was not sent yet - lParam = Doc-Handle
                           // Possible return codes: 0 = continue, 1 = cancel, i.e. do not mail the document
                           // Cancelling the operation allows you to display your own mail dialog and/or to
                           // execute your own mailing code.

   VPE_AFTER_MAIL,         // User pushed eMail-button and email was sent - wParam = VERR_xyz, lParam = Doc-Handle
                           // Possible return codes: your application should return zero when processing this message

   VPE_CANCLOSE,           // VPE requests for confirmation if the preview can be closed
                           // lParam = Document Handle
                           // Your application should return: 1 = no, the preview may not be closed; zero otherwise
									
                           // All Control Messages: wParam = Object Handle
                           //                       lParam = Document Handle
   VPE_CTRL_AFTER_ENTER,   // Control received the focus [Interactive Edition only]
                           // Possible return codes: your application should return zero when processing this message

   VPE_CTRL_CAN_EXIT,      // Control requests, if it may loose the focus [Interactive Edition only]
                           // Your Application should return: 1 = no, the control may not loose the focus, zero otherwise

   VPE_CTRL_AFTER_EXIT,    // Control has lost the focus [Interactive Edition only]
                           // Possible return codes: your application should return zero when processing this message

   VPE_CTRL_AFTER_CHANGE,  // Control changed its content [Interactive Edition only]
                           // Possible return codes: your application should return zero when processing this message

   VPE_FIELD_AFTER_CHANGE, // the user changed a Field's value by changing the value of the 
                           // associated control [Interactive Edition only]
                           // wParam = Field Handle
                           // lParam = Document Handle
                           // Possible return codes: your application should return zero when processing this message

   VPE_BEFORE_OPEN_FILE,   // the user clicked the Open File button in the toolbar - lParam = Doc-Handle
                           // Possible return codes: 0 = continue, 1 = cancel, i.e. do not show the open dialog
                           // Cancelling the operation allows you to display your own open dialog and to import or
                           // create your own document, for example from a database via a memory stream.

   VPE_AFTER_OPEN_FILE,    // the file open operation has been completed - wParam = VERR_xyz, lParam = Doc-Handle
                           // Possible return codes: your application should return zero when processing this message

   VPE_BEFORE_SAVE_FILE,   // the user clicked the Save File button in the toolbar - lParam = Doc-Handle
                           // Possible return codes: 0 = continue, 1 = cancel, i.e. do not show the save dialog
                           // Cancelling the operation allows you to display your own save dialog and to export
                           // your own document, for example to a memory stream and from there to a database.

   VPE_AFTER_SAVE_FILE,    // the file save operation has been completed - wParam = VERR_xyz, lParam = Doc-Handle
                           // Possible return codes: your application should return zero when processing this message

// $ exclude
   VPE_HIGHEST_MESSAGE,    // For internal use only
// $ endexclude
};


// WPARAM of message VPE_PRINT:
// ============================
// $ +n Type PrintAction (PRINT_MSG_)
enum
{
   PRINT_MSG_ABORT,      // User aborted
   PRINT_MSG_START,      // Print started
   PRINT_MSG_END,        // Print ended
// $ +n subst SetupAbort
   PRINT_MSG_SETUPABORT, // User aborted Setup-Dialog
// $ +n subst SetupStart
   PRINT_MSG_SETUPSTART, // Setup-Dialog started
// $ +n subst SetupEnd
   PRINT_MSG_SETUPEND,   // Setup-Dialog ended
};


// Return codes for VPE_PRINT_xyz messages:
// ========================================
// $ +n enum PrintResultingAction (PRINT_ACTION_) {
#define PRINT_ACTION_OK     0   // Engine shall continue its work (for all VPE_PRINT_xyz messages the same)
#define PRINT_ACTION_ABORT  1   // for message VPE_PRINT with wParam=PRINT_MSG_SETUPSTART: Engine shall not start printing
#define PRINT_ACTION_CHANGE 1   // for messages VPE_PRINT_NEWPAGE and VPE_PRINT_DEVDATA:
                                // Engine shall use changed Device-Properties
// $ }


// ========================================================================
//                      Barcode Parameters and Types:
// ========================================================================
// $ +n Type BarcodeParms (BCP_)
enum
{
   BCP_BOTTOM,          // Barcode Text at Bottom
   BCP_TOP,             // Barcode Text at Top
   BCP_HIDE,            // Barcode Text Hidden
   BCP_DEFAULT,         // Barcode Text at default position appropriate to barcode type
};

// $ +n enum BarcodeAlignment {
// $ +n    Left,
// $ +n    Right,
// $ +n    Center,
// $ +n }

// $ +n enum BarcodeType (BCT_) {
#define BCT_EAN13           1   // Checkdigit required, OCR-B Font required
#define BCT_EAN8            2   // Checkdigit required, OCR-B Font required
// $ +n subst UpcA
#define BCT_UPCA            3   // Checkdigit required, OCR-B Font required
#define BCT_CODABAR         5   // Checkdigit optional, uThick / uThin
#define BCT_CODE39          6   // Code 3 of 9, Checkdigit optional, uThick / uThin
// $ +n subst B2Of5
#define BCT_2OF5            7   // 2 of 5 Industrial (25ID), Checkdigit optional, uThick / uThin
#define BCT_INTERLEAVED2OF5 8   // 2 of 5 Interleaved (25IL), Checkdigit optional, uThick / uThin
// $ +n subst UpcE
#define BCT_UPCE            9   // first digit always '0', Checkdigit required, OCR-B Font required
// $ +n subst Ean13_2
#define BCT_EAN13_2         10  // Checkdigit required, OCR-B Font required
// $ +n subst Ean13_5
#define BCT_EAN13_5         11  // Checkdigit required, OCR-B Font required
// $ +n subst Ean8_2
#define BCT_EAN8_2          12  // Checkdigit required, OCR-B Font required
// $ +n subst Ean8_5
#define BCT_EAN8_5          13  // Checkdigit required, OCR-B Font required
// $ +n subst UpcA2
#define BCT_UPCA_2          14  // Checkdigit required, OCR-B Font required
// $ +n subst UpcA5
#define BCT_UPCA_5          15  // Checkdigit required, OCR-B Font required
// $ +n subst UpcE2
#define BCT_UPCE_2          16  // first digit always '0', Checkdigit required, OCR-B Font required
// $ +n subst UpcE5
#define BCT_UPCE_5          17  // first digit always '0', Checkdigit required, OCR-B Font required
// $ exclude
#define BCT_EAN128A         18  // obsolete, use EAN128
#define BCT_EAN128B         19  // obsolete, use EAN128
#define BCT_EAN128C         20  // obsolete, use EAN128
// $ endexclude
#define BCT_CODE93          21  // Code 93, Checkdigit optional
#define BCT_POSTNET         22  // uThick / uThin
// $ exclude
#define BCT_CODE128A        23  // obsolete, use CODE128
#define BCT_CODE128B        24  // obsolete, use CODE128
#define BCT_CODE128C        25  // obsolete, use CODE128
// $ endexclude
#define BCT_CODE128         26  // set of CODE 128A, B and C, Checkdigit required
#define BCT_EAN128          27  // set of  EAN 128A, B and C, Checkdigit required
#define BCT_EAN2            28  // no Checkdigit
#define BCT_EAN5            29  // no Checkdigit
#define BCT_CODE39EXT       30  // Code 3 of 9 Extended, Checkdigit optional, uThick / uThin
#define BCT_CODE93EXT       31  // Code 93 Extended, Checkdigit optional
#define BCT_RM4SCC          32  // Royal Mail 4 State Customer Code, Checkdigit required
#define BCT_MSI             33  // Checkdigit optional, uThick / uThin
#define BCT_ISBN            34  // International Standard Book Number, Checkdigit required
#define BCT_ISBN_5          35  // ISBN + EAN5 (for pricing), Checkdigit required
#define BCT_IDENTCODE       36  // Identcode of the Deutsche Post AG, Checkdigit required
#define BCT_LEITCODE        37  // Leitcode of the Deutsche Post AG, Checkdigit required
#define BCT_PZN             38  // Pharma Zentral Code, Checkdigit required, uThick / uThin
#define BCT_CODE11          39  // Code 11, Checkdigit optional, uThick / uThin
// $ +n subst B2Of5Matrix
#define BCT_2OF5MATRIX      40  // 2 of 5 Matrix, Checkdigit optional, uThick / uThin
// $ +n subst TelepenA
#define BCT_TELEPENA        41  // Telepen-A, Checkdigit optional
// $ }


// ========================================================================
//                          Positioning Codes:
// ========================================================================
// $ +n endtransform
#define VFREE           -2147483549
#define VLEFT           -2147483550
#define VRIGHT          -2147483551
#define VLEFTMARGIN     -2147483552
#define VRIGHTMARGIN    -2147483553
#define VTOP            -2147483554
#define VBOTTOM         -2147483555
#define VTOPMARGIN      -2147483556
#define VBOTTOMMARGIN   -2147483557
// $ skip
#define VCENTER         -2147483558	// yet unused
// $ endskip
#define VMINVFLAG		-2147483549	// for internal use only

#define VWIDTH          -100    // only usable for VpeGet()
#define VHEIGHT         -101    // only usable for VpeGet()
#define VRENDERWIDTH    -102    // only usable for VpeGet() / VpeSet()
#define VRENDERHEIGHT   -103    // only usable for VpeGet() / VpeSet()

#define VUDO_LEFT       -104    // only usable for VpeGet()
#define VUDO_RIGHT      -105    // only usable for VpeGet()
#define VUDO_TOP        -106    // only usable for VpeGet()
#define VUDO_BOTTOM     -107    // only usable for VpeGet()
#define VUDO_WIDTH      -108    // only usable for VpeGet()
#define VUDO_HEIGHT     -109    // only usable for VpeGet()
// $ +n transform


// C/C++ Shortcuts:
// ================
#define nLeft(hDoc)           VpeGet(hDoc, VLEFT)
#define nTop(hDoc)            VpeGet(hDoc, VTOP)
#define nRight(hDoc)          VpeGet(hDoc, VRIGHT)
#define nBottom(hDoc)         VpeGet(hDoc, VBOTTOM)
#define nLeftMargin(hDoc)     VpeGet(hDoc, VLEFTMARGIN)
#define nTopMargin(hDoc)      VpeGet(hDoc, VTOPMARGIN)
#define nRightMargin(hDoc)    VpeGet(hDoc, VRIGHTMARGIN)
#define nBottomMargin(hDoc)   VpeGet(hDoc, VBOTTOMMARGIN)
#define nWidth(hDoc)          VpeGet(hDoc, VWIDTH)
#define nHeight(hDoc)         VpeGet(hDoc, VHEIGHT)
#define nRenderWidth(hDoc)    VpeGet(hDoc, VRENDERWIDTH)
#define nRenderHeight(hDoc)   VpeGet(hDoc, VRENDERHEIGHT)
#define nUDOLeft(hDoc)        VpeGet(hDoc, VUDO_LEFT)
#define nUDORight(hDoc)       VpeGet(hDoc, VUDO_RIGHT)
#define nUDOTop(hDoc)         VpeGet(hDoc, VUDO_TOP)
#define nUDOBottom(hDoc)      VpeGet(hDoc, VUDO_BOTTOM)
#define nUDOWidth(hDoc)       VpeGet(hDoc, VUDO_WIDTH)
#define nUDOHeight(hDoc)      VpeGet(hDoc, VUDO_HEIGHT)


// ==========================================================================
// Return Codes of Rendering Functions for Text and RTF:
// ==========================================================================
// $ +n Type RenderStatus (RENDER_)
enum
{
// $ +n subst NoPageBreak
   RENDER_NO_BREAK,     // NO Auto Break will occur
// $ +n subst PageBreak
   RENDER_BREAK,        // Auto Break will occur
// $ +n subst SkipPageBreak
   RENDER_SKIP_BREAK,   // Auto Break will occur, but no text will be placed on the actual
                        // page, all text will be skipped to the next page
                        // (VRENDERWIDTH and VRENDERHEIGHT are not set)
};


// ==========================================================================
// Return Codes for VpeIsMAPIInstalled:
// ==========================================================================
// $ +n Type MAPIStatus (VMAPI_)
enum
{
   VMAPI_NOT_INSTALLED, // MAPI not installed
   VMAPI_INSTALLED,     // MAPI installed
   VMAPI_UNSURE,        // unsure
};


// ==========================================================================
// Error-Codes returned by VpeLastError()
// Note, that only functions setting the error state will also clear it, if
// no error occured! All other functions will keep the error state untouched.
// ==========================================================================
// $ +n Type ErrorCode (VERR_)
enum
{
   VERR_OK,               // no error
   VERR_COMMON,           // common error
   VERR_CANCELLED,        // The user cancelled an operation, for example the Open or Save file dialog
   
   VERR_MEMORY = 100,     // out of memory

   VERR_FILE_OPEN = 200,  // error opening a file; occurs when calling functions like
                          // OpenDocFile(), VpeReadDoc(), VpeWriteDoc(), ReadPrinterSetup(),
                          // Picture(), WriteRTFFile(), etc.
                          // Meaning: The specified file could not be opened, it is either not
                          // existing, the path is not existing, it is locked, incorrect
                          // logical structure (for example corrupted file)
// $ +n subst FileDocVersion
   VERR_FILE_DOCVERSION,  // Document file has the wrong (higher) version and cannot be opened / read
   VERR_FILE_CREATE,      // error creating file; occurs when calling functions like
                          // WriteDoc(), OpenDocFile(), WritePrinterSetup(),
                          // SetupPrinter() (only during write), etc.
                          // Meaning: Illegal path / filename, file locked, disk full
   VERR_FILE_ACCESS,      // Access denied (no permission); see also VpeSetDocFileReadOnly()
   VERR_FILE_READ,        // error during file read operation
   VERR_FILE_WRITE,       // error during file write operation
   
   VERR_PIC_IMPORT = 300, // image could not be imported
                          // Meaning: File or resource not found, file not accessible,
                          // or image structure unreadable / corrupted, or not enough memory
// $ exclude
   VERR_PIC_NOLICENSE,    // No license for image access
                          // (e.g. TIFF / GIF image is LZW compressed and flag PIC_ALLOWLZW not used)
                          // OBSOLETE: The LZW patent has expired. Since VPE v3.60 LZW compressed
                          //           images are imported without specifying the PIC_ALLOWLZW flag.
// $ endexclude
// $ +n subst PicDXFCoord
   VERR_PIC_DXFCOORD,     // For DXF formats, x2 and y2 may not be VFREE at the same time,
                          // either x2 or y2 must be <> VFREE
   
   VERR_PIC_EXPORT = 350, // image could not be exported
                          // Meaning: File could not be created, or not enough memory

   VERR_MOD_GRAPH_IMP = 400,  // Error loading Graphics Import Library (Dav3_32.DLL)
   VERR_MOD_GRAPH_PROC,       // Error loading Graphics Processing Library (Leon3_32.DLL)
   VERR_MOD_BARCODE,          // Error loading Barcode Library (BARVISD.DLL)
   VERR_MOD_CHART,            // Error loading Chart Library (VChart32.DLL)
   VERR_MOD_ZLIB,             // Error loading ZLIB Library (VZLIB32.DLL)
// $ +n subst ModPDF
   VERR_MOD_VPDF,             // Error loading PDF Export Library (VPDF32.DLL)
// $ +n subst ModBarcode2D
   VERR_MOD_VBAR2D,           // Error loading 2D-Barcode Library (VBAR2D32.DLL)

   VERR_MAIL_LOAD_MAPI = 450,           // Could not load MAPI
   VERR_MAIL_CREATE,                    // Could not create temporary file
   VERR_MAIL_USER_ABORT,
   VERR_MAIL_FAILURE,
   VERR_MAIL_LOGON_FAILURE,
   VERR_MAIL_DISK_FULL,
   VERR_MAIL_INSUFFICIENT_MEMORY,
   VERR_MAIL_ACCESS_DENIED,
   VERR_MAIL_RESERVED,
   VERR_MAIL_TOO_MANY_SESSIONS,
   VERR_MAIL_TOO_MANY_FILES,
   VERR_MAIL_TOO_MANY_RECIPIENTS,
   VERR_MAIL_ATTACHMENT_NOT_FOUND,
   VERR_MAIL_ATTACHMENT_OPEN_FAILURE,
   VERR_MAIL_ATTACHMENT_WRITE_FAILURE,
   VERR_MAIL_UNKNOWN_RECIPIENT,
// $ +n subst MailBadRecipType
   VERR_MAIL_BAD_RECIPTYPE,
   VERR_MAIL_NO_MESSAGES,
   VERR_MAIL_INVALID_MESSAGE,
   VERR_MAIL_TEXT_TOO_LARGE,
   VERR_MAIL_INVALID_SESSION,
   VERR_MAIL_TYPE_NOT_SUPPORTED,
   VERR_MAIL_AMBIGUOUS_RECIPIENT,
   VERR_MAIL_MESSAGE_IN_USE,
   VERR_MAIL_NETWORK_FAILURE,
// $ +n subst MailInvalidEditFields
   VERR_MAIL_INVALID_EDITFIELDS,
   VERR_MAIL_INVALID_RECIPS,
   VERR_MAIL_NOT_SUPPORTED,

   VERR_ZLIB_STREAM = 500,   // Stream Inconsistent
   VERR_ZLIB_DATA,           // Data Corrupt
   VERR_ZLIB_BUFFER,         // Internal Buffer Error
   VERR_ZLIB_VERSION,        // Wrong Version of VZLIB32.DLL

   VERR_VBAR2D_FORMAT_OUT_OF_RANGE = 550,
   VERR_VBAR2D_UNDEFINED_ID,
   VERR_VBAR2D_FORMAT_TOO_LONG,
   VERR_VBAR2D_FORMAT_OUT_OF_MEMORY,
   VERR_VBAR2D_FORMAT_DATA_INVALID, 
   VERR_VBAR2D_FORMAT_NOT_ALLOWED,
   VERR_VBAR2D_DATA_WRONG_LENGTH,
   VERR_VBAR2D_DATA_ZERO_LENGTH, 
   VERR_VBAR2D_DATA_TOO_SHORT,
   VERR_VBAR2D_DATA_TOO_LONG,
   VERR_VBAR2D_INVALID_DATA, 
   VERR_VBAR2D_SQUARE_EDGE_TOO_SMALL,
   VERR_VBAR2D_SQUARE_TOO_LARGE,
   VERR_VBAR2D_EDGE_OVER_FORCED,
   VERR_VBAR2D_SQUARE_ASPECT_SMALL,
   VERR_VBAR2D_SQUARE_ASPECT_LARGE,
   VERR_VBAR2D_SQUARE_EVEN_ODD_MATCH,
   VERR_VBAR2D_INVALID_EDGE,
   VERR_VBAR2D_SQUARE_EDGE_TOO_LARGE,
   VERR_VBAR2D_INVALID_ECC,
   VERR_VBAR2D_INVALID_BORDER,
   VERR_VBAR2D_SELF_TEST_FAILED,

   VERR_RTF_BRACES = 1000, // RTF: unbalanced braces "{}"
   VERR_RTF_OVERFLOW,      // RTF: only 16-bit version; generated internal structure > 64 KB
// $ +n subst RtfFontTable
   VERR_RTF_FONTTBL,       // RTF: error parsing font table
// $ +n subst RtfColorTable
   VERR_RTF_COLORTBL,      // RTF: error parsing color table

   VERR_TPL_OWNERSHIP = 2000,       // Tpl: tried to dump the template to a foreign VPE document (where the template was not loaded into).
   VERR_TPL_PAGE_ALREADY_DUMPED,    // Tpl: the page contains interactive objects and already had been dumped.
                                    //      A page containing interactive objects may only be dumped once.
   VERR_TPL_AUTHENTICATION,         // Tpl: the template has an authentication key and it has not been validated successfully
};


// ========================================================================
// Flags for Device-Functions
// ========================================================================
// $ +n enum PageOrientation (VORIENT_) {
#define VORIENT_PORTRAIT  1
#define VORIENT_LANDSCAPE 2
// $ }

// $ +n enum DevPrintQuality (VRES_) {
#define VRES_DRAFT       -1
#define VRES_LOW         -2
#define VRES_MEDIUM      -3
#define VRES_HIGH        -4
// $ }

// $ +n enum DevColor (VCOLOR_) {
#define VCOLOR_MONOCHROME 1
#define VCOLOR_COLOR      2
// $ }

// $ +n enum DevDuplex (VDUP_) {
#define VDUP_SIMPLEX      1
#define VDUP_VERTICAL     2
#define VDUP_HORIZONTAL   3
// $ }

// $ +n enum DevTTOption (VTT_) {
#define VTT_BITMAP        1
#define VTT_DOWNLOAD      2
#define VTT_SUBDEV        3
// $ }

// $ +n enum PageFormat (VPAPER_) {
// $ +n replace Env Envelope
#define VPAPER_A4                  -1      // A4 Sheet, 210- by 297-millimeters
#define VPAPER_LETTER              -2      // US Letter, 8 1/2- by 11-inches
#define VPAPER_LEGAL               -3      // Legal, 8 1/2- by 14-inches
// $ +n subst CSheet
#define VPAPER_CSHEET              -4      // C Sheet, 17- by 22-inches
// $ +n subst DSheet
#define VPAPER_DSHEET              -5      // D Sheet, 22- by 34-inches
// $ +n subst ESheet
#define VPAPER_ESHEET              -6      // E Sheet, 34- by 44-inches
// $ +n subst LetterSmall
#define VPAPER_LETTERSMALL         -7      // Letter Small, 8 1/2- by 11-inches
#define VPAPER_TABLOID             -8      // Tabloid, 11- by 17-inches
#define VPAPER_LEDGER              -9      // Ledger, 17- by 11-inches
#define VPAPER_STATEMENT           -10     // Statement, 5 1/2- by 8 1/2-inches
#define VPAPER_EXECUTIVE           -11     // Executive, 7 1/4- by 10 1/2-inches
#define VPAPER_A3                  -12     // A3 sheet, 297- by 420-millimeters
#define VPAPER_A4SMALL             -13     // A4 small sheet, 210- by 297-millimeters
#define VPAPER_A5                  -14     // A5 sheet, 148- by 210-millimeters
#define VPAPER_B4                  -15     // B4 sheet, 250- by 354-millimeters
#define VPAPER_B5                  -16     // B5 sheet, 182- by 257-millimeter paper
#define VPAPER_FOLIO               -17     // Folio, 8 1/2- by 13-inch paper
#define VPAPER_QUARTO              -18     // Quarto, 215- by 275-millimeter paper
// $ +n subst Standard10x14
#define VPAPER_10X14               -19     // 10- by 14-inch sheet
// $ +n subst Standard11x17
#define VPAPER_11X17               -20     // 11- by 17-inch sheet
#define VPAPER_NOTE                -21     // Note, 8 1/2- by 11-inches
#define VPAPER_ENV_9               -22     // #9 Envelope, 3 7/8- by 8 7/8-inches
#define VPAPER_ENV_10              -23     // #10 Envelope, 4 1/8- by 9 1/2-inches
#define VPAPER_ENV_11              -24     // #11 Envelope, 4 1/2- by 10 3/8-inches
#define VPAPER_ENV_12              -25     // #12 Envelope, 4 3/4- by 11-inches
#define VPAPER_ENV_14              -26     // #14 Envelope, 5- by 11 1/2-inches
// $ +n subst EnvelopeDL
#define VPAPER_ENV_DL              -27     // DL Envelope, 110- by 220-millimeters
#define VPAPER_ENV_C5              -28     // C5 Envelope, 162- by 229-millimeters
#define VPAPER_ENV_C3              -29     // C3 Envelope,  324- by 458-millimeters
#define VPAPER_ENV_C4              -30     // C4 Envelope,  229- by 324-millimeters
#define VPAPER_ENV_C6              -31     // C6 Envelope,  114- by 162-millimeters
#define VPAPER_ENV_C65             -32     // C65 Envelope, 114- by 229-millimeters
#define VPAPER_ENV_B4              -33     // B4 Envelope,  250- by 353-millimeters
#define VPAPER_ENV_B5              -34     // B5 Envelope,  176- by 250-millimeters
#define VPAPER_ENV_B6              -35     // B6 Envelope,  176- by 125-millimeters
#define VPAPER_ENV_ITALY           -36     // Italy Envelope, 110- by 230-millimeters
#define VPAPER_ENV_MONARCH         -37     // Monarch Envelope, 3 7/8- by 7 1/2-inches
#define VPAPER_ENV_PERSONAL        -38     // 6 3/4 Envelope, 3 5/8- by 6 1/2-inches
// $ +n subst FanfoldUS
#define VPAPER_FANFOLD_US          -39     // US Std Fanfold, 14 7/8- by 11-inches
#define VPAPER_FANFOLD_STD_GERMAN  -40     // German Std Fanfold, 8 1/2- by 12-inches
#define VPAPER_FANFOLD_LGL_GERMAN  -41     // German Legal Fanfold, 8 1/2- by 13-inches
#define VPAPER_USER_DEFINED          0     // User-Defined
// $ exclude
#define DIN_A_4                     -1     // for compatibility with older versions
#define US_LETTER                   -2     // for compatibility with older versions
// $ endexclude

// #if(WINVER >= 0x0400)
#define VPAPER_ISO_B4              -42  /* B4 (ISO) 250 x 353 mm              */
#define VPAPER_JAPANESE_POSTCARD   -43  /* Japanese Postcard 100 x 148 mm     */
// $ +n subst Standard9x11
#define VPAPER_9X11                -44  /* 9 x 11 in                          */
// $ +n subst Standard10x11
#define VPAPER_10X11               -45  /* 10 x 11 in                         */
// $ +n subst Standard15x11
#define VPAPER_15X11               -46  /* 15 x 11 in                         */
#define VPAPER_ENV_INVITE          -47  /* Envelope Invite 220 x 220 mm       */
#define VPAPER_RESERVED_48         -48  /* RESERVED--DO NOT USE               */
#define VPAPER_RESERVED_49         -49  /* RESERVED--DO NOT USE               */
#define VPAPER_LETTER_EXTRA        -50  /* Letter Extra 9 \275 x 12 in        */
#define VPAPER_LEGAL_EXTRA         -51  /* Legal Extra 9 \275 x 15 in         */
#define VPAPER_TABLOID_EXTRA       -52  /* Tabloid Extra 11.69 x 18 in        */
#define VPAPER_A4_EXTRA            -53  /* A4 Extra 9.27 x 12.69 in           */
#define VPAPER_LETTER_TRANSVERSE   -54  /* Letter Transverse 8 \275 x 11 in   */
#define VPAPER_A4_TRANSVERSE       -55  /* A4 Transverse 210 x 297 mm         */
#define VPAPER_LETTER_EXTRA_TRANSVERSE -56 /* Letter Extra Transverse 9\275 x 12 in */
#define VPAPER_A_PLUS              -57  /* SuperA/SuperA/A4 227 x 356 mm      */
#define VPAPER_B_PLUS              -58  /* SuperB/SuperB/A3 305 x 487 mm      */
#define VPAPER_LETTER_PLUS         -59  /* Letter Plus 8.5 x 12.69 in         */
#define VPAPER_A4_PLUS             -60  /* A4 Plus 210 x 330 mm               */
#define VPAPER_A5_TRANSVERSE       -61  /* A5 Transverse 148 x 210 mm         */
#define VPAPER_B5_TRANSVERSE       -62  /* B5 (JIS) Transverse 182 x 257 mm   */
#define VPAPER_A3_EXTRA            -63  /* A3 Extra 322 x 445 mm              */
#define VPAPER_A5_EXTRA            -64  /* A5 Extra 174 x 235 mm              */
#define VPAPER_B5_EXTRA            -65  /* B5 (ISO) Extra 201 x 276 mm        */
#define VPAPER_A2                  -66  /* A2 420 x 594 mm                    */
#define VPAPER_A3_TRANSVERSE       -67  /* A3 Transverse 297 x 420 mm         */
#define VPAPER_A3_EXTRA_TRANSVERSE -68  /* A3 Extra Transverse 322 x 445 mm   */

// #if(WINVER >= 0x0500)
// $ replace Jenv JapaneseEnvelope
#define VPAPER_DBL_JAPANESE_POSTCARD -69 /* Japanese Double Postcard 200 x 148 mm */
#define VPAPER_A6                  -70  /* A6 105 x 148 mm                 */
#define VPAPER_JENV_KAKU2          -71  /* Japanese Envelope Kaku #2       */
#define VPAPER_JENV_KAKU3          -72  /* Japanese Envelope Kaku #3       */
#define VPAPER_JENV_CHOU3          -73  /* Japanese Envelope Chou #3       */
#define VPAPER_JENV_CHOU4          -74  /* Japanese Envelope Chou #4       */
#define VPAPER_LETTER_ROTATED      -75  /* Letter Rotated 11 x 8 1/2 in    */
#define VPAPER_A3_ROTATED          -76  /* A3 Rotated 420 x 297 mm         */
#define VPAPER_A4_ROTATED          -77  /* A4 Rotated 297 x 210 mm         */
#define VPAPER_A5_ROTATED          -78  /* A5 Rotated 210 x 148 mm         */
#define VPAPER_B4_JIS_ROTATED      -79  /* B4 (JIS) Rotated 364 x 257 mm   */
#define VPAPER_B5_JIS_ROTATED      -80  /* B5 (JIS) Rotated 257 x 182 mm   */
#define VPAPER_JAPANESE_POSTCARD_ROTATED -81 /* Japanese Postcard Rotated 148 x 100 mm */
#define VPAPER_DBL_JAPANESE_POSTCARD_ROTATED -82 /* Double Japanese Postcard Rotated 148 x 200 mm */
#define VPAPER_A6_ROTATED          -83  /* A6 Rotated 148 x 105 mm         */
#define VPAPER_JENV_KAKU2_ROTATED  -84  /* Japanese Envelope Kaku #2 Rotated */
#define VPAPER_JENV_KAKU3_ROTATED  -85  /* Japanese Envelope Kaku #3 Rotated */
#define VPAPER_JENV_CHOU3_ROTATED  -86  /* Japanese Envelope Chou #3 Rotated */
#define VPAPER_JENV_CHOU4_ROTATED  -87  /* Japanese Envelope Chou #4 Rotated */
#define VPAPER_B6_JIS              -88  /* B6 (JIS) 128 x 182 mm           */
#define VPAPER_B6_JIS_ROTATED      -89  /* B6 (JIS) Rotated 182 x 128 mm   */
// $ +n subst Standard12x11
#define VPAPER_12X11               -90  /* 12 x 11 in                      */
#define VPAPER_JENV_YOU4           -91  /* Japanese Envelope You #4        */
#define VPAPER_JENV_YOU4_ROTATED   -92  /* Japanese Envelope You #4 Rotated*/
// $ +n subst Prc16K
#define VPAPER_P16K                -93  /* PRC 16K 146 x 215 mm            */
// $ +n subst Prc32K
#define VPAPER_P32K                -94  /* PRC 32K 97 x 151 mm             */
// $ +n subst Prc32KBig
#define VPAPER_P32KBIG             -95  /* PRC 32K(Big) 97 x 151 mm        */
// $ +n replace Penv PrcEnvelope
#define VPAPER_PENV_1              -96  /* PRC Envelope #1 102 x 165 mm    */
#define VPAPER_PENV_2              -97  /* PRC Envelope #2 102 x 176 mm    */
#define VPAPER_PENV_3              -98  /* PRC Envelope #3 125 x 176 mm    */
#define VPAPER_PENV_4              -99  /* PRC Envelope #4 110 x 208 mm    */
#define VPAPER_PENV_5              -100 /* PRC Envelope #5 110 x 220 mm    */
#define VPAPER_PENV_6              -101 /* PRC Envelope #6 120 x 230 mm    */
#define VPAPER_PENV_7              -102 /* PRC Envelope #7 160 x 230 mm    */
#define VPAPER_PENV_8              -103 /* PRC Envelope #8 120 x 309 mm    */
#define VPAPER_PENV_9              -104 /* PRC Envelope #9 229 x 324 mm    */
#define VPAPER_PENV_10             -105 /* PRC Envelope #10 324 x 458 mm   */
// $ +n subst Prc16KRotated
#define VPAPER_P16K_ROTATED        -106 /* PRC 16K Rotated                 */
// $ +n subst Prc32KRotated
#define VPAPER_P32K_ROTATED        -107 /* PRC 32K Rotated                 */
// $ +n subst Prc32KBigRotated
#define VPAPER_P32KBIG_ROTATED     -108 /* PRC 32K(Big) Rotated            */
#define VPAPER_PENV_1_ROTATED      -109 /* PRC Envelope #1 Rotated 165 x 102 mm */
#define VPAPER_PENV_2_ROTATED      -110 /* PRC Envelope #2 Rotated 176 x 102 mm */
#define VPAPER_PENV_3_ROTATED      -111 /* PRC Envelope #3 Rotated 176 x 125 mm */
#define VPAPER_PENV_4_ROTATED      -112 /* PRC Envelope #4 Rotated 208 x 110 mm */
#define VPAPER_PENV_5_ROTATED      -113 /* PRC Envelope #5 Rotated 220 x 110 mm */
#define VPAPER_PENV_6_ROTATED      -114 /* PRC Envelope #6 Rotated 230 x 120 mm */
#define VPAPER_PENV_7_ROTATED      -115 /* PRC Envelope #7 Rotated 230 x 160 mm */
#define VPAPER_PENV_8_ROTATED      -116 /* PRC Envelope #8 Rotated 309 x 120 mm */
#define VPAPER_PENV_9_ROTATED      -117 /* PRC Envelope #9 Rotated 324 x 229 mm */
#define VPAPER_PENV_10_ROTATED     -118 /* PRC Envelope #10 Rotated 458 x 324 mm */
// $ +n endreplace
// $ }


// =========================================================================
//                              Paper-Bins
// =========================================================================
// $ +n enum PaperBin (VBIN_) {
#define VBIN_UNTOUCHED     -1   // default
#define VBIN_UPPER          1
// $ +n subst OnlyOne
#define VBIN_ONLYONE        1
#define VBIN_LOWER          2
#define VBIN_MIDDLE         3
#define VBIN_MANUAL         4
#define VBIN_ENVELOPE       5
// $ +n subst EnvelopeManual
#define VBIN_ENVMANUAL      6
#define VBIN_AUTO           7 
#define VBIN_TRACTOR        8
// $ +n subst SmallFormat
#define VBIN_SMALLFMT       9
// $ +n subst LargeFormat
#define VBIN_LARGEFMT      10
// $ +n subst LargeCapacity
#define VBIN_LARGECAPACITY 11
#define VBIN_CASSETTE      14
// $ }


// =========================================================================
//                              Mail
// =========================================================================
// $ +n enum RecipientClass (VMAIL_) {
#define VMAIL_ORIG   0          // Recipient is message originator
#define VMAIL_TO     1          // Recipient is a primary recipient
#define VMAIL_CC     2          // Recipient is a copy recipient
#define VMAIL_BCC    3          // Recipient is blind copy recipient

#define VMAIL_RESOLVE_NAME   0x80000000     // try to resolve the name from the mail client's address book
// $ }


// =========================================================================
// Behavior of preview for page forward / backward action
// (vertical positioning of page in the preview after user
// moved a page forward / back)
// =========================================================================
// $ +n Type PreviewCtrl (PREVIEW_)
enum PREVIEWCTRL
{
   PREVIEW_STAY,        // vertical position unaffected
// $ +n subst JumpTop
   PREVIEW_JUMPTOP,     // vertical position on top of page (default)
};


// =========================================================================
//                              Keyboard-Layout
// =========================================================================
// $ +n enum KeyFunction (VKEY_) {
#define VKEY_SCROLL_LEFT         0
#define VKEY_SCROLL_PAGE_LEFT    1
#define VKEY_SCROLL_RIGHT        2
#define VKEY_SCROLL_PAGE_RIGHT   3
#define VKEY_SCROLL_UP           4
#define VKEY_SCROLL_PAGE_UP      5
#define VKEY_SCROLL_DOWN         6
#define VKEY_SCROLL_PAGE_DOWN    7
#define VKEY_SCROLL_TOP          8
#define VKEY_SCROLL_BOTTOM       9
#define VKEY_PRINT              10
#define VKEY_MAIL               11
#define VKEY_FULL_PAGE          12
#define VKEY_PAGE_WIDTH         13
#define VKEY_ZOOM_IN            14
#define VKEY_ZOOM_OUT           15
#define VKEY_GRID               16
#define VKEY_PAGE_FIRST         17
#define VKEY_PAGE_LEFT          18
#define VKEY_PAGE_RIGHT         19
#define VKEY_PAGE_LAST          20
#define VKEY_HELP               21
#define VKEY_INFO               22
#define VKEY_CLOSE              23
#define VKEY_GOTO_PAGE          24
#define VKEY_OPEN               25
#define VKEY_SAVE               26
#define VKEY_ESCAPE             27
// $ exclude
#define VKEY_LAST_KEY           28      // for internal use only
// $ endexclude
// $ }


// =========================================================================
//                             GUI Languages:
// =========================================================================
// $ +n enum GUILanguage (VGUI_LANGUAGE_) {
#define VGUI_LANGUAGE_USER_DEFINED -1
#define VGUI_LANGUAGE_ENGLISH       0
#define VGUI_LANGUAGE_GERMAN        1
#define VGUI_LANGUAGE_FRENCH        2
#define VGUI_LANGUAGE_DUTCH         3
#define VGUI_LANGUAGE_SPANISH       4
#define VGUI_LANGUAGE_DANISH        5
#define VGUI_LANGUAGE_SWEDISH       6
#define VGUI_LANGUAGE_FINNISH       7
#define VGUI_LANGUAGE_ITALIAN       8
#define VGUI_LANGUAGE_NORWEGIAN     9
#define VGUI_LANGUAGE_PORTUGUESE   10
// $ exclude
#define VGUI_LANGUAGE_LAST         10      // for internal use only
// $ endexclude
// $ }


// ========================================================================
//                          Other Definitions:
// ========================================================================
// $ +n enum RulersMeasure {
// $ +n subst Centimeter
#define CM      0       // Parameters for RulersMeasure
#define INCH    1
// $ }


// ========================================================================
//						      Bookmarks
// ========================================================================
// Bookmark Destination Flags:
// $ +n Type BookmarkDestination (VBOOKMARK_DEST_)
enum 
{
   VBOOKMARK_DEST_NONE = 0,   // No Destination
// $ +n subst LeftTopZoom
   VBOOKMARK_DEST_LTZ,        // Normal Destination (left, top, zoom are used) [default]
// $ +n subst Fit
   VBOOKMARK_DEST_FIT,        // Fit the page in the window (no coordinates used)
// $ +n subst FitHorizontally
   VBOOKMARK_DEST_FITH,       // Fit the page in the window horizontally (top coordinate used)
// $ +n subst FitVertically
   VBOOKMARK_DEST_FITV,       // Fit the page in the window vertically (left coordinate used)
// $ +n subst FitRectangle
   VBOOKMARK_DEST_FITR,       // Fit the page in the specified rectangle (all coordinates used)
// $ +n subst FitBounding
   VBOOKMARK_DEST_FITB,       // Fit the page's bounding rectangle in the window
// $ +n subst FitBoundingHorizontally
   VBOOKMARK_DEST_FITBH,      // Fit the page's bounding rectangle horizontally in the window (top coordinate used)
// $ +n subst FitBoundingVertically
   VBOOKMARK_DEST_FITBV,      // Fit the page's bounding rectangle vertically in the window (left coordinate used)
};

// Bookmark Style Flags:
// $ +n enum BookmarkStyle (VBOOKMARK_STYLE_) {
#define VBOOKMARK_STYLE_NONE     0     // [default]
#define VBOOKMARK_STYLE_BOLD     1     // (PDF 1.4 or higher)
#define VBOOKMARK_STYLE_ITALIC   2     // (PDF 1.4 or higher)
#define VBOOKMARK_STYLE_OPEN     4
// $ }


// ========================================================================
//                       Picture Export Definitions:
// ========================================================================
// $ +n enum DocExportPictureQuality (PICEXP_QUALITY_) {
#define PICEXP_QUALITY_NORMAL    0     // embedded images will have normal quality [default]
#define PICEXP_QUALITY_HIGH      1     // embedded images will have high quality (using Scale2Gray, 
                                       // only available in VPE Professional Edition or higher)
// $ }


// ========================================================================
//                       Document Export Definitions:
// ========================================================================
// $ +n enum DocExportType (VPE_DOC_TYPE_) {
#define VPE_DOC_TYPE_AUTO        0     // VpeWriteDoc() will create VPE, PDF or HTML files, depending on the file suffix [default]
// $ +n subst VPE
#define VPE_DOC_TYPE_VPE         1     // VpeWriteDoc() will create VPE files, regardless of the file suffix
// $ +n subst PDF
#define VPE_DOC_TYPE_PDF         2     // VpeWriteDoc() will create PDF files, regardless of the file suffix
// $ +n subst HTML
#define VPE_DOC_TYPE_HTML        3     // VpeWriteDoc() will create HTML files, regardless of the file suffix
// $ }

// $ +n enum PDFVersion (VPE_PDF_) {
#define VPE_PDF_ACROBAT_4        1300  // Create PDF documents compatible to Acrobat Reader 4 (PDF v1.3) [default]
#define VPE_PDF_ACROBAT_5        1400  // Create PDF documents compatible to Acrobat Reader 5 (PDF v1.4)
// $ }

// $ +n enum Compression (DOC_COMPRESS_) {
#define DOC_COMPRESS_NONE        0     // no compression
#define DOC_COMPRESS_FLATE       1     // Flate compression (also known as ZLIB Compression) [default]
// $ }

// $ +n enum Encryption (DOC_ENCRYPT_) {
#define DOC_ENCRYPT_NONE         0     // no encryption [default]
#define DOC_ENCRYPT_STREAM       1     // use stream encryption (compatible to PDF 1.3 and higher)
// $ }

// Document access flags, only in effect, when using encryption
// Any combination of these flags can be used by just adding their values
// $ +n enum Protection {
#define PDF_ALLOW_NONE       0     // full protection, no flag is raised

#define PDF_ALLOW_PRINT      4     // allow to print the document (see also PDF_ALLOW_HIQ_PRINT)

#define PDF_ALLOW_MODIFY     8     // allow to modify the contents of the document by operations other than
                                   // those controlled by the flags PDF_ALLOW_TA_IFF, PDF_ALLOW_FILL_IFF
                                   // and PDF_ALLOW_ASSEMBLE

#define PDF_ALLOW_COPY       16    // PDF v1.3: allow to copy or otherwise extract text and graphics from the
                                   //           document, including extracting text and graphics (in support
                                   //           of accessibility to disabled users or for other purposes)
                                   // PDF v1.4: allow to copy or otherwise extract text and graphics from the
                                   //           document by operations other than that controlled by
                                   //           PDF_ALLOW_EXTRACT

#define PDF_ALLOW_TA_IFF     32    // allow to add or modify text annotations, fill in interactive form fields,
                                   // and - if PDF_ALLOW_MODIFY is also used - create or modify interactive form
                                   // fields (including signature fields)

#define PDF_ALLOW_FILL_IFF   256   // PDF v1.4 only: allow to fill in existing interactive form fields
                                   // (including signature fields), even if PDF_ALLOW_TA_IFF is not used

#define PDF_ALLOW_EXTRACT    512   // PDF v1.4 only: allow to extract text and graphics (in support of
                                   // accessibility to disabled users or for other purposes)

#define PDF_ALLOW_ASSEMBLE   1024  // PDF v1.4 only: allow to assemble the document (insert, rotate, or delete
                                   // pages and create bookmarks or thumbnail images), even if PDF_ALLOW_MODIFY
                                   // is not used

// $ +n subst PDFAllowHiQualityPrint
#define PDF_ALLOW_HIQ_PRINT  2048  // PDF v1.4 only: allow to print the document to a representation from
                                   // which a faithful digital copy of the PDF content could be generated.
                                   // When this flag is NOT specified (and PDF_ALLOW_PRINT is used), printing
                                   // is limited to a lowlevel representation of the appearance, possibly
                                   // of degraded quality.

#define PDF_ALLOW_ALL        3900  // no protection, all flags are raised
// $ }
// $ +n endreplace


// $ +n enum GridMode {
// $ +n    In_Background,
// $ +n    In_Foreground,
// $ +n }


// $ +n enum EditProtection {
// $ +n    Unprotected,
// $ +n    Protected,
// $ +n }


// $ +av endskip


// Charsets
// $ +n enum CharSet (VCHARSET_) {
// $ +av enum CharSet {
#define VCHARSET_DEFAULT         1
#define VCHARSET_SYMBOL          2
#define VCHARSET_MAC_ROMAN       77

#define VCHARSET_WIN_ANSI        0
#define VCHARSET_WIN_HEBREW      177
#define VCHARSET_WIN_ARABIC      178
#define VCHARSET_WIN_GREEK       161
#define VCHARSET_WIN_TURKISH     162
#define VCHARSET_WIN_VIETNAMESE  163
#define VCHARSET_WIN_THAI        222
#define VCHARSET_WIN_EAST_EUROPE 238
#define VCHARSET_WIN_CYRILLIC    204
#define VCHARSET_WIN_BALTIC      186

#define VCHARSET_ISO_LATIN_1     50
#define VCHARSET_ISO_LATIN_2     51
#define VCHARSET_ISO_LATIN_3     52
#define VCHARSET_ISO_LATIN_4     53
#define VCHARSET_ISO_CYRILLIC    54
#define VCHARSET_ISO_ARABIC      55
#define VCHARSET_ISO_GREEK       56
#define VCHARSET_ISO_HEBREW      57
#define VCHARSET_ISO_LATIN_5     58
#define VCHARSET_ISO_LATIN_6     59
#define VCHARSET_ISO_THAI        60
#define VCHARSET_ISO_LATIN_7     62
#define VCHARSET_ISO_LATIN_8     63
#define VCHARSET_ISO_LATIN_9     64
// $ }


// Resource ID's for User Defined Resources
// $ +n Type ResourceId (VRSCID_)
// $ +av Type ResourceId
enum
{
	VRSCID_COPY_OF,			// "Copy %u of %u"
	VRSCID_PAGE_OF,			// "Page %u of %u"
	VRSCID_BAND,			// "Band %u" (i.e. one could also say "stripe" for "band")
	VRSCID_ERROR,			// "Error"
	VRSCID_WARNING,			// "Warning"
	VRSCID_PRINTING,		// "Printing"
	VRSCID_CANCEL,			// "Cancel"
	VRSCID_OK,				// "Ok"

	// Tooltips
	VRSCID_PRINT_DOC,		// "Print the Document"
	VRSCID_FULL_PAGE,		// "Show Full Page"
	VRSCID_PAGE_WIDTH,		// "Fit Page Width"
	VRSCID_ZOOM_IN,			// "Zoom in"
	VRSCID_ZOOM_OUT,		// "Zoom out"
	VRSCID_FIRST_PAGE,		// "Go to First Page"
	VRSCID_PREV_PAGE,		// "Go to Previous Page"
	VRSCID_NEXT_PAGE,		// "Go to Next Page"
	VRSCID_LAST_PAGE,		// "Go to Last Page"
	VRSCID_HELP,			// "Help on Control"
	VRSCID_INFORMATION,		// "Program Information"
	VRSCID_CLOSE_PREVIEW,	// "Close Preview"
	VRSCID_GRID,			// "Show / Hide Grid"
	VRSCID_ENTER_PAGENO,	// "enter page# with mouse-click"
	VRSCID_ZOOM_FACTOR,		// "Zoom Factor"
	VRSCID_STATUS,			// "Status"
	VRSCID_READY,			// "Ready"
	VRSCID_EMAIL,			// "eMail"
	VRSCID_OPEN,			// "Open"
	VRSCID_SAVE,			// "Save"

	// Other
	VRSCID_ERROR_OPEN,		// "Could not open file!"
	VRSCID_ERROR_WRITE,		// "Error writing file!"

	VRSCID_USAGE,			// "Usage" (caption of the help-dialog)
	VRSCID_MOUSE,			// "Mouse"
	VRSCID_USAGE_MOUSE,		// <all mouse usage text, lines separated by "\n" (carriage return or linefeed)>
	VRSCID_KEYBOARD,		// "Keyboard"
	VRSCID_USAGE_KEYBOARD,	// <all keyboard usage text, lines separated by "\n" (carriage return or linefeed)>
};


// GUI Themes
// $ +n Type GUITheme (VGUI_THEME_)
// $ +av Type GUITheme
enum
{
	VGUI_THEME_OFFICE2000,
	VGUI_THEME_OFFICE2003,
	VGUI_THEME_WHIDBEY,
};


// Scale Modes
// $ +n Type ScaleMode (VSCALE_MODE_)
// $ +av Type ScaleMode
enum
{
	VSCALE_MODE_FULL_PAGE,		// show full page
	VSCALE_MODE_PAGE_WIDTH,		// fit page width
	VSCALE_MODE_ZOOM_TOOL,		// zoom tool activated
	VSCALE_MODE_FREE,			// any scale allowed
};


// Engine Render Modes
// $ +n Type EngineRenderMode (VENGINE_RENDER_MODE_)
// $ +av Type EngineRenderMode
enum
{
	VENGINE_RENDER_MODE_VER3 = 0,	// VPE Renders Text compatible to Version 3.xx
	VENGINE_RENDER_MODE_VER4 = 1,	// VPE Renders Text compatible to Version 4.00 and higher
};


// Scale Modes
// $ +n Type UnitFactor (VUNIT_FACTOR_)
// $ +av Type UnitFactor
enum
{
// $ +n subst Centimeter
	VUNIT_FACTOR_CM		= 100000,	// coordinates are handled in cm units by the VPE API
	VUNIT_FACTOR_INCH	= 254000,	// coordinates are handled in inch units by the VPE API
	VUNIT_FACTOR_MM10	= 1000,		// coordinates are handled in 1/10 of a mm by the VPE API (compatible to VPE < v4.0)
};


#endif      // _MODULE_VPECOMON
